/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.ant;

import static org.jtiger.ant.FixtureResultsHandlerFactory.newFixtureResultsHandler;
import static org.jtiger.ant.BuildExceptionMessages.missingJavaElement;
import static org.jtiger.ant.BuildExceptionMessages.missingFixturesElement;
import static org.jtiger.ant.BuildExceptionMessages.missingFixturesClasses;
import static org.jtiger.ant.BuildExceptionMessages.missingClassnameAttributeFixtureElement;
import static org.jtiger.ant.BuildExceptionMessages.missingNameAttributeResultElement;
import static org.jtiger.ant.BuildExceptionMessages.missingRegexAttributeCategoryElement;
import static org.jtiger.ant.BuildExceptionMessages.badResultName;

import static org.jtiger.ant.LogMessages.usingDefinitionClass;
import static org.jtiger.ant.LogMessages.usingSetUpTearDownClass;
import static org.jtiger.ant.LogMessages.usingJUnit;
import static org.jtiger.ant.LogMessages.haltOnFailure;
import static org.jtiger.ant.LogMessages.totalNumberOfCategories;
import static org.jtiger.ant.LogMessages.usingDefaultResultName;
import static org.jtiger.ant.LogMessages.usingResultName;
import static org.jtiger.ant.LogMessages.usingCategoryRegularExpression;
import static org.jtiger.ant.LogMessages.usingResultParams;
import static org.jtiger.ant.LogMessages.usingFixtureClasses;

import static org.jtiger.framework.FixturesRunnerMain.ARG_FIXTURE_CLASSES;
import static org.jtiger.framework.FixturesRunnerMain.ARG_DEFINITION_CLASS;
import static org.jtiger.framework.FixturesRunnerMain.ARG_SUTD_CLASS;
import static org.jtiger.framework.FixturesRunnerMain.ARG_JUNIT;
import static org.jtiger.framework.FixturesRunnerMain.ARG_HALT_ON_FAILURE;
import static org.jtiger.framework.FixturesRunnerMain.ARG_CATEGORIES;
import static org.jtiger.framework.FixturesRunnerMain.ARG_RESULT;
import static org.jtiger.framework.FixturesRunnerMain.ARG_RESULT_PARAMETERS;

import static org.apache.tools.ant.Project.MSG_VERBOSE;

import java.util.LinkedList;
import java.util.List;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.jtiger.framework.FixturesRunnerMain;

/**
 * An <a href="http://ant.apache.org//manual/using.html#tasks" target="_blank">Apache Ant task</a> for performing a test execution
 * run from an Ant build.
 * <br/>
 * An example excerpt of an Ant build file that uses this task:
 * <br/>
<pre>
 &lt;target name=&quot;test&quot; description=&quot;Execute JTiger tests&quot;&gt;
     &lt;mkdir dir=&quot;test-report&quot;/&gt;
     &lt;taskdef name=&quot;jtiger&quot; classname=&quot;org.jtiger.ant.JTigerTask&quot; classpathref=&quot;project.class.path&quot;/&gt;

     &lt;jtiger haltonfailure=&quot;true&quot;&gt;
         &lt;category regex=&quot;DatabaseTest&quot;/&gt;
         &lt;fixtures&gt;
             &lt;fixture classname=&quot;com.foo.AdditionalTestFixture&quot;/&gt;
             &lt;fileset dir=&quot;test-src&quot;&gt;
                 &lt;include name=&quot;&#42&#42/&#42.java&quot;/&gt;
             &lt;/fileset&gt;
         &lt;/fixtures&gt;
         &lt;result name=&quot;~html&quot;&gt;
             &lt;param value=&quot;test-report&quot;/&gt;
         &lt;/result&gt;
         &lt;java&gt;
             &lt;classpath refid=&quot;project.class.path&quot;/&gt;
         &lt;/java&gt;
     &lt;/jtiger&gt;
 &lt;/target&gt;
</pre>
 * @see Category
 * @see Fixture
 * @see Fixtures
 * @see Java
 * @see Result
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class JTigerTask extends Task
{
    private Fixtures fixtures;
    private String definitionClass;
    private String sutdClass;
    private final List<Category> categories;
    private boolean haltOnFailure;
    private boolean jUnit;
    private Result result;
    private Java j;

    /**
     * Create a default <tt>JTigerTask</tt>.
     */
    public JTigerTask()
    {
        categories = new LinkedList<Category>();
    }

    /**
     * Creates a {@link Fixtures}.
     *
     * @return A new {@link Fixtures}.
     */
    public Fixtures createFixtures()
    {
        final Fixtures fixtures = new Fixtures();
        this.fixtures = fixtures;
        return fixtures;
    }

    /**
     * Sets the test definition class name for this task.
     * @see org.jtiger.framework.FixturesRunnerConfig#getDefinitionClass()
     * @see org.jtiger.framework.TestDefinition
     *
     * @param definitionClass The test definition class name for this task.
     */
    public void setDefinitionClass(final String definitionClass)
    {
        this.definitionClass = definitionClass;
    }

    /**
     * Sets the set up/tear down class name for this task.
     * @see org.jtiger.framework.FixturesRunnerConfig#getSutdClass()
     * @see org.jtiger.framework.SetUpTearDown
     *
     * @param sutdClass The set up/tear down class name for this task.
     */
    public void setSutdClass(final String sutdClass)
    {
        this.sutdClass = sutdClass;
    }

    /**
     * Creates a {@link Category}.
     *
     * @return A new {@link Category}.
     */
    public Category createCategory()
    {
        final Category c = new Category();
        categories.add(c);
        return c;
    }

    /**
     * Sets the halt on failure attribute for this task.
     * @see org.jtiger.framework.FixturesRunnerConfig#isHaltOnFailure()
     *
     * @param haltOnFailure The halt on failure attribute for this task.
     */
    public void setHaltOnFailure(final boolean haltOnFailure)
    {
        this.haltOnFailure = haltOnFailure;
    }

    /**
     * Sets the junit attribute for this task.
     * Setting this value to <code>true</code> overrides any values set for {@link #setDefinitionClass(String)} and
     * {@link #setSutdClass(String)}.
     * Set this attribute to execute <a href="http://www.junit.org/" target="_blank">JUnit</a> unit test cases.
     *
     * @param jUnit The junit attribute for this task.
     */
    public void setJUnit(final boolean jUnit)
    {
        this.jUnit = jUnit;
    }

    /**
     * Creates a {@link Result}.
     *
     * @return A new {@link Result}.
     */
    public Result createResult()
    {
        final Result r = new Result();
        result = r;
        return r;
    }

    /**
     * Creates a {@link Java}.
     *
     * @return A new {@link Java}.
     */
    public Java createJava()
    {
        final Java j = new Java(new org.apache.tools.ant.taskdefs.Java());

        j.setProject(super.getProject());
        j.setClassname(FixturesRunnerMain.class.getName());
        j.setFork(true);
        j.setOwningTarget(super.getOwningTarget());
        j.setLocation(super.getLocation());
        j.setTaskName(super.getTaskName());

        this.j = j;
        return j;
    }

    /**
     * Executes the Ant task by calling {@link Java#execute()} and passing the set arguments to
     * {@link FixturesRunnerMain#main(String[])}. These parameters are set by specifying attributes and elements in the
     * Ant build file. 
     *
     * @throws BuildException
     * <li>If the &lt;java&gt; element is not present in the Ant build file.</li>
     * <li>If the &ltfixtures&gt; element is not present in the Ant build file.</li>
     * <li>If the &ltfixtures&gt; element does not contain any classes from &lt;fixture&gt; or &lt;fileset&gt;
     * subelements in the Ant build file.</li>
     * <li>If a &ltfixture&gt; element does not contain a <code>classname</code> attribute in the Ant build file.</li>
     * <li>If a &ltcategory&gt; element does not contain a <code>regex</code> attribute in the Ant build file.</li>
     * <li>If a &ltresult&gt; element does not contain a <code>type</code> attribute in the Ant build file.</li>
     * <li>If a &ltresult&gt; element does not contain a <code>destination</code> attribute in the Ant build file.</li>
     * <li>If a &ltresult&gt; element contains a <code>destination</code> attribute that is an invalid value in the Ant
     * build file.</li>
     */
    public void execute() throws BuildException
    {
        if(j == null)
        {
            throw new BuildException(missingJavaElement());
        }

        if(fixtures == null)
        {
            throw new BuildException(missingFixturesElement());
        }

        final String[] fixtureClasses = fixtures.toStringArray(super.getProject());

        if(fixtureClasses.length == 0)
        {
            throw new BuildException(missingFixturesClasses());
        }

        super.log(usingFixtureClasses(fixtureClasses), MSG_VERBOSE);

        for(Fixture f : fixtures.getFixtures())
        {
            if(f.getClassname() == null || f.getClassname().length() == 0)
            {
                throw new BuildException(missingClassnameAttributeFixtureElement());
            }
        }

        if(fixtures != null)
        {
            j.createArg().setValue(ARG_FIXTURE_CLASSES);

            for(String fixtureClass : fixtureClasses)
            {
                j.createArg().setValue(fixtureClass);
            }
        }

        if(definitionClass != null)
        {
            super.log(usingDefinitionClass(definitionClass), MSG_VERBOSE);

            j.createArg().setValue(ARG_DEFINITION_CLASS);
            j.createArg().setValue(definitionClass);
        }

        if(sutdClass != null)
        {
            super.log(usingSetUpTearDownClass(sutdClass), MSG_VERBOSE);

            j.createArg().setValue(ARG_SUTD_CLASS);
            j.createArg().setValue(sutdClass);
        }

        if(jUnit)
        {
            super.log(usingJUnit(), MSG_VERBOSE);

            j.createArg().setValue(ARG_JUNIT);
        }

        if(haltOnFailure)
        {
            j.createArg().setValue(ARG_HALT_ON_FAILURE);
        }

        super.log(haltOnFailure(haltOnFailure), MSG_VERBOSE);

        if(categories != null && categories.size() > 0)
        {
            j.createArg().setValue(ARG_CATEGORIES);

            for(Category category : categories)
            {
                final String regex = category.getRegex();

                if(regex == null)
                {
                    throw new BuildException(missingRegexAttributeCategoryElement());
                }

                super.log(usingCategoryRegularExpression(regex), MSG_VERBOSE);

                j.createArg().setValue(regex);
            }
        }

        super.log(totalNumberOfCategories(categories.size()), MSG_VERBOSE);

        if(result == null)
        {
            super.log(usingDefaultResultName(), MSG_VERBOSE);
        }
        else
        {
            final String name = result.getName();

            if(name == null)
            {
                throw new BuildException(missingNameAttributeResultElement());
            }

            super.log(usingResultName(name), MSG_VERBOSE);

            final List<Param> params = result.getParams();

            j.createArg().setValue(ARG_RESULT);

            try
            {
                final Class<?> c = newFixtureResultsHandler(name);

                if(c == null)
                {
                    throw new BuildException(badResultName(name));
                }

                j.createArg().setValue(c.getName());
            }
            catch(ClassNotFoundException e)
            {
                throw new BuildException(e.getMessage(), e);
            }

            super.log(usingResultParams(params), MSG_VERBOSE);

            j.createArg().setValue(ARG_RESULT_PARAMETERS);

            for(Param p : params)
            {
                final String value = p.getValue();

                if(value != null && value.length() > 0)
                {
                    j.createArg().setValue(value);
                }
            }
        }

        j.execute();
    }
}
