/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.ant;

import java.io.File;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;
import org.apache.tools.ant.types.Assertions;
import org.apache.tools.ant.types.Commandline;
import org.apache.tools.ant.types.Environment;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Permissions;
import org.apache.tools.ant.types.PropertySet;
import org.apache.tools.ant.types.Reference;

/**
 * An element used in {@link JTigerTask the Ant task} to specify the environment for the Java Virtual Machine that
 * executes the test run. The JVM will be a forked process from the JVM that the Ant task is running in. This element is
 * the same as <a href="http://ant.apache.org//manual/CoreTasks/java.html" target="_blank">the core Ant &lt;java&gt; task</a> except
 * that some of the elements and attributes are not present on this element.
 *
 * @see JTigerTask
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class Java
{
    private final org.apache.tools.ant.taskdefs.Java java;

    /**
     * Create a <tt>Java</tt> that wraps the given Apache Ant Java task.
     *
     * @param java The Apache Ant Java task to wrap.
     */
    public Java(final org.apache.tools.ant.taskdefs.Java java)
    {
        this.java = java;
    }

    /**
     * Control enablement of
     * <a href="http://ant.apache.org//manual/CoreTypes/assertions.html" target="_blank">Java 1.4 assertions</a> with an
     * &lt;assertions&gt; subelement.
     *
     * @param assertions The assertion set.
     */
    public void addAssertions(final Assertions assertions)
    {
        java.addAssertions(assertions);
    }

    /**
     * Adds an environment variable.
     *
     * @param env The new environment variable.
     */
    public void addEnv(final Environment.Variable env)
    {
        java.addEnv(env);
    }

    /**
     * Adds a system property.
     *
     * @param sysproperty The new system property.
     */
    public void addSysproperty(final Environment.Variable sysproperty)
    {
        java.addSysproperty(sysproperty);
    }

    /**
     * Adds a <a href="http://ant.apache.org//manual/CoreTypes/propertyset.html" target="_blank">system property set</a>.
     *
     * @param syspropertyset The new system property set.
     */
    public void addSyspropertyset(final PropertySet syspropertyset)
    {
        java.addSyspropertyset(syspropertyset);
    }

    /**
     * Creates <a href="http://ant.apache.org//manual/using.html#path" target="_blank">a boot classpath</a>.
     *
     * @return A new boot classpath.
     */
    public Path createBootclasspath()
    {
        return java.createBootclasspath();
    }

    /**
     * Creates <a href="http://ant.apache.org//manual/using.html#path" target="_blank">a classpath</a>.
     *
     * @return A new classpath.
     */
    public Path createClasspath()
    {
        return java.createClasspath();
    }

    /**
     * Creates <a href="http://ant.apache.org//manual/using.html#arg" target="_blank">a JVM argument</a>.
     *
     * @return A new JVM argument.
     */
    public Commandline.Argument createJvmarg()
    {
        return java.createJvmarg();
    }

    /**
     * Creates <a href="http://ant.apache.org//manual/CoreTypes/permissions.html" target="_blank">a set of permissions</a>.
     *
     * @return A new set of permissions.
     */
    public Permissions createPermissions()
    {
        return java.createPermissions();
    }

    /**
     * Sets whether output and error files should be appended to (<code>true</code>)or overwritten (<code>false</code>).
     *
     * @param append The value of whether output and error files should be appended to (<code>true</code>)or overwritten
     * (<code>false</code>).
     */
    public void setAppend(final boolean append)
    {
        java.setAppend(append);
    }

    /**
     * Sets <a href="http://ant.apache.org//manual/using.html#path" target="_blank">a classpath</a>.
     *
     * @param classpath A new <a href="http://ant.apache.org//manual/using.html#path" target="_blank">classpath</a>.
     */
    public void setClasspath(final Path classpath)
    {
        java.setClasspath(classpath);
    }

    /**
     * Sets <a href="http://ant.apache.org//manual/using.html#references" target="_blank">a classpath reference</a>.
     *
     * @param classpathRef A new <a href="http://ant.apache.org//manual/using.html#references" target="_blank">classpath reference</a>.
     */
    public void setClasspathRef(final Reference classpathRef)
    {
        java.setClasspathRef(classpathRef);
    }

    /**
     * Sets the directory to invoke the VM in.
     *
     * @param dir The directory to invoke the VM in.
     */
    public void setDir(final File dir)
    {
        java.setDir(dir);
    }

    /**
     * Sets the file to which the standard error of the command should be redirected.
     *
     * @param error The file to which the standard error of the command should be redirected.
     */
    public void setError(final File error)
    {
        java.setError(error);
    }

    /**
     * Sets the name of a property in which the standard error of the command should be stored.
     *
     * @param errorProperty The name of a property in which the standard error of the command should be stored.
     */
    public void setErrorProperty(final String errorProperty)
    {
        java.setErrorProperty(errorProperty);
    }

    /**
     * Sets whether to stop the build process if the command exits with a returncode other than 0.
     *
     * @param failonerror The value of whether to stop the build process if the command exits with a returncode other
     * than 0.
     */
    public void setFailonerror(final boolean failonerror)
    {
        java.setFailonerror(failonerror);
    }

    /**
     * Sets the command used to invoke the Java Virtual Machine, default is '<code>java</code>'. The command is resolved
     * by <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Runtime.html#exec">java.lang.Runtime.exec()</a>.
     *
     * @param jvm The command used to invoke the Java Virtual Machine, default is '<code>java</code>'. The command is
     * resolved by <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Runtime.html#exec">java.lang.Runtime.exec()</a>.
     */
    public void setJvm(final String jvm)
    {
        java.setJvm(jvm);
    }

    /**
     * This attribute is used when you wish to see error output in Ant's log and you are redirecting output to a
     * file/property. The error output will not be included in the output file/property. If you redirect error with the
     * "error" or "errorProperty" attributes, this will have no effect.
     *
     * @param logError This attribute is used when you wish to see error output in Ant's log and you are redirecting
     * output to a file/property. The error output will not be included in the output file/property. If you redirect
     * error with the "error" or "errorProperty" attributes, this will have no effect.
     */
    public void setLogError(final boolean logError)
    {
        java.setLogError(logError);
    }

    /**
     * Max amount of memory to allocate to the forked VM.
     *
     * @param maxmemory The maximum amount of memory to allocate to the forked VM.
     */
    public void setMaxmemory(final String maxmemory)
    {
        java.setMaxmemory(maxmemory);
    }

    /**
     * Sets whether to propagate old environment when new environment variables are specified.
     *
     * @param newenvironment The value of whether to propagate old environment when new environment variables are
     * specified.
     */
    public void setNewenvironment(final boolean newenvironment)
    {
        java.setNewenvironment(newenvironment);
    }

    /**
     * Sets the name of a file to which to write the output. If the error stream is not also redirected to a file or
     * property, it will appear in this output.
     *
     * @param output The name of a file to which to write the output. If the error stream is not also redirected to a
     * file or property, it will appear in this output.
     */
    public void setOutput(final File output)
    {
        java.setOutput(output);
    }

    /**
     * Sets the name of a property in which the output of the command should be stored. Unless the error stream is
     * redirected to a separate file or stream, this property will include the error output.
     *
     * @param outputproperty The name of a property in which the output of the command should be stored. Unless the
     * error stream is redirected to a separate file or stream, this property will include the error output.
     */
    public void setOutputproperty(final String outputproperty)
    {
        java.setOutputproperty(outputproperty);
    }

    /**
     * Sets the name of a property in which the return code of the command should be stored. Only of interest if
     * {@link #setFailonerror(boolean) <code>failonerror=false</code>}.
     *
     * @param resultProperty The name of a property in which the return code of the command should be stored. Only of
     * interest if {@link #setFailonerror(boolean) <code>failonerror=false</code>}.
     */
    public void setResultProperty(final String resultProperty)
    {
        java.setResultProperty(resultProperty);
    }

    /**
     * Sets the ability to start a process which will outlive ant.
     *
     * @param spawn The ability to start a process which will outlive ant.
     */
    public void setSpawn(final boolean spawn)
    {
        java.setSpawn(spawn);
    }

    /**
     * Stop the command if it doesn't finish within the specified time (given in milliseconds).
     *
     * @param timeout The specified time (given in milliseconds).
     */
    public void setTimeout(final Long timeout)
    {
        java.setTimeout(timeout);
    }

    /**
     * Forks the JVM using
     * <a href="http://ant.apache.org//manual/CoreTasks/java.html" target="_blank">the core Ant &lt;java&gt; task</a>.
     *
     * @throws BuildException IF a problem occurs during execution of the forked JVM.
     */
    public void execute() throws BuildException
    {
        java.execute();
    }

    void setDescription(final String description)
    {
        java.setDescription(description);
    }

    Commandline.Argument createArg()
    {
        return java.createArg();
    }

    void setClassname(final String classname)
    {
        java.setClassname(classname);
    }

    void setFork(final boolean fork)
    {
        java.setFork(fork);
    }

    void setLocation(final Location location)
    {
        java.setLocation(location);
    }

    void setOwningTarget(final Target target)
    {
        java.setOwningTarget(target);
    }

    void setTaskName(final String taskName)
    {
        java.setTaskName(taskName);
    }

    void setProject(final Project project)
    {
        java.setProject(project);
    }

    org.apache.tools.ant.taskdefs.Java getJava()
    {
        return java;
    }
}
