/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

import static org.jtiger.assertion.BoxedTypePromoterFactory.newBoxedTypePromoter;
import java.util.Arrays;

/**
 * Provides the ability to make assertions that are of the most basic and trivial concepts.
 *
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class Basic
{
    private Basic()
    {
        /*throw new UnsupportedOperationException(
            "class Basic is not instantiable");*/
    }

    /**
     * Asserts that the given <code>boolean</code> argument is <code>true</code>.
     *
     * @param b The argument to assert is <code>true</code>.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>boolean</code> argument is not <code>true</code>.
     */
    public static void assertTrue(final boolean b, final Object... message) throws AssertionException
    {
        if(!b)
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>boolean</code> argument is <code>true</code>.
     *
     * @param b The argument to assert is <code>true</code>.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>boolean</code> argument is not <code>true</code>.
     */
    public static void assertFalse(final boolean b, final Object... message) throws AssertionException
    {
        if(b)
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>Object</code> argument is <code>null</code>.
     *
     * @param o The argument to assert is <code>null</code>.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>Object</code> argument is not <code>null</code>.
     */
    public static void assertNull(final Object o, final Object... message) throws AssertionException
    {
        if(o != null)
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>Object</code> argument is not <code>null</code>.
     *
     * @param o The argument to assert is not <code>null</code>.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>Object</code> argument is <code>null</code>.
     */
    public static void assertNotNull(final Object o, final Object... message) throws AssertionException
    {
        if(o == null)
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>Object</code> argument is equal to the actual <code>Object</code> argument.
     * Equality of two references is determined by both referring to <code>null</code> or being equal according to the
     * semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#equals(java.lang.Object)">the java.lang.Object equals method</a>.
     * <br/>
     * <code>expected == null && actual == null || (expected != null && expected.equals(actual))</code><br/>
     * If either the <code>expected</code> or <code>actual</code> argument is of type <tt>Byte</tt>, <tt>Short</tt>,
     * <tt>Character</tt>, or <tt>Integer</tt>, it is explicitly promoted to type <tt>Long</tt>. If either argument is
     * of type <tt>Float</tt>, it is promoted to type <tt>Double</tt>. This is so that the boxed type will be equal if
     * it has the same value, even though it is of a different type. For example, <code>assertEqual(7, 7L)</code> will
     * autobox each argument to type <tt>Integer</tt> and <tt>Long</tt> respectively, an since
     * <code>new Integer(7).equals(new Long(7L))</code> will be <code>false</code>, the assertion would fail, therefore,
     * the first argument is promoted to type <tt>Long</tt> first, so that the test for equality will succeed.
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>Object</code> argument is not equal to the actual
     * <code>Object</code> argument.
     */
    public static void assertEqual(final Object expected, final Object actual, final Object... message) throws AssertionException
    {
        final BoxedTypePromoter btp = newBoxedTypePromoter();

        final Object promotedExpected = btp.promote(expected);
        final Object promotedActual = btp.promote(actual);

        if(!(promotedExpected == null && promotedActual == null || (promotedExpected != null && promotedExpected.equals(promotedActual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>Object</code> array argument is equal to the actual <code>Object</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(java.lang.Object[], java.lang.Object[])">java.util.Arrays equals(Object[], Object[]) method</a>
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>Object</code> array argument is not equal to the actual
     * <code>Object</code> array argument.
     */
    public static void assertEqual(final Object[] expected, final Object[] actual, final Object... message) throws AssertionException
    {
        if(!(expected == null && actual == null || (Arrays.equals(expected, actual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>double</code> array argument is equal to the actual <code>double</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(double[], double[])">the java.util.Arrays equals(double[], double[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>double</code> array argument is not equal to the actual
     * <code>double</code> array argument.
     */
    public static void assertEqual(final double[] expected, final double[] actual, final Object... message) throws AssertionException
    {
        if(!(expected == null && actual == null || (Arrays.equals(expected, actual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>float</code> array argument is equal to the actual <code>float</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(float[], float[])">the java.util.Arrays equals(float[], float[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>float</code> array argument is not equal to the actual
     * <code>float</code> array argument.
     */
    public static void assertEqual(final float[] expected, final float[] actual, final Object... message) throws AssertionException
    {
        if(!(expected == null && actual == null || (Arrays.equals(expected, actual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>boolean</code> array argument is equal to the actual <code>boolean</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(boolean[], boolean[])">the java.util.Arrays equals(boolean[], boolean[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>boolean</code> array argument is not equal to the actual
     * <code>boolean</code> array argument.
     */
    public static void assertEqual(final boolean[] expected, final boolean[] actual, final Object... message) throws AssertionException
    {
        if(!(expected == null && actual == null || (Arrays.equals(expected, actual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>byte</code> array argument is equal to the actual <code>byte</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(byte[], byte[])">the java.util.Arrays equals(byte[], byte[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>byte</code> array argument is not equal to the actual
     * <code>byte</code> array argument.
     */
    public static void assertEqual(final byte[] expected, final byte[] actual, final Object... message) throws AssertionException
    {
        if(!(expected == null && actual == null || (Arrays.equals(expected, actual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>short</code> array argument is equal to the actual <code>short</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(short[], short[])">the java.util.Arrays equals(short[], short[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>short</code> array argument is not equal to the actual
     * <code>short</code> array argument.
     */
    public static void assertEqual(final short[] expected, final short[] actual, final Object... message) throws AssertionException
    {
        if(!(expected == null && actual == null || (Arrays.equals(expected, actual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>char</code> array argument is equal to the actual <code>char</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(char[], char[])">the java.util.Arrays equals(char[], char[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>char</code> array argument is not equal to the actual
     * <code>char</code> array argument.
     */
    public static void assertEqual(final char[] expected, final char[] actual, final Object... message) throws AssertionException
    {
        if(!(expected == null && actual == null || (Arrays.equals(expected, actual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>int</code> array argument is equal to the actual <code>int</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(int[], int[])">the java.util.Arrays equals(int[], int[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>int</code> array argument is not equal to the actual
     * <code>int</code> array argument.
     */
    public static void assertEqual(final int[] expected, final int[] actual, final Object... message) throws AssertionException
    {
        if(!(expected == null && actual == null || (Arrays.equals(expected, actual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>long</code> array argument is equal to the actual <code>long</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(long[], long[])">the java.util.Arrays equals(long[], long[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>long</code> array argument is not equal to the actual
     * <code>long</code> array argument.
     */
    public static void assertEqual(final long[] expected, final long[] actual, final Object... message) throws AssertionException
    {
        if(!(expected == null && actual == null || (Arrays.equals(expected, actual))))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>Object</code> argument is not equal to the actual
     * <code>Object</code> argument.
     * Equality of two references is determined by both referring to <code>null</code> or being equal according to the
     * semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#equals(java.lang.Object)">the java.lang.Object equals method</a>.
     * <br/>
     * <code>!(expected == null && actual == null || (expected != null && expected.equals(actual)))</code>
     * If either the <code>expected</code> or <code>actual</code> argument is of type <tt>Byte</tt>, <tt>Short</tt>,
     * <tt>Character</tt>, or <tt>Integer</tt>, it is explicitly promoted to type <tt>Long</tt>. If either argument is
     * of type <tt>Float</tt>, it is promoted to type <tt>Double</tt>. This is so that the boxed type will be equal if
     * it has the same value, even though it is of a different type. For example, <code>assertEqual(7, 7L)</code> will
     * autobox each argument to type <tt>Integer</tt> and <tt>Long</tt> respectively, an since
     * <code>new Integer(7).equals(new Long(7L))</code> will be <code>false</code>, the assertion would fail, therefore,
     * the first argument is promoted to type <tt>Long</tt> first, so that the test for equality will succeed.
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>Object</code> argument is equal to the actual
     * <code>Object</code> argument.
     */
    public static void assertNotEqual(final Object expected, final Object actual, final Object... message) throws AssertionException
    {
        final BoxedTypePromoter btp = newBoxedTypePromoter();

        final Object promotedExpected = btp.promote(expected);
        final Object promotedActual = btp.promote(actual);

        if(promotedExpected == null && promotedActual == null || (promotedExpected != null && promotedExpected.equals(promotedActual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>Object</code> array argument is not equal to the actual <code>Object</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(java.lang.Object[], java.lang.Object[])">the java.util.Arrays equals(Object[], Object[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>Object</code> array argument is equal to the actual
     * <code>Object</code> array argument.
     */
    public static void assertNotEqual(final Object[] expected, final Object[] actual, final Object... message) throws AssertionException
    {
        if(expected == null && actual == null || (Arrays.equals(expected, actual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>double</code> array argument is not equal to the actual <code>double</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(double[], double[])">the java.util.Arrays equals(double[], double[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>double</code> array argument is equal to the actual
     * <code>double</code> array argument.
     */
    public static void assertNotEqual(final double[] expected, final double[] actual, final Object... message) throws AssertionException
    {
        if(expected == null && actual == null || (Arrays.equals(expected, actual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>float</code> array argument is not equal to the actual <code>float</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(float[], float[])">the java.util.Arrays equals(float[], float[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>float</code> array argument is equal to the actual
     * <code>float</code> array argument.
     */
    public static void assertNotEqual(final float[] expected, final float[] actual, final Object... message) throws AssertionException
    {
        if(expected == null && actual == null || (Arrays.equals(expected, actual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>boolean</code> array argument is not equal to the actual <code>boolean</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(boolean[], boolean[])">the java.util.Arrays equals(boolean[], boolean[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>boolean</code> array argument is equal to the actual
     * <code>boolean</code> array argument.
     */
    public static void assertNotEqual(final boolean[] expected, final boolean[] actual, final Object... message) throws AssertionException
    {
        if(expected == null && actual == null || (Arrays.equals(expected, actual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>byte</code> array argument is not equal to the actual <code>byte</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(byte[], byte[])">the java.util.Arrays equals(byte[], byte[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>byte</code> array argument is equal to the actual
     * <code>byte</code> array argument.
     */
    public static void assertNotEqual(final byte[] expected, final byte[] actual, final Object... message) throws AssertionException
    {
        if(expected == null && actual == null || (Arrays.equals(expected, actual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>short</code> array argument is not equal to the actual <code>short</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(short[], short[])">the java.util.Arrays equals(short[], short[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>short</code> array argument is equal to the actual
     * <code>short</code> array argument.
     */
    public static void assertNotEqual(final short[] expected, final short[] actual, final Object... message) throws AssertionException
    {
        if(expected == null && actual == null || (Arrays.equals(expected, actual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>char</code> array argument is not equal to the actual <code>char</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(char[], char[])">the java.util.Arrays equals(char[], char[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>char</code> array argument is equal to the actual
     * <code>char</code> array argument.
     */
    public static void assertNotEqual(final char[] expected, final char[] actual, final Object... message) throws AssertionException
    {
        if(expected == null && actual == null || (Arrays.equals(expected, actual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>int</code> array argument is not equal to the actual <code>int</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(int[], int[])">the java.util.Arrays equals(int[], int[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>int</code> array argument is equal to the actual
     * <code>int</code> array argument.
     */
    public static void assertNotEqual(final int[] expected, final int[] actual, final Object... message) throws AssertionException
    {
        if(expected == null && actual == null || (Arrays.equals(expected, actual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>long</code> array argument is not equal to the actual <code>long</code>
     * array argument.
     * Equality of two arrays references is determined by both referring to <code>null</code> or being equal according
     * to the semantics of
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/util/Arrays.html#equals(long[], long[])">the java.util.Arrays equals(long[], long[]) method</a>.
     * <br/>
     * <code>expected == null && actual == null || (Arrays.equals(expected, actual))</code>
     *
     * @param expected The expected value to test for inequality.
     * @param actual The actual value to test for inequality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>long</code> array argument is equal to the actual
     * <code>long</code> array argument.
     */
    public static void assertNotEqual(final long[] expected, final long[] actual, final Object... message) throws AssertionException
    {
        if(expected == null && actual == null || (Arrays.equals(expected, actual)))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>Object</code> argument is referring to the same object as the actual
     * <code>Object</code> argument.
     * <br/>
     * <code>expected == actual</code>
     *
     * @param expected The expected value to test for sameness.
     * @param actual The actual value to test for sameness.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>Object</code> argument is not referring to the same object
     * as the actual <code>Object</code> argument.
     */
    public static void assertSame(final Object expected, final Object actual, final Object... message) throws AssertionException
    {
        if(expected != actual)
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given expected <code>Object</code> argument is not referring to the same object as the actual
     * <code>Object</code> argument.
     * <br/>
     * <code>expected != actual</code>
     *
     * @param expected The expected value to test for equality.
     * @param actual The actual value to test for equality.
     * @param message The assertion message.
     * @throws AssertionException If the given expected <code>Object</code> argument is referring to the same object as
     * the actual <code>Object</code> argument.
     */
    public static void assertNotSame(final Object expected, final Object actual, final Object... message) throws AssertionException
    {
        if(expected == actual)
        {
            throw new AssertionException(message);
        }
    }
}
