/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

import static org.jtiger.assertion.EqualsMethodContractTesterFactory.newEqualsMethodContractTester;

/**
 * Makes assertions on the general contract of the
 * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.htmll#equals(java.lang.Object)">java.lang.Object equals method</a>.
 * This requires the creation of an instance of {@link ObjectFactory} to return instances to use to test.
 * The given {@link ObjectFactory} must meet the its own general contract.
 *
 * @see ObjectFactory
 * @see ObjectFactoryContract
 * @see HashCodeMethodContract
 * @see Object#equals(Object) 
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class EqualsMethodContract
{
    private EqualsMethodContract()
    {

    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet one aspect of the equals method
     * general contract.
     * The <code>equals</code> method must return <code>false</code> for a <code>null</code> argument.
     *
     * @param factory The factory to use to return instances for asserting the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the aspect
     * of the equals method general contract.
     */
    public static void assertEqualsMethodUnequalToNull(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);

        if(!tester.isUnequalToNull())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet one aspect of the equals method
     * general contract.
     * The <code>equals</code> method must consistently return <code>true</code> or <code>false</code> given the same
     * argument when called on the same instance.
     *
     * @param factory The factory to use to return instances for asserting the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the aspect
     * of the equals method general contract.
     */
    public static void assertEqualsMethodConsistentAcrossInvocations(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);

        if(!tester.isConsistentAcrossInvocations())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet one aspect of the equals method
     * general contract.
     * The <code>equals</code> method must return <code>true</code> when given an argument that refers to the same instance
     * on which the method is called.
     *
     * @param factory The factory to use to return instances for asserting the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the aspect
     * of the equals method general contract.
     */
    public static void assertEqualsMethodReflexive(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);

        if(!tester.isReflexive())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet one aspect of the equals method
     * general contract.
     * The <code>equals</code> method must be symmetric.
     *
     * @param factory The factory to use to return instances for asserting the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the aspect
     * of the equals method general contract.
     */
    public static void assertEqualsMethodSymmetric(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);

        if(!tester.isSymmetric())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet one aspect of the equals method
     * general contract.
     * The <code>equals</code> method must be transitive.
     *
     * @param factory The factory to use to return instances for asserting the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the aspect
     * of the equals method general contract.
     */
    public static void assertEqualsMethodTransitive(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);

        if(!tester.isTransitive())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet one aspect of the equals method
     * general contract.
     * For an instance that has defined unequal semantics, the <code>equals</code> method must be consistently return
     * <code>false</code>.
     *
     * @param factory The factory to use to return instances for asserting the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the aspect
     * of the equals method general contract.
     */
    public static void assertEqualsMethodUnequalToUnequalInstance(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);

        if(!tester.isUnequalToUnequalInstance())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet the entire equals method general
     * contract.
     * The contract is defined by
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.htmll#equals(java.lang.Object)">the java.lang.Object equals method</a>.
     *
     * @param factory The factory to use to return instances for asserting the entire general contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the entire
     * equals method general contract.
     */
    public static void assertEqualsMethodFillsContract(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);

        if(!tester.fillsContract())
        {
            throw new AssertionException(message);
        }
    }
}
