/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

import static org.jtiger.assertion.HashCodeMethodContractTesterFactory.newHashCodeMethodContractTester;

/**
 * Makes assertions on the general contract of the
 * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.htmll#hashCode()">java.lang.Object hashCode method</a>.
 * This requires the creation of an instance of {@link ObjectFactory} to return instances to use to test.
 * The given {@link ObjectFactory} must meet the its own general contract.
 *
 * @see ObjectFactory
 * @see ObjectFactoryContract
 * @see EqualsMethodContract
 * @see Object#hashCode()
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class HashCodeMethodContract
{
    private HashCodeMethodContract()
    {

    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet one aspect of the hashCode method
     * general contract.
     * The <code>hashCode</code> method must consistently return the same value over multiple invocations.
     *
     * @param factory The factory to use to return instances for asserting the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the aspect
     * of the hashCode method general contract.
     */
    public static void assertHashCodeMethodConsistentResult(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final HashCodeMethodContractTester tester = newHashCodeMethodContractTester(factory);

        if(!tester.isConsistentResult())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet one aspect of the hashCode method
     * general contract.
     * The <code>hashCode</code> method must return the same value for instances that are equal
     * according to their equals method implementation.
     *
     * @see Object#equals(Object) 
     * @param factory The factory to use to return instances for asserting the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the aspect
     * of the hashCode method general contract.
     */
    public static void assertHashCodeMethodEqualOnEqualInstance(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final HashCodeMethodContractTester tester = newHashCodeMethodContractTester(factory);

        if(!tester.isEqualOnEqualInstance())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> returns instances that meet the entire hashCode method general
     * contract.
     * The contract is defined by
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.htmll#hashCode()">the java.lang.Object hashCode method</a>.
     *
     * @param factory The factory to use to return instances for asserting the entire general contract.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>ObjectFactory</code> returns instances that do not meet the entire
     * hashCode method general contract.
     */
    public static void assertHashCodeMethodFillsContract(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final HashCodeMethodContractTester tester = newHashCodeMethodContractTester(factory);

        if(!tester.fillsContract())
        {
            throw new AssertionException(message);
        }
    }
}
