/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import static org.jtiger.assertion.ModifierTesterFactory.newModifierTester;

/**
 * Provides the ability make assertions that modifiers appear or do not appear on various language constructs, such as
 * types (i.e. classes, interfaces, enums and annotations), constructors, methods and fields.
 *
 * @see java.lang.reflect.Modifier
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class Modifier
{
    private Modifier()
    {

    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>abstract</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface methods are implicitly abstract.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert has the
     * <code>abstract</code> modifier (implicitly or explicitly).
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not
     * have the <code>abstract</code> modifier (implicitly or explicitly).
     */
    public static void assertAbstract(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isAbstract(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not have the
     * <code>abstract</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface methods are implicitly abstract.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert does not have the
     * <code>abstract</code> modifier (implicitly or explicitly).
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>abstract</code> modifier (implicitly or explicitly).
     */
    public static void assertNotAbstract(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isAbstract(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>abstract</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interfaces are implicitly abstract.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert has the
     * <code>abstract</code> modifier (implicitly or explicitly).
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>abstract</code> modifier (implicitly or explicitly).
     */
    public static void assertAbstract(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isAbstract(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>abstract</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interfaces are implicitly abstract.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert does not have the
     * <code>abstract</code> modifier (implicitly or explicitly).
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>abstract</code> modifier (implicitly or explicitly).
     */
    public static void assertNotAbstract(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isAbstract(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>final</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert has the
     * <code>final</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not
     * have the <code>final</code> modifier.
     */
    public static void assertFinal(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isFinal(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not have the
     * <code>final</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert does not have the
     * <code>final</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>final</code> modifier.
     */
    public static void assertNotFinal(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isFinal(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>final</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that inner classes of interfaces, and enums are
     * implicitly final.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert has the
     * <code>final</code> modifier (implicitly or explicitly).
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>final</code> modifier (implicitly or explicitly).
     */
    public static void assertFinal(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isFinal(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>final</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that inner classes of interfaces, and enums are
     * implicitly final.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert does not have the
     * <code>final</code> modifier (implicitly or explicitly).
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>final</code> modifier (implicitly or explicitly).
     */
    public static void assertNotFinal(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isFinal(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>final</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface fields are implicitly final.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert has the
     * <code>final</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not
     * have the <code>final</code> modifier (implicitly or explicitly).
     */
    public static void assertFinal(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isFinal(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have the
     * <code>final</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface fields are implicitly final.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert does not have the
     * <code>final</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>final</code> modifier (implicitly or explicitly).
     */
    public static void assertNotFinal(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isFinal(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>native</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert has the
     * <code>native</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not
     * have the <code>native</code> modifier.
     */
    public static void assertNative(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isNative(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not have the
     * <code>native</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert does not have the
     * <code>native</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>native</code> modifier.
     */
    public static void assertNotNative(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isNative(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>private</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert has the
     * <code>private</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not
     * have the <code>private</code> modifier.
     */
    public static void assertPrivate(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isPrivate(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not have the
     * <code>private</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert does not have the
     * <code>private</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>private</code> modifier.
     */
    public static void assertNotPrivate(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isPrivate(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> has the
     * <code>private</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that enum constructors are implicitly private.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> to assert has the
     * <code>private</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a>
     * does not have the <code>private</code> modifier (implicitly or explicitly).
     */
    public static void assertPrivate(final Constructor<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isPrivate(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> does not have
     * the <code>private</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that enum constructors are implicitly private.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> to assert does not have
     * the <code>private</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a>
     * has the <code>private</code> modifier (implicitly or explicitly).
     */
    public static void assertNotPrivate(final Constructor<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isPrivate(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>private</code> modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert has the
     * <code>private</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>private</code> modifier.
     */
    public static void assertPrivate(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isPrivate(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>private</code> modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert does not have the
     * <code>private</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>private</code> modifier.
     */
    public static void assertNotPrivate(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isPrivate(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>private</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface fields are implicitly private.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert has the
     * <code>private</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not
     * have the <code>private</code> modifier (implicitly or explicitly).
     */
    public static void assertPrivate(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isPrivate(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have the
     * <code>private</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface fields are implicitly private.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert does not have the
     * <code>private</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>private</code> modifier (implicitly or explicitly).
     */
    public static void assertNotPrivate(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isPrivate(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>protected</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert has the
     * <code>protected</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not
     * have the <code>protected</code> modifier.
     */
    public static void assertProtected(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isProtected(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not have the
     * <code>protected</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert does not have the
     * <code>protected</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>protected</code> modifier.
     */
    public static void assertNotProtected(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isProtected(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> has the
     * <code>protected</code> modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> to assert has the
     * <code>protected</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a>
     * does not have the <code>protected</code> modifier.
     */
    public static void assertProtected(final Constructor<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isProtected(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> does not have
     * the <code>protected</code> modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> to assert does not have
     * the <code>protected</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a>
     * has the <code>protected</code> modifier.
     */
    public static void assertNotProtected(final Constructor<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isProtected(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>protected</code> modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert has the
     * <code>protected</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>protected</code> modifier.
     */
    public static void assertProtected(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isProtected(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>protected</code> modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert does not have the
     * <code>protected</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>protected</code> modifier.
     */
    public static void assertNotProtected(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isProtected(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>protected</code> modifier.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert has the
     * <code>protected</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not
     * have the <code>protected</code> modifier.
     */
    public static void assertProtected(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isProtected(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have the
     * <code>protected</code> modifier.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert does not have the
     * <code>protected</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>protected</code> modifier.
     */
    public static void assertNotProtected(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isProtected(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>public</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface methods are implicitly public.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert has the
     * <code>public</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not
     * have the <code>public</code> modifier (implicitly or explicitly).
     */
    public static void assertPublic(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isPublic(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not have the
     * <code>public</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface methods are implicitly public.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert does not have the
     * <code>public</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>public</code> modifier (implicitly or explicitly).
     */
    public static void assertNotPublic(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isPublic(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> has the
     * <code>public</code> modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> to assert has the
     * <code>public</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a>
     * does not have the <code>public</code> modifier.
     */
    public static void assertPublic(final Constructor<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isPublic(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> does not have
     * the <code>public</code> modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> to assert does not have
     * the <code>public</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a>
     * has the <code>public</code> modifier.
     */
    public static void assertNotPublic(final Constructor<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isPublic(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>public</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that inner classes of interfaces are implicitly public.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert has the
     * <code>public</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>public</code> modifier (implicitly or explicitly).
     */
    public static void assertPublic(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isPublic(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>public</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that inner classes of interfaces are implicitly public.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert does not have the
     * <code>public</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>public</code> modifier (implicitly or explicitly).
     */
    public static void assertNotPublic(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isPublic(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>public</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface fields are implicitly public.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert has the
     * <code>public</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not
     * have the <code>public</code> modifier (implicitly or explicitly).
     */
    public static void assertPublic(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isPublic(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have the
     * <code>public</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface fields are implicitly public.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert does not have the
     * <code>public</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>public</code> modifier (implicitly or explicitly).
     */
    public static void assertNotPublic(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isPublic(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the default
     * (none specified) access modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert has the default
     * (none specified) access modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has an
     * explicit access modifier (<code>private</code>, <code>protected</code>, or <code>public</code>).
     */
    public static void assertDefaultAccess(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isDefaultAccess(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not have the
     * default (none specified) access modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert does not have the
     * default (none specified) access modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not
     * have an explicit access modifier (<code>private</code>, <code>protected</code>, or <code>public</code>).
     */
    public static void assertNotDefaultAccess(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isDefaultAccess(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> has the
     * default (none specified) access modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> to assert has
     * the default (none specified) access modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a>
     * has an explicit access modifier (<code>private</code>, <code>protected</code>, or <code>public</code>).
     */
    public static void assertDefaultAccess(final Constructor<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isDefaultAccess(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> does not have
     * the default (none specified) access modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a> to assert does not have
     * the default (none specified) access modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Constructor.html">Constructor</a>
     * does not have an explicit access modifier (<code>private</code>, <code>protected</code>, or <code>public</code>).
     */
    public static void assertNotDefaultAccess(final Constructor<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isDefaultAccess(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the default
     * (none specified) access modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert has the default
     * (none specified) access modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has an
     * explicit access modifier (<code>private</code>, <code>protected</code>, or <code>public</code>).
     */
    public static void assertDefaultAccess(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isDefaultAccess(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the default
     * (none specified) access modifier.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert does not have the default
     * (none specified) access modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have an
     * explicit access modifier (<code>private</code>, <code>protected</code>, or <code>public</code>).
     */
    public static void assertNotDefaultAccess(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isDefaultAccess(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the default
     * (none specified) access modifier.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert has the default
     * (none specified) access modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has an
     * explicit access modifier (<code>private</code>, <code>protected</code>, or <code>public</code>).
     */
    public static void assertDefaultAccess(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isDefaultAccess(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have the default
     * (none specified) access modifier.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert does not have the default
     * (none specified) access modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have
     * an explicit access modifier (<code>private</code>, <code>protected</code>, or <code>public</code>).
     */
    public static void assertNotDefaultAccess(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isDefaultAccess(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>static</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert has the
     * <code>static</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not
     * have the <code>static</code> modifier.
     */
    public static void assertStatic(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isStatic(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not have the
     * <code>static</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert does not have the
     * <code>static</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>static</code> modifier.
     */
    public static void assertNotStatic(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isStatic(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>static</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that inner classes of interfaces are implicitly static.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert has the
     * <code>static</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>static</code> modifier (implicitly or explicitly).
     */
    public static void assertStatic(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isStatic(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>static</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that inner classes of interfaces are implicitly static.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert does not have the
     * <code>static</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>static</code> modifier (implicitly or explicitly).
     */
    public static void assertNotStatic(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isStatic(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>static</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface fields are implicitly static.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert has the
     * <code>static</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have
     * the <code>static</code> modifier (implicitly or explicitly).
     */
    public static void assertStatic(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isStatic(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have the
     * <code>static</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that interface fields are implicitly static.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert does not have the
     * <code>static</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>static</code> modifier (implicitly or explicitly).
     */
    public static void assertNotStatic(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isStatic(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>strictfp</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that methods and all nested types of a strictfp class are implicitly
     * strictfp.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert has the
     * <code>strictfp</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>strictfp</code> modifier (implicitly or explicitly).
     */
    public static void assertStrictfp(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isStrictfp(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> does not have the
     * <code>strictfp</code> modifier (implicitly or explicitly).
     * The Java Language Specification defines that methods and all nested types of a strictfp class are implicitly
     * strictfp.
     *
     * @param c The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> to assert does not have the
     * <code>strictfp</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">Class</a> has the
     * <code>strictfp</code> modifier (implicitly or explicitly).
     */
    public static void assertNotStrictfp(final Class<?> c, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isStrictfp(c))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>synchronized</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert has the
     * <code>synchronized</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not
     * have the <code>synchronized</code> modifier.
     */
    public static void assertSynchronized(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isSynchronized(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> does not have the
     * <code>synchronized</code> modifier.
     *
     * @param m The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> to assert does not have the
     * <code>synchronized</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Method.html">Method</a> has the
     * <code>synchronized</code> modifier.
     */
    public static void assertNotSynchronized(final Method m, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isSynchronized(m))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>transient</code> modifier.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert has the
     * <code>transient</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have
     * the <code>transient</code> modifier.
     */
    public static void assertTransient(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isTransient(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have the
     * <code>transient</code> modifier.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert does not have the
     * <code>transient</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>transient</code> modifier.
     */
    public static void assertNotTransient(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isTransient(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>volatile</code> modifier.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert has the
     * <code>volatile</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have
     * the <code>volatile</code> modifier.
     */
    public static void assertVolatile(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(!tester.isVolatile(f))
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> does not have the
     * <code>volatile</code> modifier.
     *
     * @param f The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> to assert does not have the
     * <code>volatile</code> modifier.
     * @param message The assertion message.
     * @throws AssertionException If the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/reflect/Field.html">Field</a> has the
     * <code>volatile</code> modifier.
     */
    public static void assertNotVolatile(final Field f, final Object... message) throws AssertionException
    {
        final ModifierTester tester = newModifierTester();

        if(tester.isVolatile(f))
        {
            throw new AssertionException(message);
        }
    }
}
