/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

/**
 * A callback interface for the creation of objects that are typically under test by the framework.
 * This interface defines a contract, which <b>must</b> be met.
 * This contract can be asserted by using {@link ObjectFactoryContract the ObjectFactory contract assertions}.
 * Although this contract mandates five (5) aspects, it is trivial to meet for a typical class.
 * The contract is:
 * <br/>
 * <li>
 * The method {@link #newInstanceX()} must consistently return new instances.
 * <code>(factory.newInstanceX() == factory.newInstanceX())</code><br/>
 * must always evaluate to <code>false</code>.
 * </li>
 *
 * <li>
 * The method {@link #newInstanceY()} must consistently return new instances.
 * <code>(factory.newInstanceY() == factory.newInstanceY())</code><br/>
 * must always evaluate to <code>false</code>.
 * </li>
 *
 * <li>
 * The method {@link #newInstanceX()} must consistently return instances that are equal to each other.<br/>
 * <code>(factory.newInstanceX().equals(factory.newInstanceX())</code><br/>
 * must always evaluate to <code>true</code>.
 * </li>
 *
 * <li>
 * The method {@link #newInstanceY()} must consistently return instances that are equal to each other.<br/>
 * <code>(factory.newInstanceY().equals(factory.newInstanceY())</code><br/>
 * must always evaluate to <code>true</code>.
 * </li>
 *
 * <li>
 * The method {@link #newInstanceX()} must consistently return instances that are <b>not</b> equal to
 * instances that are returned by {@link #newInstanceY()}.<br/>
 * <code>(factory.newInstanceX().equals(factory.newInstanceY())</code><br/>
 * must always evaluate to <code>false</code>.
 * </li>
 *
 * @see ObjectFactoryContract
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public interface ObjectFactory<T>
{
    /**
     * Returns a new instance of a class, that is always equal on successive calls and never equal to objects returned
     * by {@link #newInstanceY()}.
     *
     * @return A new instance of a class, that is always equal on successive calls and never equal to objects returned
     * by {@link #newInstanceY()}.
     */
    T newInstanceX();

    /**
     * Returns a new instance of a class, that is always equal on successive calls and never equal to objects returned
     * by {@link #newInstanceX()}.
     *
     * @return A new instance of a class, that is always equal on successive calls and never equal to objects returned
     * by {@link #newInstanceX()}.
     */
    T newInstanceY();
}
