/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

import static org.jtiger.assertion.ObjectFactoryContractTesterFactory.newObjectFactoryContractTester;

/**
 * Provides the ability to make assertions on the contract defined by {@link ObjectFactory}.
 * The contract has five (5) aspects, which are specified by {@link ObjectFactory}.
 *
 * @see ObjectFactory
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class ObjectFactoryContract
{
    private ObjectFactoryContract()
    {

    }

    /**
     * Asserts that the given <code>ObjectFactory</code> meets one aspect of the entire contract.
     * The {@link ObjectFactory#newInstanceX()} method must consistently return new instances.
     *
     * @param factory The object factory to assert has met the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the {@link ObjectFactory#newInstanceX()} method does not consistently return new
     * instances.
     */
    public static void assertObjectFactoryXConsistentlyNewInstance(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final ObjectFactoryContractTester tester = newObjectFactoryContractTester(factory);

        if(!tester.isXConsistentlyNewInstance())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> meets one aspect of the entire contract.
     * The {@link ObjectFactory#newInstanceY()} method must consistently return new instances.
     *
     * @param factory The object factory to assert has met the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the {@link ObjectFactory#newInstanceY()} method does not consistently return new
     * instances.
     */
    public static void assertObjectFactoryYConsistentlyNewInstance(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final ObjectFactoryContractTester tester = newObjectFactoryContractTester(factory);

        if(!tester.isYConsistentlyNewInstance())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> meets one aspect of the entire contract.
     * The {@link ObjectFactory#newInstanceX()} method must consistently return equal instances.
     *
     * @param factory The object factory to assert has met the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the {@link ObjectFactory#newInstanceX()} method does not consistently return equal
     * instances.
     */
    public static void assertObjectFactoryXConsistentlyEqual(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final ObjectFactoryContractTester tester = newObjectFactoryContractTester(factory);

        if(!tester.isXConsistentlyEqual())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> meets one aspect of the entire contract.
     * The {@link ObjectFactory#newInstanceY()} method must consistently return equal instances.
     *
     * @param factory The object factory to assert has met the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the {@link ObjectFactory#newInstanceY()} method does not consistently return equal
     * instances.
     */
    public static void assertObjectFactoryYConsistentlyEqual(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final ObjectFactoryContractTester tester = newObjectFactoryContractTester(factory);

        if(!tester.isYConsistentlyEqual())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> meets one aspect of the entire contract.
     * The {@link ObjectFactory#newInstanceX()} method must consistently return instances that are unequal to instances
     * returned by the {@link ObjectFactory#newInstanceY()} method.
     *
     * @param factory The object factory to assert has met the aspect of the contract.
     * @param message The assertion message.
     * @throws AssertionException If the {@link ObjectFactory#newInstanceY()} method does not consistently return
     * instances that are unequal to instances returned by the {@link ObjectFactory#newInstanceY()} method.
     */
    public static void assertObjectFactoryXYConsistentlyUnequal(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final ObjectFactoryContractTester tester = newObjectFactoryContractTester(factory);

        if(!tester.isXYConsistentlyUnequal())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>ObjectFactory</code> meets the entire contract.
     *
     * @see ObjectFactory
     * @param factory The object factory to assert has met the contract.
     * @param message The assertion message.
     * @throws AssertionException If the object factory does not meet the entire contract.
     */
    public static void assertObjectFactoryFillsContract(final ObjectFactory<?> factory, final Object... message) throws AssertionException
    {
        final ObjectFactoryContractTester tester = newObjectFactoryContractTester(factory);

        if(!tester.fillsContract())
        {
            throw new AssertionException(message);
        }
    }
}
