/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * Provides the ability to make assertions on Java reflection types.
 *
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class Reflection
{
    private Reflection()
    {

    }

    /**
     * Asserts that the given class has a constructor with the given parameter types.
     *
     * @see Class#getConstructor(Class[])
     * @param c The class to assert has the given constructor.
     * @param parameterTypes The parameter types of the constructor.
     * @param message The assertion message.
     * @return The constructor that the given class has.
     * @throws AssertionException If the given class does not have a constructor with the given parameter types.
     */
    public static Constructor<?> assertHasConstructor(final Class<?> c, final Class<?>[] parameterTypes, final Object... message) throws AssertionException
    {
        try
        {
            return c.getConstructor(parameterTypes);
        }
        catch(NoSuchMethodException e)
        {
            throw new AssertionException(e, message);
        }
    }

    /**
     * Asserts that the given class does not have a constructor with the given parameter types.
     *
     * @see Class#getConstructor(Class[])
     * @param c The class to assert does not have the given constructor.
     * @param parameterTypes The parameter types of the constructor.
     * @param message The assertion message.
     * @throws AssertionException If the given class has a constructor with the given parameter types.
     */
    public static void assertNotHasConstructor(final Class<?> c, final Class<?>[] parameterTypes, final Object... message) throws AssertionException
    {
        try
        {
            c.getConstructor(parameterTypes);
            throw new AssertionException(message);
        }
        catch(NoSuchMethodException e)
        {
            // test passed
        }
    }

    /**
     * Asserts that the given class has a declared constructor with the given parameter types.
     *
     * @see Class#getDeclaredConstructor(Class[])
     * @param c The class to assert has the given declared constructor.
     * @param parameterTypes The parameter types of the declared constructor.
     * @param message The assertion message.
     * @return The declared constructor that the given class has.
     * @throws AssertionException If the given class does not have a declared constructor with the given parameter
     * types.
     */
    public static Constructor<?> assertHasDeclaredConstructor(final Class<?> c, final Class<?>[] parameterTypes, final Object... message) throws AssertionException
    {
        try
        {
            return c.getDeclaredConstructor(parameterTypes);
        }
        catch(NoSuchMethodException e)
        {
            throw new AssertionException(e, message);
        }
    }

    /**
     * Asserts that the given class does not have a declared constructor with the given parameter types.
     *
     * @see Class#getDeclaredConstructor(Class[])
     * @param c The class to assert does not have the given declared constructor.
     * @param parameterTypes The parameter types of the declared constructor.
     * @param message The assertion message.
     * @throws AssertionException If the given class has a declared constructor with the given parameter
     * types.
     */
    public static void assertNotHasDeclaredConstructor(final Class<?> c, final Class<?>[] parameterTypes, final Object... message) throws AssertionException
    {
        try
        {
            c.getDeclaredConstructor(parameterTypes);
            throw new AssertionException(message);
        }
        catch(NoSuchMethodException e)
        {
            // test passed
        }
    }

    /**
     * Asserts that the given class has a method with the given name and parameter types.
     *
     * @see Class#getMethod(String, Class[])
     * @param c The class to assert has the given method.
     * @param name The name of the method.
     * @param parameterTypes The parameter types of the method.
     * @param message The assertion message.
     * @return The method that the given class has.
     * @throws AssertionException If the given class does not have a method with the given name and parameter types.
     */
    public static Method assertHasMethod(final Class<?> c, final String name, final Class<?>[] parameterTypes, final Object... message) throws AssertionException
    {
        try
        {
            return c.getMethod(name, parameterTypes);
        }
        catch(NoSuchMethodException e)
        {
            throw new AssertionException(e, message);
        }
    }

    /**
     * Asserts that the given class does not have a method with the given name and parameter types.
     *
     * @see Class#getMethod(String, Class[])
     * @param c The class to assert does not have the given method.
     * @param name The name of the method.
     * @param parameterTypes The parameter types of the method.
     * @param message The assertion message.
     * @throws AssertionException If the given class has a method with the given name and parameter types.
     */
    public static void assertNotHasMethod(final Class<?> c, final String name, final Class<?>[] parameterTypes, final Object... message) throws AssertionException
    {
        try
        {
            c.getMethod(name, parameterTypes);
            throw new AssertionException(message);
        }
        catch(NoSuchMethodException e)
        {
            // test passed
        }
    }

    /**
     * Asserts that the given class has a declared method with the given name and parameter types.
     *
     * @see Class#getDeclaredMethod(String, Class[])
     * @param c The class to assert has the given method.
     * @param name The name of the method.
     * @param parameterTypes The parameter types of the method.
     * @param message The assertion message.
     * @return The method that the given class has.
     * @throws AssertionException If the given class does not have a declared method with the given name and parameter
     * types.
     */
    public static Method assertHasDeclaredMethod(final Class<?> c, final String name, final Class<?>[] parameterTypes, final Object... message) throws AssertionException
    {
        try
        {
            return c.getDeclaredMethod(name, parameterTypes);
        }
        catch(NoSuchMethodException e)
        {
            throw new AssertionException(e, message);
        }
    }

    /**
     * Asserts that the given class does not have a declared method with the given name and parameter types.
     *
     * @see Class#getDeclaredMethod(String, Class[])
     * @param c The class to assert does not have the given method.
     * @param name The name of the method.
     * @param parameterTypes The parameter types of the method.
     * @param message The assertion message.
     * @throws AssertionException If the given class has a declared method with the given name and parameter types.
     */
    public static void assertNotHasDeclaredMethod(final Class<?> c, final String name, final Class<?>[] parameterTypes, final Object... message) throws AssertionException
    {
        try
        {
            c.getDeclaredMethod(name, parameterTypes);
            throw new AssertionException(message);
        }
        catch(NoSuchMethodException e)
        {
            // test passed
        }
    }

    /**
     * Asserts that the given class has a field with the given name.
     *
     * @see Class#getField(String)
     * @param c The class to assert has the given field.
     * @param name The name of the field.
     * @param message The assertion message.
     * @return The field that the given class has.
     * @throws AssertionException If the given class does not have a field with the given name.
     */
    public static Field assertHasField(final Class<?> c, final String name, final Object... message) throws AssertionException
    {
        try
        {
            return c.getField(name);
        }
        catch(NoSuchFieldException e)
        {
            throw new AssertionException(e, message);
        }
    }

    /**
     * Asserts that the given class does not have a field with the given name.
     *
     * @see Class#getField(String)
     * @param c The class to assert does not have the given field.
     * @param name The name of the field.
     * @param message The assertion message.
     * @throws AssertionException If the given class has a field with the given name.
     */
    public static void assertNotHasField(final Class<?> c, final String name, final Object... message) throws AssertionException
    {
        try
        {
            c.getField(name);
            throw new AssertionException(message);
        }
        catch(NoSuchFieldException e)
        {
            // test passed
        }
    }

    /**
     * Asserts that the given class has a declared field with the given name.
     *
     * @see Class#getDeclaredField(String)
     * @param c The class to assert has the given field.
     * @param name The name of the field.
     * @param message The assertion message.
     * @return The field that the given class has.
     * @throws AssertionException If the given class does not have a declared field with the given name.
     */
    public static Field assertHasDeclaredField(final Class<?> c, final String name, final Object... message) throws AssertionException
    {
        try
        {
            return c.getDeclaredField(name);
        }
        catch(NoSuchFieldException e)
        {
            throw new AssertionException(e, message);
        }
    }

    /**
     * Asserts that the given class does not have a declared field with the given name.
     *
     * @see Class#getDeclaredField(String)  
     * @param c The class to assert does not have the given field.
     * @param name The name of the field.
     * @param message The assertion message.
     * @throws AssertionException If the given class has a declared field with the given name.
     */
    public static void assertNotHasDeclaredField(final Class<?> c, final String name, final Object... message) throws AssertionException
    {
        try
        {
            c.getDeclaredField(name);
            throw new AssertionException(message);
        }
        catch(NoSuchFieldException e)
        {
            // test passed
        }
    }
}
