/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

import java.io.Serializable;
import static org.jtiger.assertion.SerializationTesterFactory.newSerializationTester;

/**
 * Provides the ability to make assertions regarding the serializability of objects.
 *
 * @see Serializable
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class Serialization
{
    private Serialization()
    {

    }

    /**
     * Asserts that the given <code>Serializable</code> instance successfully (without exceptions) serializes and
     * deserializes.
     *
     * @param s The <code>Serializable</code> instance to make the assertion on.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>Serializable</code> instance does not successfully
     * (without exceptions) serialize and deserialize.
     */
    public static void assertSerializes(final Serializable s, final Object... message) throws AssertionException
    {
        final SerializationTester tester = newSerializationTester(s);

        if(!tester.serializes())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>Serializable</code> instance does not successfully (without exceptions) serialize
     * and deserialize.
     *
     * @param s The <code>Serializable</code> instance to make the assertion on.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>Serializable</code> instance successfully (without exceptions)
     * serializes and deserializes.
     */
    public static void assertNotSerializes(final Serializable s, final Object... message) throws AssertionException
    {
        final SerializationTester tester = newSerializationTester(s);

        if(tester.serializes())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>Serializable</code> instance serializes and deserializes to produce the same
     * instance. This can only be achieved by overriding the <code>readResolve</code> method.
     *
     * @param s The <code>Serializable</code> instance to make the assertion on.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>Serializable</code> instance does not serialize and deserialize to
     * produce the same instance.
     */
    public static void assertSerializesSame(final Serializable s, final Object... message) throws AssertionException
    {
        final SerializationTester tester = newSerializationTester(s);

        if(!tester.serializesSame())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>Serializable</code> instance serializes and deserializes to produce a new
     * instance.
     *
     * @param s The <code>Serializable</code> instance to make the assertion on.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>Serializable</code> instance does not serialize and deserialize to
     * produce a new instance.
     */
    public static void assertSerializesUnsame(final Serializable s, final Object... message) throws AssertionException
    {
        final SerializationTester tester = newSerializationTester(s);

        if(tester.serializesSame())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>Serializable</code> instance serializes and deserializes to produce an instance that
     * is equal according to its equals method implementation.
     *
     * @see Object#equals(Object)
     * @param s The <code>Serializable</code> instance to make the assertion on.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>Serializable</code> instance does not serialize and deserialize to
     * produce an instance that is equal according to its equals method implementation.
     */
    public static void assertSerializesEqual(final Serializable s, final Object... message) throws AssertionException
    {
        final SerializationTester tester = newSerializationTester(s);

        if(!tester.serializesEqual())
        {
            throw new AssertionException(message);
        }
    }

    /**
     * Asserts that the given <code>Serializable</code> instance serializes and deserializes to produce an instance that
     * is unequal according to its equals method implementation.
     *
     * @see Object#equals(Object)
     * @param s The <code>Serializable</code> instance to make the assertion on.
     * @param message The assertion message.
     * @throws AssertionException If the given <code>Serializable</code> instance does not serialize and deserialize to
     * produce an instance that is unequal according to its equals method implementation.
     */
    public static void assertSerializesUnequal(final Serializable s, final Object... message) throws AssertionException
    {
        final SerializationTester tester = newSerializationTester(s);

        if(tester.serializesEqual())
        {
            throw new AssertionException(message);
        }
    }
}
