/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.framework.ClassAliasMappingFactory.newClassAliasMapping;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import static org.jtiger.framework.FixturesRunnerConfigFactory.newFixturesRunnerConfig;
import static org.jtiger.framework.SequenceFactory.newSequence;
import static org.jtiger.framework.ArgsFactory.newArgs;
import org.jtiger.framework.junit.JUnitTestDefinition;
import org.jtiger.framework.junit.JUnitSetUpTearDown;

final class ArgsConverterFactory
{
    private ArgsConverterFactory()
    {

    }

    static ArgsConverter newArgsToFixtureRunnerParams()
    {
        return new ArgsConverterImpl();
    }

    enum ArgReadState
    {
        NONE,
        FIXTURE_CLASSES,
        DEFINITION_CLASS,
        SUTD_CLASS,
        CATEGORIES,
        RESULT_HANDLER_CLASS,
        RESULT_HANDLER_PARAMETERS;
    }

    private static final class ArgsConverterImpl implements ArgsConverter
    {
        ArgsConverterImpl()
        {

        }

        public Args convert(final String... args) throws ClassNotFoundException, NullPointerException
        {
            if(args == null)
            {
                throw new NullPointerException();
            }

            if(args.length == 0)
            {
                return null;
            }

            final List<String> fixtureClassNames = new LinkedList<String>();
            String definitionClassName = null;
            String sutdClassName = null;
            boolean haltOnFailure = false;
            final List<String> categories = new LinkedList<String>();
            String resultHandlerClassName = null;
            final List<String> resultHandlerParameters = new LinkedList<String>();

            ArgReadState state = ArgReadState.NONE;

            for(String arg : args)
            {
                if(arg.equalsIgnoreCase(FixturesRunnerMain.ARG_FIXTURE_CLASSES))
                {
                    state = ArgReadState.FIXTURE_CLASSES;
                }
                else if(arg.equalsIgnoreCase(FixturesRunnerMain.ARG_DEFINITION_CLASS))
                {
                    state = ArgReadState.DEFINITION_CLASS;
                }
                else if(arg.equalsIgnoreCase(FixturesRunnerMain.ARG_SUTD_CLASS))
                {
                    state = ArgReadState.SUTD_CLASS;
                }
                else if(arg.equalsIgnoreCase(FixturesRunnerMain.ARG_HALT_ON_FAILURE))
                {
                    haltOnFailure = true;
                    state = ArgReadState.NONE;
                }
                else if(arg.equalsIgnoreCase(FixturesRunnerMain.ARG_JUNIT))
                {
                    definitionClassName = JUnitTestDefinition.class.getName();
                    sutdClassName = JUnitSetUpTearDown.class.getName();
                    state = ArgReadState.NONE;
                }
                else if(arg.equalsIgnoreCase(FixturesRunnerMain.ARG_CATEGORIES))
                {
                    state = ArgReadState.CATEGORIES;
                }
                else if(arg.equalsIgnoreCase(FixturesRunnerMain.ARG_RESULT))
                {
                    state = ArgReadState.RESULT_HANDLER_CLASS;
                }
                else if(arg.equalsIgnoreCase(FixturesRunnerMain.ARG_RESULT_PARAMETERS))
                {
                    state = ArgReadState.RESULT_HANDLER_PARAMETERS;
                }
                else
                {
                    if(state == ArgReadState.NONE)
                    {
                        return null;
                    }
                    else if(state == ArgReadState.FIXTURE_CLASSES)
                    {
                        fixtureClassNames.add(arg);
                    }
                    else if(state == ArgReadState.DEFINITION_CLASS && definitionClassName == null)
                    {
                        definitionClassName = arg;
                        state = ArgReadState.NONE;
                    }
                    else if(state == ArgReadState.SUTD_CLASS && sutdClassName == null)
                    {
                        sutdClassName = arg;
                        state = ArgReadState.NONE;
                    }
                    else if(state == ArgReadState.CATEGORIES)
                    {
                        categories.add(arg);
                    }
                    else if(state == ArgReadState.RESULT_HANDLER_CLASS && resultHandlerClassName == null)
                    {
                        resultHandlerClassName = arg;

                        final String aliasedName = newClassAliasMapping().getMappedClassName(resultHandlerClassName);

                        if(aliasedName != null)
                        {
                            resultHandlerClassName = aliasedName;
                        }

                        state = ArgReadState.NONE;
                    }
                    else if(state == ArgReadState.RESULT_HANDLER_PARAMETERS)
                    {
                        resultHandlerParameters.add(arg);
                    }
                }
            }

            final List<Class<?>> fixtureClasses = new ArrayList<Class<?>>(fixtureClassNames.size());
            Class<? extends TestDefinition> definitionClass = null;
            Class<? extends SetUpTearDown> sutdClass = null;
            Class<? extends FixtureResultsHandler> resultHandlerClass = null;

            for(String fixtureClassName : fixtureClassNames)
            {
                fixtureClasses.add(Class.forName(fixtureClassName));
            }

            if(definitionClassName != null)
            {
                final Class<?> c = Class.forName(definitionClassName);

                if(TestDefinition.class.isAssignableFrom(c))
                {
                    definitionClass = c.asSubclass(TestDefinition.class);
                }
            }

            if(sutdClassName != null)
            {
                final Class<?> c = Class.forName(sutdClassName);

                if(SetUpTearDown.class.isAssignableFrom(c))
                {
                    sutdClass = c.asSubclass(SetUpTearDown.class);
                }
            }

            if(resultHandlerClassName != null)
            {
                final Class<?> c = Class.forName(resultHandlerClassName);

                if(FixtureResultsHandler.class.isAssignableFrom(c))
                {
                    resultHandlerClass = c.asSubclass(FixtureResultsHandler.class);
                }
            }

            return newArgs(newFixturesRunnerConfig(
                    newSequence(fixtureClasses.toArray(new Class<?>[fixtureClasses.size()])),
                    definitionClass,
                    sutdClass,
                    haltOnFailure,
                    categories.toArray(new String[categories.size()])),
                    resultHandlerClass,
                    newSequence(resultHandlerParameters.toArray(new String[resultHandlerParameters.size()])));
        }
    }
}
