/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.framework.SequenceFactory.newSequence;
import java.util.Properties;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.io.InputStream;
import java.io.IOException;

/**
 * Provides an alias name to a class name using {@link #CLASS_ALIAS_MAPPING_RESOURCE a resource} as the mapping. This
 * resource forms part of the JTiger core.
 *
 * @see ClassAliasMapping
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class ClassAliasMappingFactory
{
    /**
     * The value that aliases are prefixed with before being obtained by calling
     * {@link ClassAliasMapping#getMappedClassName(String)}. That is, if an alias exists in the resource as "alias", the value
     * that is mapped to that alias can be obtained by calling
     * <code>{@link ClassAliasMapping#getMappedClassName(String) theClassAliasMapping.getMappedClassName(ALIAS_PREFIX + "alias")}</code>
     */
    public static final String ALIAS_PREFIX = "~";

    /**
     * The name of the resource that provides the class name mapping. This resource forms part of the JTiger core.
     */
    public static final String CLASS_ALIAS_MAPPING_RESOURCE = "/org/jtiger/classaliases.properties";

    private ClassAliasMappingFactory()
    {

    }

    /**
     * Returns a new <code>ClassAliasMapping</code> instance.
     *
     * @return A new <code>ClassAliasMapping</code> instance.
     */
    public static ClassAliasMapping newClassAliasMapping()
    {
        return new ClassAliasMappingImpl();
    }

    private static final class ClassAliasMappingImpl implements ClassAliasMapping
    {
        private static final Map<String, String> m;
        private static boolean initialised;

        static
        {
            m = new HashMap<String, String>();
        }

        ClassAliasMappingImpl()
        {

        }

        public String getMappedClassName(final String alias) throws NullPointerException
        {
            if(alias == null)
            {
                throw new NullPointerException();
            }

            if(!initialised)
            {
                initialiseMapping();
                initialised = true;
            }

            return ClassAliasMappingImpl.m.get(alias.toLowerCase());
        }

        public ReadOnlyArray<String> getAliases()
        {
            if(!initialised)
            {
                initialiseMapping();
                initialised = true;
            }

            final Set<String> keys = ClassAliasMappingImpl.m.keySet();
            return newSequence(keys.toArray(new String[keys.size()]));
        }

        private static void initialiseMapping()
        {
            final InputStream in = ClassAliasMappingImpl.class.getResourceAsStream(CLASS_ALIAS_MAPPING_RESOURCE);
            final Properties props = new Properties();

            try
            {
                props.load(in);

                final Set<Object> names = props.keySet();

                for(Object o : names)
                {
                    final String name = (String)o;
                    final String aliasedName = new StringBuilder(ALIAS_PREFIX).append(name.toLowerCase()).toString();
                    ClassAliasMappingImpl.m.put(aliasedName, props.getProperty(name));
                }
            }
            catch(IOException e)
            {
                throw new IllegalStateException(e.getMessage(), e);
            }
        }
    }
}
