/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * The default implementation of setting up and tearing down a unit test case.
 * This implementation searches for the {@link SetUp} annotation present on a method and calls it to set up a test case.
 * This implementation searches for the {@link TearDown} annotation present on a method and calls it to tear down a test
 * case. If more than one method contains either of these annotations, it cannot be guaranteed which order they are
 * executed in.
 *
 * @see SetUpTearDown
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class DefaultSetUpTearDownFactory
{
    private DefaultSetUpTearDownFactory()
    {

    }

    /**
     * Returns a new instance of the default implementation to handle setting up and tearing down of unit test cases.
     *
     * @return A new instance of the default implementation to handle setting up and tearing down of unit test cases.
     */
    public static SetUpTearDown newDefaultSetUpTearDown()
    {
        return new DefaultSetUpTearDownImpl();
    }

    private static final class DefaultSetUpTearDownImpl implements SetUpTearDown
    {
        DefaultSetUpTearDownImpl()
        {

        }

        public void setUp(final Object fixture) throws SetUpException
        {
            final Class<?> c = fixture.getClass();

            final Method[] methods = c.getMethods();
            final Method[] declaredMethods = c.getDeclaredMethods();
            final Set<Method> allMethods = new HashSet<Method>(Arrays.asList(methods));
            allMethods.addAll(Arrays.asList(declaredMethods));

            for(Method m : allMethods)
            {
                if(m.getParameterTypes().length == 0 && m.isAnnotationPresent(SetUp.class))
                {
                    final boolean isAccessible = m.isAccessible();

                    if(!isAccessible)
                    {
                        m.setAccessible(true);
                    }

                    try
                    {
                        m.invoke(fixture);
                    }
                    catch(IllegalAccessException e)
                    {
                        throw new SetUpException(e.getMessage(), e);
                    }
                    catch(InvocationTargetException e)
                    {
                        throw new SetUpException(e.getMessage(), e);
                    }

                    m.setAccessible(isAccessible);
                }
            }
        }

        public void tearDown(final Object fixture) throws TearDownException
        {
            final Class<?> c = fixture.getClass();

            final Method[] methods = c.getMethods();
            final Method[] declaredMethods = c.getDeclaredMethods();
            final Set<Method> allMethods = new HashSet<Method>(Arrays.asList(methods));
            allMethods.addAll(Arrays.asList(declaredMethods));

            for(Method m : allMethods)
            {
                if(m.getParameterTypes().length == 0 && m.isAnnotationPresent(TearDown.class))
                {
                    final boolean isAccessible = m.isAccessible();

                    if(!isAccessible)
                    {
                        m.setAccessible(true);
                    }

                    try
                    {
                        m.invoke(fixture);
                    }
                    catch(IllegalAccessException e)
                    {
                        throw new TearDownException(e.getMessage(), e);
                    }
                    catch(InvocationTargetException e)
                    {
                        throw new TearDownException(e.getMessage(), e);
                    }

                    m.setAccessible(isAccessible);
                }
            }
        }
    }
}
