/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Used to annotate a {@link Test test case method} that is expecting an exception to occur. If the exception or one of
 * its subclasses (if annotated with <code>subclass = true</code>) occurs during execution of the test case, the test
 * case has a {@link TestResultType#SUCCESS success} result.
 * If the exception doesn't occur or some other exception occurs, the test case has a
 * {@link TestResultType#FAILURE fail} result. If the exception is a compile-time checked exception, it will need to be
 * declared to be thrown on the test case method as per the language rules for the Java programming language. There is
 * no such requirement for unchecked exceptions.
 * <br/>
 * Example:<br/>
<pre>
    &#64Fixture("Demonstration")
    class MyTestFixture
    {
        &#64Test
        &#64ExpectException(NullPointerException.class)
        public void myTestCase1()
        {

        }

        &#64Test
        &#64ExpectException(NullPointerException.class)
        public void myTestCase2()
        {
            throw new NullPointerException();
        }

        &#64Test
        &#64ExpectException(NullPointerException.class)
        public void myTestCase3()
        {
            throw new IllegalArgumentException();
        }

        &#64Test
        &#64ExpectException(value=RuntimeException.class, subclass=true)
        public void myTestCase4()
        {
            throw new NullPointerException();
        }

        &#64Test
        &#64ExpectException(value=java.io.IOException.class, subclass=true)
        public void myTestCase5() throws java.io.FileNotFoundException
        {
            throw new java.io.FileNotFoundException();
        }
    }
</pre>
 * In the example given above:<br/>
 * <li>The test case method <code>myTestCase1</code> will give a fail result because it declares to expect a
 * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/NullPointerException.html">java.lang.NullPointerException</a>, however it is never
 * thrown.</li><br/>
 * <li>The test case method <code>myTestCase2</code> will give a success result because it declares to expect a
 * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/NullPointerException.html">java.lang.NullPointerException</a> and it is thrown in
 * the body of the method.</li><br/>
 * <li>The test case method <code>myTestCase3</code> will give a fail result because it declares to expect a
 * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/NullPointerException.html">java.lang.NullPointerException</a>, however it is never
 * thrown and some other type of exception is thrown.</li><br/>
 * <li>The test case method <code>myTestCase4</code> will give a success result because it declares to expect a
 * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/RuntimeException.html">java.lang.RuntimeException</a> or any of its subclasses,
 * and one of its subclasses,
 * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/NullPointerException.html">java.lang.NullPointerException</a>, is thrown in the
 * body of the method.</li><br/>
 * <li>The test case method <code>myTestCase5</code> will give a success result because it declares to expect a
 * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/io/IOException.html">java.io.IOException</a> or any of its subclasses,
 * and one of its subclasses,
 * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/io/FileNotFoundException.html">java.io.FileNotFoundException</a>, is thrown in the
 * body of the method. This method is compile-time checked and so must be declared on the body of the method as per the
 * language rules for the Java programming language.</li>
 *
 * @see Test
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
@Documented
@Target(ElementType.METHOD)
@Retention(RetentionPolicy.RUNTIME)
@Inherited
public @interface ExpectException
{
    /**
     * Returns the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">java.lang.Class</a> that the test case is expecting
     * will be thrown.
     *
     * @return The <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html">java.lang.Class</a> that the test case is expecting
     * will be thrown.
     */
    Class<? extends Throwable> value();

    /**
     * Returns <code>true</code> if the test case should succeed if a subclass of the exception is thrown,
     * <code>false</code> otherwise.
     *
     * @return <code>true</code> if the test case should succeed if a subclass of the exception is thrown,
     * <code>false</code> otherwise.
     */
    boolean subclass() default false;
}
