/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.framework.SequenceFactory.newSequence;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.LinkedList;
import java.util.List;
import static org.jtiger.framework.TestFailureFactory.newTestFailure;
import static org.jtiger.framework.FixtureResultFactory.newFixtureResult;
import static org.jtiger.framework.MethodCategoryFactory.newMethodCategory;
import static org.jtiger.framework.TestRunnerFactory.newTestRunner;
import static org.jtiger.framework.ClassConstructionFactory.newClassConstruction;
import static org.jtiger.framework.DefaultSetUpTearDownFactory.newDefaultSetUpTearDown;
import static org.jtiger.framework.DefaultTestDefinitionFactory.newDefaultTestDefinition;

final class FixtureRunnerFactory
{
    private FixtureRunnerFactory()
    {

    }

    static FixtureRunner newFixtureRunner()
    {
        return new FixtureRunnerImpl();
    }

    private static final class FixtureRunnerImpl implements FixtureRunner
    {
        FixtureRunnerImpl()
        {

        }

        public FixtureResult run(Class<?> fixtureClass,
                                 final Class<? extends TestDefinition> definitionClass,
                                 final Class<? extends SetUpTearDown> sutdClass,
                                 final boolean haltOnFailure,
                                 ReadOnlyArray<String> categories)
                throws RunnerException
        {
            if(fixtureClass == null)
            {
                return null;
            }

            try
            {
                TestDefinition definition = newClassConstruction().construct(definitionClass);
                SetUpTearDown sutd = newClassConstruction().construct(sutdClass);

                if(definition == null)
                {
                    definition = newDefaultTestDefinition();
                }

                if(sutd == null)
                {
                    sutd = newDefaultSetUpTearDown();
                }

                if(categories == null)
                {
                    categories = newSequence(new String[0]);
                }

                final List<TestResult> results = new LinkedList<TestResult>();
                final TestRunner runner = newTestRunner();
                final MethodCategory mc = newMethodCategory();

                while(fixtureClass != null)
                {
                    final Method[] methods = fixtureClass.getDeclaredMethods();

                    for(Method m : methods)
                    {
                        if(definition.isTest(m) &&
                                mc.isInAnyCategories(m, categories))
                        {
                            final Repeat r = m.getAnnotation(Repeat.class);

                            final long repeat = (r == null || r.value() < 0 ? 1L : r.value());

                            for(long count = 0; count < repeat; count++)
                            {
                                final Object fixture = newClassConstruction().construct(fixtureClass);

                                final TestResult result = runner.run(m, fixture, sutd);

                                results.add(result);

                                final TestFailure tf = newTestFailure();

                                if(haltOnFailure && tf.isFailure(result.getTestResultType()))
                                {
                                    return newFixtureResult(results);
                                }
                            }
                        }
                    }

                    fixtureClass = fixtureClass.getSuperclass();
                }

                return newFixtureResult(results);
            }
            catch(InstantiationException e)
            {
                throw new RunnerException(e.getMessage(), e);
            }
            catch(IllegalAccessException e)
            {
                throw new RunnerException(e.getMessage(), e);
            }
            catch(InvocationTargetException e)
            {
                throw new RunnerException(e.getMessage(), e);
            }
        }
    }
}
