/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.framework.SequenceFactory.newSequence;

/**
 * Returns instances of {@link FixturesRunnerConfig}.
 *
 * @see FixturesRunnerConfig
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class FixturesRunnerConfigFactory
{
    private FixturesRunnerConfigFactory()
    {

    }

    /**
     * Returns a new instance of {@link FixturesRunnerConfig}.
     *
     * @param fixtureClasses The test fixture classes.
     * @param definitionClass The test definition class (may be <code>null</code>).
     * @param sutdClass The test fixture set up and tear down class (may be <code>null</code>).
     * @param haltOnFailure Halt failure on test case execution failure.
     * @param categories The regular expressions representing the categories to execute during the test run.
     * @return A new instance of {@link FixturesRunnerConfig}.
     * @throws NullPointerException If <code>categories</code> is <code>null</code>. 
     */
    public static FixturesRunnerConfig newFixturesRunnerConfig(final ReadOnlyArray<Class<?>> fixtureClasses,
                                  final Class<? extends TestDefinition> definitionClass,
                                  final Class<? extends SetUpTearDown> sutdClass,
                                  final boolean haltOnFailure,
                                  final String... categories) throws NullPointerException
    {
        if(categories == null)
        {
            throw new NullPointerException();
        }

        return new FixturesRunnerConfigImpl(
                fixtureClasses,
                definitionClass,
                sutdClass,
                haltOnFailure,
                categories);
    }

    private static final class FixturesRunnerConfigImpl implements FixturesRunnerConfig
    {
        private static final long serialVersionUID = 2L;

        private final ReadOnlyArray<Class<?>> fixtureClasses;
        private final Class<? extends TestDefinition> definitionClass;
        private final Class<? extends SetUpTearDown> sutdClass;
        private final boolean haltOnFailure;
        private final ReadOnlyArray<String> categories;

        FixturesRunnerConfigImpl(final ReadOnlyArray<Class<?>> fixtureClasses,
                                  final Class<? extends TestDefinition> definitionClass,
                                  final Class<? extends SetUpTearDown> sutdClass,
                                  final boolean haltOnFailure,
                                  final String... categories)
        {
            this.fixtureClasses = fixtureClasses;
            this.definitionClass = definitionClass;
            this.sutdClass = sutdClass;
            this.haltOnFailure = haltOnFailure;
            this.categories = newSequence(categories);
        }

        public ReadOnlyArray<Class<?>> getFixtureClasses()
        {
            return fixtureClasses;
        }

        public Class<? extends TestDefinition> getDefinitionClass()
        {
            return definitionClass;
        }

        public Class<? extends SetUpTearDown> getSutdClass()
        {
            return sutdClass;
        }

        public boolean isHaltOnFailure()
        {
            return haltOnFailure;
        }

        public ReadOnlyArray<String> getCategories()
        {
            return categories;
        }

        @Override
        public String toString()
        {
            final StringBuilder sb = new StringBuilder();

            sb.append('[');
            sb.append(fixtureClasses);
            sb.append("][");
            sb.append(definitionClass);
            sb.append("][");
            sb.append(sutdClass);
            sb.append("][");
            sb.append(haltOnFailure);
            sb.append("][");
            sb.append(categories);
            sb.append(']');

            return sb.toString();
        }

        @Override
        public boolean equals(final Object o)
        {
            if(this == o)
            {
                return true;
            }

            if(o == null || o.getClass() != FixturesRunnerConfigImpl.class)
            {
                return false;
            }

            final FixturesRunnerConfigImpl p = (FixturesRunnerConfigImpl)o;

            return fixtureClasses.equals(p.fixtureClasses) &&
                    definitionClass == p.definitionClass &&
                    sutdClass == p.sutdClass &&
                    haltOnFailure == p.haltOnFailure &&
                    categories.equals(p.categories);
        }

        @Override
        public int hashCode()
        {
            final int oddPrime = 461;
            int result = 73;

            result = result * oddPrime + fixtureClasses.hashCode();
            result = result * oddPrime + definitionClass.hashCode();
            result = result * oddPrime + sutdClass.hashCode();
            result = result * oddPrime + (haltOnFailure ? 1 : 0);
            result = result * oddPrime + categories.hashCode();

            return result;
        }
    }
}
