/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import java.lang.reflect.InvocationTargetException;
import static org.jtiger.framework.ArgsConverterFactory.newArgsToFixtureRunnerParams;
import static org.jtiger.framework.ClassConstructionFactory.newClassConstruction;
import static org.jtiger.framework.TestFailureFactory.newTestFailure;
import static org.jtiger.framework.DefaultFixtureResultsHandlerFactory.newDefaultFixtureResultsHandler;

/**
 * Provides an entry point into a test execution by using a <code>main</code> method.
 * The main method creates a {@link FixturesRunnerConfig} from the command line arguments then delegates to a
 * {@link FixturesRunner} instance, which is created by a {@link FixturesRunnerFactory}.
 *
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class FixturesRunnerMain
{
    /**
     * A case-insensitive command line argument which should be followed by one or more arguments that represent the
     * class names of the test fixtures to execute. By omitting this argument, no test fixtures will be executed.
     *
     * @see FixturesRunnerConfig#getFixtureClasses()
     */
    public static final String ARG_FIXTURE_CLASSES = "-fixtureClasses";

    /**
     * A case-insensitive command line argument which should be followed by one class name argument that represents the
     * test definition class. By omitting this argument, the default implementation will be used.
     * This class should implement the interface {@link TestDefinition}, otherwise the default implementation is used.
     *
     * @see FixturesRunnerConfig#getDefinitionClass()
     */
    public static final String ARG_DEFINITION_CLASS = "-definitionClass";

    /**
     * A case-insensitive command line argument which should be followed by one argument that represents the class name
     * of the test fixture set up and tear down class. By omitting this argument, the default implementation will be
     * used.
     * This class should implement the interface {@link SetUpTearDown}, otherwise the default implementation is used.
     *
     * @see FixturesRunnerConfig#getSutdClass()
     */
    public static final String ARG_SUTD_CLASS = "-sutdClass";

    /**
     * A case-insensitive command line argument which should be followed by no arguments that represents that a test
     * execution will be of <a href="http://www.junit.org/" target="_blank">JUnit</a> test cases. Specifying this argument
     * overrides any value that is set for the {@link #ARG_DEFINITION_CLASS} or {@link #ARG_SUTD_CLASS} arguments.
     */
    public static final String ARG_JUNIT = "-junit";

    /**
     * A case-insensitive command line argument which should be followed by no arguments that represents that a test
     * execution should discontinue if a failure is encountered. By omitting this argument, the default value of
     * <code>false</code> will be used.
     *
     * @see FixturesRunnerConfig#isHaltOnFailure()
     */
    public static final String ARG_HALT_ON_FAILURE = "-haltOnFailure";

    /**
     * A case-insensitive command line argument which should be followed by one or more arguments that represent the
     * regular expressions of test case categories to execute. By omitting this argument, all categories will be
     * executed.
     *
     * @see FixturesRunnerConfig#getCategories()
     */
    public static final String ARG_CATEGORIES = "-categories";

    /**
     * A case-insensitive command line argument which should be followed by one argument that represents the class name
     * of the test fixture result handler.
     * This class should implement the interface {@link FixtureResultsHandler}, otherwise the default implementation is
     * used.
     */
    public static final String ARG_RESULT = "-result";

    /**
     * A case-insensitive command line argument which should be followed by one or more arguments that represent the
     * parameters to be passed to the test fixture result handler.
     */
    public static final String ARG_RESULT_PARAMETERS = "-resultParams";

    /**
     * The return code of the Java Virtual Machine if the command line arguments are in an incorrect format.
     * This includes passing no command line arguments.
     */
    public static final int RC_WRONG_USAGE = 1;

    /**
     * The return code of the Java Virtual Machine if {@link #ARG_HALT_ON_FAILURE} is used and a test case failure
     * occurs.
     */
    public static final int RC_HALTED_ON_FAILURE = 2;

    private FixturesRunnerMain()
    {

    }

    /**
     * Returns <code>true</code> if the given arguments can be used to execute {@link #main(String[]) main method}
     * successfully, <code>false</code> otherwise.
     *
     * @param args The arguments to test for a correct format for execution by the {@link #main(String[]) main method}.
     * @return <code>true</code> if the given arguments can be used to execute {@link #main(String[]) main method}
     * successfully, <code>false</code> otherwise.
     * @throws ClassNotFoundException If the passed class names in the command line arguments cannot be loaded.
     */
    public static boolean isCorrectFormat(final String... args) throws ClassNotFoundException
    {
        return args != null && args.length > 0 && newArgsToFixtureRunnerParams().convert(args) != null;
    }

    /**
     * Execute the test run with the given command line arguments.
     * This method will cause a forced JVM shut down by calling
     * <a href="%j2se.api.spec/java/lang/System#exit(int)">System.exit</a> only if the passed command line arguments are
     * not in a correct format (or not present at all) or if {@link #ARG_HALT_ON_FAILURE} has been set and a test
     * failure occurs. It is guaranteed that if {@link #isCorrectFormat(String[])} returns <code>true</code> and if
     * {@link #ARG_HALT_ON_FAILURE} is not passed, that a forced JVM shut down will not occur.
     * Compile-time checked exceptions, which may be thrown during test execution, will be handled by outputting them to
     * the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/System.html#err">standard error stream</a>.
     *
     * @param args The arguments to execute the test run with.
     */
    public static void main(final String... args)
    {
        if(args.length == 0)
        {
            usage();

            System.exit(RC_WRONG_USAGE);
        }

        try
        {
            final Args a = newArgsToFixtureRunnerParams().convert(args);

            if(a == null)
            {
                usage();

                System.exit(RC_WRONG_USAGE);
            }
            else
            {
                final FixturesRunner runner = FixturesRunnerFactory.newFixturesRunner();
                final FixturesRunnerConfig config = a.getFixturesRunnerConfig();
                final FixtureResults results = runner.run(config);

                if(config.isHaltOnFailure())
                {
                    final TestFailure tf = newTestFailure();

                    for(FixtureResult result : results)
                    {
                        for(TestResult tr : result)
                        {
                            if(tf.isFailure(tr.getTestResultType()))
                            {
                                System.exit(RC_HALTED_ON_FAILURE);
                            }
                        }
                    }
                }

                final ClassConstruction construction = newClassConstruction();
                FixtureResultsHandler rh = construction.construct(a.getResultHandlerClass());

                if(rh == null)
                {
                    rh = newDefaultFixtureResultsHandler();
                }

                rh.handleResult(results, a.getResultHandlerParameters());
            }
        }
        catch(FixtureResultsHandlerException e)
        {
            System.err.println(e);
        }
        catch(ClassNotFoundException e)
        {
            System.err.println(e);
        }
        catch(IllegalAccessException e)
        {
            System.err.println(e);
        }
        catch(InstantiationException e)
        {
            System.err.println(e);
        }
        catch(InvocationTargetException e)
        {
            System.err.println(e);
        }
        catch(RunnerException e)
        {
            System.err.println(e);
        }
    }

    /**
     * Prints usage information to the <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/System.html#err">standard error stream</a>.
     */
    public static void usage()
    {
        System.err.println(Messages.usage());
    }
}
