/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import java.io.Serializable;

/**
 * A sequentially ordered set of elements that cannot be mutated (changed).
 *
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218</i>
 */
public interface Sequence<E> extends Iterable<E>, ReadOnlyArray<E>, Serializable
{
    /**
     * Returns the element at the given index in the sequence.
     *
     * @param index The index of the element to return.
     * @return The element at the specified index.
     * @throws IndexOutOfBoundsException If the given index is out of range
     * (<code>index < 0 || index >= {@link #length length()}</code>).
     */
    E get(int index) throws IndexOutOfBoundsException;

    /**
     * Returns the number of elements held in this sequence.
     *
     * @return The number of elements held in this sequence.
     */
    int length();

    /**
     * Appends the given sequence to this sequence and returns a new sequence.
     *
     * @param sequence The sequence to append to this sequence.
     * @return A new sequence that has the passed sequence appended.
     * @throws NullPointerException If <code>sequence</code> is <code>null</code>.
     */
    Sequence<E> append(Sequence<E> sequence) throws NullPointerException;

    /**
     * Returns a sequence that contains a subset of the elements of this sequence within the given positional range.
     * The position is indexed from zero and represents a position between elements within the sequence.
     * For example, the start position is zero, where the position between the first and second element is one.
     *
     * @param fromPosition The position from which to take the subsequence.
     * @param toPosition The position to which to take the subsequence.
     * @return A sequence that contains a subset of the elements of this sequence within the given range.
     * @throws PositionOutOfBoundsException If
     * <code>(fromPosition < 0 || toPosition > length() || fromPosition > toPosition)</code> evaluates to
     * <code>true</code>.
     */
    Sequence<E> getSubSequence(int fromPosition, int toPosition) throws PositionOutOfBoundsException;

    /**
     * Inserts the given sequence at the given position and returns the resulting sequence.
     * The position is indexed from zero and represents a position between elements within the sequence.
     * For example, the start position is zero, where the position between the first and second element is one.
     *
     * @param position The position at which to insert the given sequence.
     * @param sequence The sequence to insert at the given position.
     * @return A sequence that contains the given sequence inserted at the given position.
     * @throws PositionOutOfBoundsException If <code>position</code> is less than zero or greater than the
     * {@link #length() length} of this <tt>Sequence</tt>. If <code>(fromIndex < 0 || fromIndex > length())</code> holds
     * <code>true</code>.
     * @throws NullPointerException If <code>sequence</code> is <code>null</code>.
     */
    Sequence<E> insert(int position, Sequence<E> sequence) throws PositionOutOfBoundsException, NullPointerException;

    /**
     * Deletes elements between the given positions and returns the resulting sequence.
     * The position is indexed from zero and represents a position between elements within the sequence.
     * For example, the start position is zero, where the position between the first and second element is one.
     *
     * @param fromPosition The position from which to start deleting elements.
     * @param toPosition The position to which to start deleting elements.
     * @return A sequence that contains the elements of this sequence without the deleted elements.
     * @throws PositionOutOfBoundsException If
     * <code>(fromPosition < 0 || toPosition > length() || fromPosition > toPosition)</code> evaluates to
     * <code>true</code>.
     */
    Sequence<E> delete(int fromPosition, int toPosition) throws PositionOutOfBoundsException;

    /**
     * Returns the first index of the given element in the sequence, or less than zero if it is not present.
     *
     * @param e The element to find the first index of.
     * @return The first index of the given element in the sequence, or less than zero if it is not present.
     */
    int indexOf(E e);

    /**
     * Returns the first index of the given element in the sequence from the given position, or less than zero if it is
     * not present.
     * The position is indexed from zero and represents a position between elements within the sequence.
     * For example, the start position is zero, where the position between the first and second element is one.
     *
     * @param position The position from which to find the element.
     * @param e The element to find the first index of from the given position.
     * @return The first index of the given element in the sequence from the given position, or less than zero if it is
     * not present.
     * @throws PositionOutOfBoundsException If <code>position</code> is less than zero or greater than the
     * {@link #length() length} of this <tt>Sequence</tt>. If <code>(fromIndex < 0 || fromIndex > length())</code> holds
     * <code>true</code>.
     */
     int indexOf(int position, E e) throws PositionOutOfBoundsException;

    /**
     * Returns the last index of the given element in the sequence, or less than zero if it is not present.
     *
     * @param e The element to find the last index of.
     * @return The last index of the given element in the sequence, or less than zero if it is not present.
     */
    int lastIndexOf(E e);

    /**
     * Returns the last index of the given element in the sequence from the given position, or less than zero if it is not
     * present.
     * The position is indexed from zero and represents a position between elements within the sequence.
     * For example, the start position is zero, where the position between the first and second element is one.
     *
     * @param position The position from which to find the element.
     * @param e The element to find the last index of from the given position.
     * @return The last index of the given element from the given position in the sequence, or less than zero if it is
     * not present.
     * @throws PositionOutOfBoundsException If <code>position</code> is less than zero or greater than the
     * {@link #length() length} of this <tt>Sequence</tt>. If <code>(fromIndex < 0 || fromIndex > length())</code>
     * holds <code>true</code>.
     */
     int lastIndexOf(int position, E e) throws PositionOutOfBoundsException;

    /**
     * Returns an iterator for this <tt>Sequence</tt>.
     *
     * @return An iterator for this <tt>Sequence</tt>.
     */
    SequenceIterator<E> getIterator();

    /**
     * Returns an iterator for this <tt>Sequence</tt> from the given position.
     * The position is indexed from zero and represents a position between elements within the sequence.
     * For example, the start position is zero, where the position between the first and second element is one.
     *
     * @param position The position from which the iterator should iterate.
     * @return An iterator for this <tt>Sequence</tt> from the given position.
     * @throws PositionOutOfBoundsException If <code>position</code> is less than zero or greater than the
     * {@link #length() length} of this <tt>Sequence</tt>. If <code>(fromIndex < 0 || fromIndex > length())</code>
     * holds <code>true</code>.
     */
    SequenceIterator<E> getIterator(int position) throws PositionOutOfBoundsException;

    /**
     * Returns <code>true</code> is the sequence is empty, <code>false</code> otherwise.
     *
     * @return <code>true</code> is the sequence is empty, <code>false</code> otherwise.
     */
    boolean isEmpty();

    /**
     * Returns a <tt>String</tt> representation of this <tt>Sequence</tt>.
     *
     * @return A <tt>String</tt> representation of this <tt>Sequence</tt>.
     */
    String toString();

    /**
     * Determines equality for this <tt>Sequence</tt>. Returns <code>true</code> if each element in this
     * <tt>Sequence</tt> is equal to each element in the <tt>Sequence</tt> that is passed as an argument, returns
     * <code>false</code> otherwise. Equal elements in a different order of two <tt>Sequence</tt> instances deems the
     * two <tt>Sequence</tt> instances unequal and therefore, <code>false</code> is returned. Implementations of this
     * method must adhere to the contract set out by {@link Object#equals(Object)}.
     *
     * @param o The object to test equality for.
     * @return <code>true</code> if each element in this <tt>Sequence</tt> is equal to each element in the
     * <tt>Sequence</tt> that is passed as an argument, returns <code>false</code> otherwise.
     */
    boolean equals(Object o);

    /**
     * Returns a hash code for this <tt>Sequence</tt>. Implementations of this method must adhere to the contract set
     * out by {@link Object#hashCode()}.
     *
     * @return A hash code for this <tt>Sequence</tt>.
     */
    int hashCode();

    /**
     * Returns a clone of this <tt>Sequence</tt>.
     *
     * @return A clone of this <tt>Sequence</tt>.
     */
    Sequence<E> clone();
}
