/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.List;
import java.util.ListIterator;

/**
 * Returns instances of {@link Sequence} such that state cannot be modified.
 *
 * @see Sequence
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218</i>
 */
public final class SequenceFactory
{
    private SequenceFactory() throws UnsupportedOperationException
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Returns a new instance of {@link Sequence} that is backed by the given arguments.
     *
     * @param a The array that backs the returned {@link Sequence}.
     * @return A new instance of {@link Sequence} that is backed by the given arguments.
     * @throws NullPointerException If <code>a</code> is <code>null</code>.
     */
    public static <E> Sequence<E> newSequence(final E... a) throws NullPointerException
    {
        if(a == null)
        {
            throw new NullPointerException();
        }
        else
        {
            return new ArraySequence<E>(a);
        }
    }

    /**
     * Returns a new instance of {@link Sequence} that is backed by the given {@link java.util.List}.
     *
     * @param l The {@link java.util.List} that backs the returned {@link Sequence}.
     * @return A new instance of {@link Sequence} that is backed by the given {@link java.util.List}.
     * @throws NullPointerException If <code>l</code> is <code>null</code>.
     */
    public static <E> Sequence<E> newSequence(final List<E> l) throws NullPointerException
    {
        if(l == null)
        {
            throw new NullPointerException();
        }
        else
        {
            return new ListSequence<E>(l);
        }
    }

    /**
     * Returns a new instance of {@link Sequence} that is backed by the given constant for the given length.
     *
     * @param constant The constant that backs the returned {@link Sequence}.
     * @param length The length of the constant that backs the returned {@link Sequence}.
     * @return A new instance of {@link Sequence} that is backed by the given constant.
     * @throws IllegalArgumentException If <code>length</code> is less than <code>1</code>.
     */
    public static <E> Sequence<E> newConstantSequence(final E constant, final int length) throws IllegalArgumentException
    {
        if(length < 1)
        {
            throw new IllegalArgumentException();
        }
        else
        {
            return new ConstantSequence<E>(constant, length);
        }
    }

    private static final class ArraySequence<E> implements Sequence<E>
    {
        private static final long serialVersionUID = 2L;

        private final E[] a;
        private int hc;
        private boolean hcCalculated;
        private String ts;

        ArraySequence(final E... a)
        {
            this.a = a;
        }

        public E get(final int index) throws IndexOutOfBoundsException
        {
            if(index < 0 || index >= length())
            {
                throw new IndexOutOfBoundsException(String.valueOf(index));
            }
            else
            {
                return a[index];
            }
        }

        public int length()
        {
            return a.length;
        }

        public Sequence<E> append(final Sequence<E> sequence) throws NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else if(length() == 0)
            {
                return sequence;
            }
            else if(sequence.length() == 0)
            {
                return this;
            }
            else
            {
                return new CompositeSequence<E>(this, sequence);
            }
        }

        public Sequence<E> getSubSequence(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else
            {
                return new SubSequence<E>(this, fromPosition, toPosition);
            }
        }

        public Sequence<E> insert(final int position, final Sequence<E> sequence) throws PositionOutOfBoundsException, NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else
            {
                if(position < 0 || position > length())
                {
                    throw new PositionOutOfBoundsException(position);
                }
                else if(position == 0)
                {
                    return sequence.append(this);
                }
                else if(position == length())
                {
                    return this.append(sequence);
                }
                else
                {
                    return new InsertSequence<E>(position, this, sequence);
                }
            }
        }

        public Sequence<E> delete(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(fromPosition == toPosition)
            {
                return this;
            }
            else
            {
                return new DeleteSequence<E>(fromPosition, toPosition, this);
            }
        }

        public int indexOf(final E e)
        {
            return indexOf(0, e);
        }

        public int indexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasNext())
                {
                    final int itPosition = it.getPosition();
                    final E x = it.next();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public int lastIndexOf(final E e)
        {
            return lastIndexOf(length(), e);
        }

        public int lastIndexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasPrevious())
                {
                    final E x = it.previous();
                    final int itPosition = it.getPosition();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public boolean isEmpty()
        {
            return length() == 0;
        }

        public Iterator<E> iterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator(final int position) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                return new SequenceIterator<E>()
                {
                    private int pos;

                    {
                        pos = position;
                    }

                    public final boolean hasNext()
                    {
                        return pos != length();
                    }

                    public final E next()
                    {
                        if(pos == length())
                        {
                            throw new NoSuchElementException();
                        }
                        else
                        {
                            return a[pos++];
                        }
                    }

                    public final void remove()
                    {
                        throw new UnsupportedOperationException();
                    }

                    public boolean hasPrevious()
                    {
                        return pos != 0;
                    }

                    public E previous() throws NoSuchElementException
                    {
                        if(pos == 0)
                        {
                            throw new NoSuchElementException();
                        }
                        else
                        {
                            return a[--pos];
                        }
                    }

                    public int getPosition()
                    {
                        return pos;
                    }
                };
            }
        }

        @Override
        public String toString()
        {
            if(ts == null)
            {
                final StringBuilder sb = new StringBuilder();

                sb.append("[");

                final SequenceIterator<E> it = getIterator();

                while(it.hasNext())
                {
                    sb.append(it.next());

                    if(it.hasNext())
                    {
                        sb.append("][");
                    }
                }

                sb.append("]");

                ts = sb.toString();
            }

            return ts;
        }

        @Override
        public boolean equals(final Object o)
        {
            if(this == o)
            {
                return true;
            }
            else if(o == null || (!(o instanceof Sequence)))
            {
                return false;
            }
            else
            {
                final Sequence<?> s = (Sequence<?>)o;

                if(s.length() != length())
                {
                    return false;
                }
                else
                {
                    final Iterator<?> it1 = iterator();
                    final Iterator<?> it2 = s.iterator();

                    while(it1.hasNext() && it2.hasNext())
                    {
                        final Object se1 = it1.next();
                        final Object se2 = it2.next();

                        if(se1 == null ? se2 != null : !se1.equals(se2))
                        {
                            return false;
                        }
                    }

                    return true;
                }
            }
        }

        @Override
        public int hashCode()
        {
            if(!hcCalculated)
            {
                final int oddPrime = 461;
                int result = 73;

                for(E e : this)
                {
                    if(e != null)
                    {
                        result = result * oddPrime + e.hashCode();
                    }
                }

                hc = result;
                hcCalculated = true;
            }

            return hc;
        }

        public Sequence<E> clone()
        {
            return new ArraySequence<E>(a);
        }
    }

    private static final class ListSequence<E> implements Sequence<E>
    {
        private static final long serialVersionUID = 2L;

        private final List<E> l;
        private int hc;
        private boolean hcCalculated;
        private String ts;

        ListSequence(final List<E> l)
        {
            this.l = l;
        }

        public E get(final int index) throws IndexOutOfBoundsException
        {
            if(index < 0 || index >= length())
            {
                throw new IndexOutOfBoundsException(String.valueOf(index));
            }
            else
            {
                return l.get(index);
            }
        }

        public int length()
        {
            return l.size();
        }

        public Sequence<E> append(final Sequence<E> sequence) throws NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else if(length() == 0)
            {
                return sequence;
            }
            else if(sequence.length() == 0)
            {
                return this;
            }
            else
            {
                return new CompositeSequence<E>(this, sequence);
            }
        }

        public Sequence<E> getSubSequence(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else
            {
                return new SubSequence<E>(this, fromPosition, toPosition);
            }
        }

        public Sequence<E> insert(final int position, final Sequence<E> sequence) throws PositionOutOfBoundsException, NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else
            {
                if(position < 0 || position > length())
                {
                    throw new PositionOutOfBoundsException(position);
                }
                else if(position == 0)
                {
                    return sequence.append(this);
                }
                else if(position == length())
                {
                    return this.append(sequence);
                }
                else
                {
                    return new InsertSequence<E>(position, this, sequence);
                }
            }
        }

        public Sequence<E> delete(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(fromPosition == toPosition)
            {
                return this;
            }
            else
            {
                return new DeleteSequence<E>(fromPosition, toPosition, this);
            }
        }

        public int indexOf(final E e)
        {
            return indexOf(0, e);
        }

        public int indexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasNext())
                {
                    final int itPosition = it.getPosition();
                    final E x = it.next();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public int lastIndexOf(final E e)
        {
            return lastIndexOf(length(), e);
        }

        public int lastIndexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasPrevious())
                {
                    final E x = it.previous();
                    final int itPosition = it.getPosition();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public boolean isEmpty()
        {
            return length() == 0;
        }

        public Iterator<E> iterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator(final int position) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                return new SequenceIterator<E>()
                {
                    private int pos;
                    private final ListIterator<E> it;

                    {
                        pos = position;
                        it = l.listIterator(position);
                    }

                    public final boolean hasNext()
                    {
                        return pos != length();
                    }

                    public final E next()
                    {
                        if(pos == length())
                        {
                            throw new NoSuchElementException();
                        }
                        else
                        {
                            pos++;
                            return it.next();
                        }
                    }

                    public final void remove()
                    {
                        throw new UnsupportedOperationException();
                    }

                    public boolean hasPrevious()
                    {
                        return pos != 0;
                    }

                    public E previous() throws NoSuchElementException
                    {
                        if(pos == 0)
                        {
                            throw new NoSuchElementException();
                        }
                        else
                        {
                            pos--;
                            return it.previous();
                        }
                    }

                    public int getPosition()
                    {
                        return pos;
                    }
                };
            }
        }

        @Override
        public String toString()
        {
            if(ts == null)
            {
                final StringBuilder sb = new StringBuilder();

                sb.append("[");

                final SequenceIterator<E> it = getIterator();

                while(it.hasNext())
                {
                    sb.append(it.next());

                    if(it.hasNext())
                    {
                        sb.append("][");
                    }
                }

                sb.append("]");

                ts = sb.toString();
            }

            return ts;
        }

        @Override
        public boolean equals(final Object o)
        {
            if(this == o)
            {
                return true;
            }
            else if(o == null || (!(o instanceof Sequence)))
            {
                return false;
            }
            else
            {
                final Sequence<?> s = (Sequence<?>)o;

                if(s.length() != length())
                {
                    return false;
                }
                else
                {
                    final Iterator<?> it1 = iterator();
                    final Iterator<?> it2 = s.iterator();

                    while(it1.hasNext() && it2.hasNext())
                    {
                        final Object se1 = it1.next();
                        final Object se2 = it2.next();

                        if(se1 == null ? se2 != null : !se1.equals(se2))
                        {
                            return false;
                        }
                    }

                    return true;
                }
            }
        }

        @Override
        public int hashCode()
        {
            if(!hcCalculated)
            {
                final int oddPrime = 461;
                int result = 73;

                for(E e : this)
                {
                    if(e != null)
                    {
                        result = result * oddPrime + e.hashCode();
                    }
                }

                hc = result;
                hcCalculated = true;
            }

            return hc;
        }

        public Sequence<E> clone()
        {
            return new ListSequence<E>(l);
        }
    }

    private static final class CompositeSequence<E> implements Sequence<E>
    {
        private static final long serialVersionUID = 2L;

        private final Sequence<E> seq1;
        private final Sequence<E> seq2;
        private int hc;
        private boolean hcCalculated;
        private String ts;

        CompositeSequence(final Sequence<E> seq1, final Sequence<E> seq2)
        {
            this.seq1 = seq1;
            this.seq2 = seq2;
        }

        public E get(final int index) throws IndexOutOfBoundsException
        {
            if(index < 0 || index >= length())
            {
                throw new IndexOutOfBoundsException();
            }
            else if(index < seq1.length())
            {
                return seq1.get(index);
            }
            else
            {
                return seq2.get(index - seq1.length());
            }
        }

        public int length()
        {
            return seq1.length() + seq2.length();
        }

        public Sequence<E> append(final Sequence<E> sequence) throws NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else if(sequence.length() == 0)
            {
                return this;
            }
            else
            {
                return new CompositeSequence<E>(this, sequence);
            }
        }

        public Sequence<E> getSubSequence(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else
            {
                return new SubSequence<E>(this, fromPosition, toPosition);
            }
        }

        public Sequence<E> insert(final int position, final Sequence<E> sequence) throws PositionOutOfBoundsException, NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else
            {
                if(position < 0 || position > length())
                {
                    throw new PositionOutOfBoundsException(position);
                }
                else if(position == 0)
                {
                    return sequence.append(this);
                }
                else if(position == length())
                {
                    return this.append(sequence);
                }
                else
                {
                    return new InsertSequence<E>(position, this, sequence);
                }
            }
        }

        public Sequence<E> delete(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(fromPosition == toPosition)
            {
                return this;
            }
            else
            {
                return new DeleteSequence<E>(fromPosition, toPosition, this);
            }
        }

        public int indexOf(final E e)
        {
            return indexOf(0, e);
        }

        public int indexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasNext())
                {
                    final int itPosition = it.getPosition();
                    final E x = it.next();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public int lastIndexOf(final E e)
        {
            return lastIndexOf(length(), e);
        }

        public int lastIndexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasPrevious())
                {
                    final E x = it.previous();
                    final int itPosition = it.getPosition();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public boolean isEmpty()
        {
            return false;
        }

        public Iterator<E> iterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator(final int position) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                return new SequenceIterator<E>()
                {
                    private int pos;

                    {
                        pos = position;
                    }

                    public final boolean hasNext()
                    {
                        return pos != length();
                    }

                    public final E next()
                    {
                        if(pos == length())
                        {
                            throw new NoSuchElementException();
                        }
                        else if(pos < seq1.length())
                        {
                            return seq1.get(pos++);
                        }
                        else
                        {
                            return seq2.get(pos++ - seq1.length());
                        }
                    }

                    public final void remove()
                    {
                        throw new UnsupportedOperationException();
                    }

                    public boolean hasPrevious()
                    {
                        return pos != 0;
                    }

                    public E previous() throws NoSuchElementException
                    {
                        if(pos == 0)
                        {
                            throw new NoSuchElementException();
                        }
                        else if(pos <= seq1.length())
                        {
                            return seq1.get(--pos);
                        }
                        else
                        {
                            return seq2.get(--pos - seq1.length());
                        }
                    }

                    public int getPosition()
                    {
                        return pos;
                    }
                };
            }
        }

        @Override
        public String toString()
        {
            if(ts == null)
            {
                final StringBuilder sb = new StringBuilder();

                sb.append("[");

                final SequenceIterator<E> it = getIterator();

                while(it.hasNext())
                {
                    sb.append(it.next());

                    if(it.hasNext())
                    {
                        sb.append("][");
                    }
                }

                sb.append("]");

                ts = sb.toString();
            }

            return ts;
        }

        @Override
        public boolean equals(final Object o)
        {
            if(this == o)
            {
                return true;
            }
            else if(o == null || (!(o instanceof Sequence)))
            {
                return false;
            }
            else
            {
                final Sequence<?> s = (Sequence<?>)o;

                if(s.length() != length())
                {
                    return false;
                }
                else
                {
                    final Iterator<?> it1 = iterator();
                    final Iterator<?> it2 = s.iterator();

                    while(it1.hasNext() && it2.hasNext())
                    {
                        final Object se1 = it1.next();
                        final Object se2 = it2.next();

                        if(se1 == null ? se2 != null : !se1.equals(se2))
                        {
                            return false;
                        }
                    }

                    return true;
                }
            }
        }

        @Override
        public int hashCode()
        {
            if(!hcCalculated)
            {
                final int oddPrime = 461;
                int result = 73;

                for(E e : this)
                {
                    if(e != null)
                    {
                        result = result * oddPrime + e.hashCode();
                    }
                }

                hc = result;
                hcCalculated = true;
            }

            return hc;
        }

        public Sequence<E> clone()
        {
            return new CompositeSequence<E>(seq1, seq2);
        }
    }

    private static final class SubSequence<E> implements Sequence<E>
    {
        private static final long serialVersionUID = 2L;

        private final Sequence<E> sequence;
        private final int fromPosition;
        private final int toPosition;
        private int hc;
        private boolean hcCalculated;
        private String ts;

        SubSequence(final Sequence<E> sequence, final int fromPosition, final int toPosition)
        {
            this.sequence = sequence;
            this.fromPosition = fromPosition;
            this.toPosition = toPosition;
        }

        public E get(final int index) throws IndexOutOfBoundsException
        {
            if(index < 0 || index >= length())
            {
                throw new IndexOutOfBoundsException();
            }
            else
            {
                return sequence.get(index + fromPosition);
            }
        }

        public int length()
        {
            return toPosition - fromPosition;
        }

        public Sequence<E> append(final Sequence<E> sequence) throws NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else if(length() == 0)
            {
                return sequence;
            }
            else if(sequence.length() == 0)
            {
                return this;
            }
            else
            {
                return new CompositeSequence<E>(this, sequence);
            }
        }

        public Sequence<E> getSubSequence(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else
            {
                return new SubSequence<E>(this, fromPosition, toPosition);
            }
        }

        public Sequence<E> insert(final int position, final Sequence<E> sequence) throws PositionOutOfBoundsException, NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else
            {
                if(position < 0 || position > length())
                {
                    throw new PositionOutOfBoundsException(position);
                }
                else if(position == 0)
                {
                    return sequence.append(this);
                }
                else if(position == length())
                {
                    return this.append(sequence);
                }
                else
                {
                    return new InsertSequence<E>(position, this, sequence);
                }
            }
        }

        public Sequence<E> delete(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(fromPosition == toPosition)
            {
                return this;
            }
            else
            {
                return new DeleteSequence<E>(fromPosition, toPosition, this);
            }
        }

        public int indexOf(final E e)
        {
            return indexOf(0, e);
        }

        public int indexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasNext())
                {
                    final int itPosition = it.getPosition();
                    final E x = it.next();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public int lastIndexOf(final E e)
        {
            return lastIndexOf(length(), e);
        }

        public int lastIndexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasPrevious())
                {
                    final E x = it.previous();
                    final int itPosition = it.getPosition();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public boolean isEmpty()
        {
            return length() == 0;
        }

        public Iterator<E> iterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator(final int position) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                return new SequenceIterator<E>()
                {
                    private int pos;

                    {
                        pos = position;
                    }

                    public boolean hasNext()
                    {
                        return pos != length();
                    }

                    public E next() throws NoSuchElementException
                    {
                        if(pos == length())
                        {
                            throw new NoSuchElementException();
                        }
                        else
                        {
                            return sequence.get(SubSequence.this.fromPosition + pos++);
                        }
                    }

                    public void remove()
                    {
                        throw new UnsupportedOperationException();
                    }

                    public boolean hasPrevious()
                    {
                        return pos != 0;
                    }

                    public E previous() throws NoSuchElementException
                    {
                        if(pos == 0)
                        {
                            throw new NoSuchElementException();
                        }
                        else
                        {
                            return sequence.get(SubSequence.this.fromPosition + --pos);
                        }
                    }

                    public int getPosition()
                    {
                        return pos;
                    }
                };
            }
        }

        @Override
        public String toString()
        {
            if(ts == null)
            {
                final StringBuilder sb = new StringBuilder();

                sb.append("[");

                final SequenceIterator<E> it = getIterator();

                while(it.hasNext())
                {
                    sb.append(it.next());

                    if(it.hasNext())
                    {
                        sb.append("][");
                    }
                }

                sb.append("]");

                ts = sb.toString();
            }

            return ts;
        }

        @Override
        public boolean equals(final Object o)
        {
            if(this == o)
            {
                return true;
            }
            else if(o == null || (!(o instanceof Sequence)))
            {
                return false;
            }
            else
            {
                final Sequence<?> s = (Sequence<?>)o;

                if(s.length() != length())
                {
                    return false;
                }
                else
                {
                    final Iterator<?> it1 = iterator();
                    final Iterator<?> it2 = s.iterator();

                    while(it1.hasNext() && it2.hasNext())
                    {
                        final Object se1 = it1.next();
                        final Object se2 = it2.next();

                        if(se1 == null ? se2 != null : !se1.equals(se2))
                        {
                            return false;
                        }
                    }

                    return true;
                }
            }
        }

        @Override
        public int hashCode()
        {
            if(!hcCalculated)
            {
                final int oddPrime = 461;
                int result = 73;

                for(E e : this)
                {
                    if(e != null)
                    {
                        result = result * oddPrime + e.hashCode();
                    }
                }

                hc = result;
                hcCalculated = true;
            }

            return hc;
        }

        public Sequence<E> clone()
        {
            return new SubSequence<E>(sequence, fromPosition, toPosition);
        }
    }

    private static final class InsertSequence<E> implements Sequence<E>
    {
        private static final long serialVersionUID = 2L;

        private final int position;
        private final Sequence<E> sequence;
        private final Sequence<E> inserted;
        private int hc;
        private boolean hcCalculated;
        private String ts;

        InsertSequence(final int position, final Sequence<E> sequence, final Sequence<E> inserted)
        {
            this.position = position;
            this.sequence = sequence;
            this.inserted = inserted;
        }

        public E get(final int index) throws IndexOutOfBoundsException
        {
            if(index < 0 || index >= length())
            {
                throw new IndexOutOfBoundsException();
            }
            else if(index < position)
            {
                return sequence.get(index);
            }
            else if(index < inserted.length() + position)
            {
                return inserted.get(index - position);
            }
            else
            {
                return sequence.get(index - inserted.length());
            }
        }

        public int length()
        {
            return sequence.length() + inserted.length();
        }

        public Sequence<E> append(final Sequence<E> sequence) throws NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else if(sequence.length() == 0)
            {
                return this;
            }
            else
            {
                return new CompositeSequence<E>(this, sequence);
            }
        }

        public Sequence<E> getSubSequence(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else
            {
                return new SubSequence<E>(this, fromPosition, toPosition);
            }
        }

        public Sequence<E> insert(final int position, final Sequence<E> sequence) throws PositionOutOfBoundsException, NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else
            {
                if(position < 0 || position > length())
                {
                    throw new PositionOutOfBoundsException(position);
                }
                else if(position == 0)
                {
                    return sequence.append(this);
                }
                else if(position == length())
                {
                    return this.append(sequence);
                }
                else
                {
                    return new InsertSequence<E>(position, this, sequence);
                }
            }
        }

        public Sequence<E> delete(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(fromPosition == toPosition)
            {
                return this;
            }
            else
            {
                return new DeleteSequence<E>(fromPosition, toPosition, this);
            }
        }

        public int indexOf(final E e)
        {
            return indexOf(0, e);
        }

        public int indexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasNext())
                {
                    final int itPosition = it.getPosition();
                    final E x = it.next();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public int lastIndexOf(final E e)
        {
            return lastIndexOf(length(), e);
        }

        public int lastIndexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasPrevious())
                {
                    final E x = it.previous();
                    final int itPosition = it.getPosition();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public SequenceIterator<E> getIterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator(final int position) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                return new SequenceIterator<E>()
                {
                    private int pos;

                    {
                        pos = position;
                    }

                    public final boolean hasNext()
                    {
                        return pos != length();
                    }

                    public final E next()
                    {
                        if(pos == length())
                        {
                            throw new NoSuchElementException();
                        }
                        else if(pos < InsertSequence.this.position)
                        {
                            return sequence.get(pos++);
                        }
                        else if(pos < inserted.length() + InsertSequence.this.position)
                        {
                            return inserted.get(pos++ - InsertSequence.this.position);
                        }
                        else
                        {
                            return sequence.get(pos++ - inserted.length());
                        }
                    }

                    public final void remove()
                    {
                        throw new UnsupportedOperationException();
                    }

                    public boolean hasPrevious()
                    {
                        return pos != 0;
                    }

                    public E previous() throws NoSuchElementException
                    {
                        if(pos == 0)
                        {
                            throw new NoSuchElementException();
                        }
                        else if(pos <= InsertSequence.this.position)
                        {
                            return sequence.get(--pos);
                        }
                        else if(pos <= inserted.length() + InsertSequence.this.position)
                        {
                            return inserted.get(--pos - InsertSequence.this.position);
                        }
                        else
                        {
                            return sequence.get(--pos - inserted.length());
                        }
                    }

                    public int getPosition()
                    {
                        return pos;
                    }
                };
            }
        }

        public boolean isEmpty()
        {
            return false;
        }

        public Iterator<E> iterator()
        {
            return getIterator(0);
        }

        @Override
        public String toString()
        {
            if(ts == null)
            {
                final StringBuilder sb = new StringBuilder();

                sb.append("[");

                final SequenceIterator<E> it = getIterator();

                while(it.hasNext())
                {
                    sb.append(it.next());

                    if(it.hasNext())
                    {
                        sb.append("][");
                    }
                }

                sb.append("]");

                ts = sb.toString();
            }

            return ts;
        }

        @Override
        public boolean equals(final Object o)
        {
            if(this == o)
            {
                return true;
            }
            else if(o == null || (!(o instanceof Sequence)))
            {
                return false;
            }
            else
            {
                final Sequence<?> s = (Sequence<?>)o;

                if(s.length() != length())
                {
                    return false;
                }
                else
                {
                    final Iterator<?> it1 = iterator();
                    final Iterator<?> it2 = s.iterator();

                    while(it1.hasNext() && it2.hasNext())
                    {
                        final Object se1 = it1.next();
                        final Object se2 = it2.next();

                        if(se1 == null ? se2 != null : !se1.equals(se2))
                        {
                            return false;
                        }
                    }

                    return true;
                }
            }
        }

        @Override
        public int hashCode()
        {
            if(!hcCalculated)
            {
                final int oddPrime = 461;
                int result = 73;

                for(E e : this)
                {
                    if(e != null)
                    {
                        result = result * oddPrime + e.hashCode();
                    }
                }

                hc = result;
                hcCalculated = true;
            }

            return hc;
        }

        public Sequence<E> clone()
        {
            return new InsertSequence<E>(position, sequence, inserted);
        }
    }

    private static final class DeleteSequence<E> implements Sequence<E>
    {
        private static final long serialVersionUID = 2L;

        private final int fromPosition;
        private final int toPosition;
        private final Sequence<E> sequence;
        private int hc;
        private boolean hcCalculated;
        private String ts;

        DeleteSequence(final int fromPosition, final int toPosition, final Sequence<E> sequence)
        {
            this.fromPosition = fromPosition;
            this.toPosition = toPosition;
            this.sequence = sequence;
        }

        public E get(final int index) throws IndexOutOfBoundsException
        {
            if(index < 0 || index >= length())
            {
                throw new IndexOutOfBoundsException();
            }
            else
            {
                if(index < fromPosition)
                {
                    return sequence.get(index);
                }
                else
                {
                    return sequence.get(index + toPosition - fromPosition);
                }
            }
        }

        public int length()
        {
            return sequence.length() - toPosition + fromPosition;
        }

        public Sequence<E> append(final Sequence<E> sequence) throws NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else if(sequence.length() == 0)
            {
                return this;
            }
            else
            {
                return new CompositeSequence<E>(this, sequence);
            }
        }

        public Sequence<E> getSubSequence(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else
            {
                return new SubSequence<E>(this, fromPosition, toPosition);
            }
        }

        public Sequence<E> insert(final int position, final Sequence<E> sequence) throws PositionOutOfBoundsException, NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else
            {
                if(position < 0 || position > length())
                {
                    throw new PositionOutOfBoundsException(position);
                }
                else if(position == 0)
                {
                    return sequence.append(this);
                }
                else if(position == length())
                {
                    return this.append(sequence);
                }
                else
                {
                    return new InsertSequence<E>(position, this, sequence);
                }
            }
        }

        public Sequence<E> delete(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(fromPosition == toPosition)
            {
                return this;
            }
            else
            {
                return new DeleteSequence<E>(fromPosition, toPosition, this);
            }
        }

        public int indexOf(final E e)
        {
            return indexOf(0, e);
        }

        public int indexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasNext())
                {
                    final int itPosition = it.getPosition();
                    final E x = it.next();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public int lastIndexOf(final E e)
        {
            return lastIndexOf(length(), e);
        }

        public int lastIndexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                final SequenceIterator<E> it = getIterator(position);

                while(it.hasPrevious())
                {
                    final E x = it.previous();
                    final int itPosition = it.getPosition();

                    if(x == null ? e == null : x.equals(e))
                    {
                        return itPosition;
                    }
                }

                return -1;
            }
        }

        public SequenceIterator<E> getIterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator(final int position) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                return new SequenceIterator<E>()
                {
                    private int pos;

                    {
                        pos = position;
                    }

                    public final boolean hasNext()
                    {
                        return pos != length();
                    }

                    public final E next()
                    {
                        if(pos == length())
                        {
                            throw new NoSuchElementException();
                        }
                        else if(pos < fromPosition)
                        {
                            return sequence.get(pos++);
                        }
                        else
                        {
                            return sequence.get(pos++ + toPosition - fromPosition);
                        }
                    }

                    public final void remove()
                    {
                        throw new UnsupportedOperationException();
                    }

                    public boolean hasPrevious()
                    {
                        return pos != 0;
                    }

                    public E previous() throws NoSuchElementException
                    {
                        if(pos == 0)
                        {
                            throw new NoSuchElementException();
                        }
                        else if(pos <= fromPosition)
                        {
                            return sequence.get(--pos);
                        }
                        else
                        {
                            return sequence.get(--pos + toPosition - fromPosition);
                        }
                    }

                    public int getPosition()
                    {
                        return pos;
                    }
                };
            }
        }

        public boolean isEmpty()
        {
            return fromPosition == 0 && toPosition == sequence.length();
        }

        public Iterator<E> iterator()
        {
            return getIterator(0);
        }

        @Override
        public String toString()
        {
            if(ts == null)
            {
                final StringBuilder sb = new StringBuilder();

                sb.append("[");

                final SequenceIterator<E> it = getIterator();

                while(it.hasNext())
                {
                    sb.append(it.next());

                    if(it.hasNext())
                    {
                        sb.append("][");
                    }
                }

                sb.append("]");

                ts = sb.toString();
            }

            return ts;
        }

        @Override
        public boolean equals(final Object o)
        {
            if(this == o)
            {
                return true;
            }
            else if(o == null || (!(o instanceof Sequence)))
            {
                return false;
            }
            else
            {
                final Sequence<?> s = (Sequence<?>)o;

                if(s.length() != length())
                {
                    return false;
                }
                else
                {
                    final Iterator<?> it1 = iterator();
                    final Iterator<?> it2 = s.iterator();

                    while(it1.hasNext() && it2.hasNext())
                    {
                        final Object se1 = it1.next();
                        final Object se2 = it2.next();

                        if(se1 == null ? se2 != null : !se1.equals(se2))
                        {
                            return false;
                        }
                    }

                    return true;
                }
            }
        }

        @Override
        public int hashCode()
        {
            if(!hcCalculated)
            {
                final int oddPrime = 461;
                int result = 73;

                for(E e : this)
                {
                    if(e != null)
                    {
                        result = result * oddPrime + e.hashCode();
                    }
                }

                hc = result;
                hcCalculated = true;
            }

            return hc;
        }

        public Sequence<E> clone()
        {
            return new DeleteSequence<E>(fromPosition, toPosition, sequence);
        }
    }

    private static final class ConstantSequence<E> implements Sequence<E>
    {
        private static final long serialVersionUID = 2L;

        private final E constant;
        private final int length;

        private int hc;
        private boolean hcCalculated;
        private String ts;

        ConstantSequence(final E constant, final int length)
        {
            this.constant = constant;
            this.length = length;
        }

        public E get(final int index) throws IndexOutOfBoundsException
        {
            if(index < 0 || index >= length())
            {
                throw new IndexOutOfBoundsException();
            }
            else
            {
                return constant;
            }
        }

        public int length()
        {
            return length;
        }

        public Sequence<E> append(final Sequence<E> sequence) throws NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else if(sequence.length() == 0)
            {
                return this;
            }
            else
            {
                return new CompositeSequence<E>(this, sequence);
            }
        }

        public Sequence<E> getSubSequence(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else
            {
                return new SubSequence<E>(this, fromPosition, toPosition);
            }
        }

        public Sequence<E> insert(final int position, final Sequence<E> sequence) throws PositionOutOfBoundsException, NullPointerException
        {
            if(sequence == null)
            {
                throw new NullPointerException();
            }
            else
            {
                if(position < 0 || position > length())
                {
                    throw new PositionOutOfBoundsException(position);
                }
                else if(position == 0)
                {
                    return sequence.append(this);
                }
                else if(position == length())
                {
                    return this.append(sequence);
                }
                else
                {
                    return new InsertSequence<E>(position, this, sequence);
                }
            }
        }

        public Sequence<E> delete(final int fromPosition, final int toPosition) throws PositionOutOfBoundsException
        {
            if(fromPosition < 0)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(toPosition > length())
            {
                throw new PositionOutOfBoundsException(toPosition);
            }
            else if(fromPosition > toPosition)
            {
                throw new PositionOutOfBoundsException(fromPosition);
            }
            else if(fromPosition == toPosition)
            {
                return this;
            }
            else
            {
                return new DeleteSequence<E>(fromPosition, toPosition, this);
            }
        }

        public int indexOf(final E e)
        {
            return e == constant ? 0 : -1;
        }

        public int indexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                if(position != length() && (constant == null ? e == null : constant.equals(e)))
                {
                    return position;
                }
                else
                {
                    return -1;
                }
            }
        }

        public int lastIndexOf(final E e)
        {
            return e == constant ? length() - 1 : -1;
        }

        public int lastIndexOf(final int position, final E e) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                if(position != 0 && (constant == null ? e == null : constant.equals(e)))
                {
                    return position - 1;
                }
                else
                {
                    return -1;
                }
            }
        }

        public boolean isEmpty()
        {
            return false;
        }

        public Iterator<E> iterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator()
        {
            return getIterator(0);
        }

        public SequenceIterator<E> getIterator(final int position) throws PositionOutOfBoundsException
        {
            if(position < 0 || position > length())
            {
                throw new PositionOutOfBoundsException(position);
            }
            else
            {
                return new SequenceIterator<E>()
                {
                    private int pos;

                    {
                        pos = position;
                    }

                    public boolean hasNext()
                    {
                        return pos != length();
                    }

                    public E next() throws NoSuchElementException
                    {
                        if(pos == length())
                        {
                            throw new NoSuchElementException();
                        }
                        else
                        {
                            pos++;
                            return constant;
                        }
                    }

                    public void remove()
                    {
                        throw new UnsupportedOperationException();
                    }

                    public boolean hasPrevious()
                    {
                        return pos != 0;
                    }

                    public E previous() throws NoSuchElementException
                    {
                        if(pos == 0)
                        {
                            throw new NoSuchElementException();
                        }
                        else
                        {
                            pos--;
                            return constant;
                        }
                    }

                    public int getPosition()
                    {
                        return pos;
                    }
                };
            }
        }

        @Override
        public String toString()
        {
            if(ts == null)
            {
                final StringBuilder sb = new StringBuilder();

                sb.append("[");

                final SequenceIterator<E> it = getIterator();

                while(it.hasNext())
                {
                    sb.append(it.next());

                    if(it.hasNext())
                    {
                        sb.append("][");
                    }
                }

                sb.append("]");

                ts = sb.toString();
            }

            return ts;
        }

        @Override
        public boolean equals(final Object o)
        {
            if(this == o)
            {
                return true;
            }
            else if(o == null || (!(o instanceof Sequence)))
            {
                return false;
            }
            else
            {
                final Sequence<?> s = (Sequence<?>)o;

                final Object e = s.get(0);

                return s.length() == length() && (e == null ? constant == null : e.equals(constant));
            }
        }

        @Override
        public int hashCode()
        {
            if(!hcCalculated)
            {
                final int oddPrime = 461;
                int result = 73;

                if(constant != null)
                {
                    result = result * oddPrime + constant.hashCode();
                }

                result = result * oddPrime + length();

                hc = result;
                hcCalculated = true;
            }

            return hc;
        }

        public Sequence<E> clone()
        {
            return new ConstantSequence<E>(constant, length);
        }
    }
}
