/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.framework.SequenceFactory.newSequence;
import java.lang.reflect.Method;
import static org.jtiger.framework.FixtureDescriptionFactory.newFixtureDescription;
import static org.jtiger.framework.FixtureNameFactory.newFixtureName;
import static org.jtiger.framework.TestDescriptionFactory.newTestDescription;
import static org.jtiger.framework.TestMethodCategoriesFactory.newTestMethodCategories;
import static org.jtiger.framework.TestNameFactory.newTestName;

final class TestResultFactory
{
    private TestResultFactory()
    {

    }

    static TestResult newTestResult(final TestResultType testResultType, final String message,
                                    final Throwable exception, final Long elapsedTime, final Method m) throws NullPointerException
    {
        if(m == null)
        {
            throw new NullPointerException();
        }

        return new TestResultImpl(testResultType, message, exception, elapsedTime, m);
    }

    private static final class TestResultImpl implements TestResult
    {
        private static final long serialVersionUID = 2L;

        private final TestResultType testResultType;
        private final String message;
        private final Throwable exception;
        private final Long elapsedTime;
        private final ReadOnlyArray<String> categories;
        private final String testName;
        private final String testDescription;
        private final String testMethodName;
        private final ReadOnlyArray<Class<?>> testMethodParameterTypes;
        private final String fixtureName;
        private final String fixtureDescription;
        private final Class<?> fixtureClass;

        TestResultImpl(final TestResultType testResultType, final String message, final Throwable exception,
                       final Long elapsedTime, final Method m)
        {
            this(testResultType,
                    message,
                    exception,
                    elapsedTime,
                    newTestMethodCategories().getTestMethodCategories(m),
                    newTestName().getTestName(m),
                    newTestDescription().getTestDescription(m),
                    m.getName(),
                    newSequence(m.getParameterTypes()),
                    newFixtureName().getFixtureName(m.getDeclaringClass()),
                    newFixtureDescription().getFixtureDescription(m.getDeclaringClass()),
                    m.getDeclaringClass());
        }

        private TestResultImpl(
                final TestResultType testResultType,
                final String message,
                final Throwable exception,
                final Long elapsedTime,
                final ReadOnlyArray<String> categories,
                final String testName,
                final String testDescription,
                final String testMethodName,
                final ReadOnlyArray<Class<?>> testMethodParameterTypes,
                final String fixtureName,
                final String fixtureDescription,
                final Class<?> fixtureClass)
        {
            this.testResultType = testResultType;
            this.message = message;
            this.exception = exception;
            this.elapsedTime = elapsedTime;
            this.categories = categories;
            this.testName = testName;
            this.testDescription = testDescription;
            this.testMethodName = testMethodName;
            this.testMethodParameterTypes = testMethodParameterTypes;
            this.fixtureName = fixtureName;
            this.fixtureDescription = fixtureDescription;
            this.fixtureClass = fixtureClass;
        }

        public TestResultType getTestResultType()
        {
            return testResultType;
        }

        public String getMessage()
        {
            return message;
        }

        public Throwable getException()
        {
            return exception;
        }

        public Long getElapsedTime()
        {
            return elapsedTime;
        }

        public ReadOnlyArray<String> getCategories()
        {
            return categories;
        }

        public String getTestName()
        {
            return testName;
        }

        public String getTestDescription()
        {
            return testDescription;
        }

        public String getTestMethodName()
        {
            return testMethodName;
        }

        public ReadOnlyArray<Class<?>> getTestMethodParameterTypes()
        {
            return testMethodParameterTypes;
        }

        public String getFixtureName()
        {
            return fixtureName;
        }

        public String getFixtureDescription()
        {
            return fixtureDescription;
        }

        public Class<?> getFixtureClass()
        {
            return fixtureClass;
        }

        @Override
        public String toString()
        {
            final StringBuilder sb = new StringBuilder();

            sb.append('[');
            sb.append(testResultType);
            sb.append("][");
            sb.append(message);
            sb.append("][");
            sb.append(exception);
            sb.append("][");
            sb.append(elapsedTime);
            sb.append("][");
            sb.append(categories);
            sb.append("][");
            sb.append(testName);
            sb.append("][");
            sb.append(testDescription);
            sb.append("][");
            sb.append(testMethodName);
            sb.append("][");
            sb.append(fixtureName);
            sb.append("][");
            sb.append(fixtureDescription);
            sb.append("][");
            sb.append(fixtureClass);
            sb.append(']');

            return sb.toString();
        }

        @Override
        public boolean equals(final Object o)
        {
            if(this == o)
            {
                return true;
            }

            if(o == null || o.getClass() != TestResultImpl.class)
            {
                return false;
            }

            final TestResultImpl r = (TestResultImpl)o;

            return (testResultType == r.testResultType) &&
                    (message == null ? r.message == null : message.equals(r.message)) &&
                    (exception == null ? r.exception == null : exception.equals(r.exception)) &&
                    (elapsedTime == null ? r.elapsedTime == null : elapsedTime.equals(r.elapsedTime)) &&
                    (categories == null ? r.categories == null : categories.equals(r.categories)) &&
                    (testName == null ? r.testName == null : testName.equals(r.testName)) &&
                    (testDescription == null ? r.testDescription == null : testDescription.equals(r.testDescription)) &&
                    (testMethodName == null ? r.testMethodName == null : testMethodName.equals(r.testMethodName)) &&
                    (fixtureName == null ? r.fixtureName == null : fixtureName.equals(r.fixtureName)) &&
                    (fixtureDescription == null ? r.fixtureDescription == null : fixtureDescription.equals(r.fixtureDescription)) &&
                    (fixtureClass == r.fixtureClass);
        }

        @Override
        public int hashCode()
        {
            final int oddPrime = 461;
            int result = 73;

            if(testResultType != null)
            {
                result = result * oddPrime + testResultType.hashCode();
            }

            if(message != null)
            {
                result = result * oddPrime + message.hashCode();
            }

            if(exception != null)
            {
                result = result * oddPrime + exception.hashCode();
            }

            if(elapsedTime != null)
            {
                result = result * oddPrime + elapsedTime.hashCode();
            }

            if(categories != null)
            {
                result = result * oddPrime + categories.hashCode();
            }

            if(testName != null)
            {
                result = result * oddPrime + testName.hashCode();
            }

            if(testDescription != null)
            {
                result = result * oddPrime + testDescription.hashCode();
            }

            if(testMethodName != null)
            {
                result = result * oddPrime + testMethodName.hashCode();
            }

            if(fixtureName != null)
            {
                result = result * oddPrime + fixtureName.hashCode();
            }

            if(fixtureDescription != null)
            {
                result = result * oddPrime + fixtureDescription.hashCode();
            }

            if(fixtureClass != null)
            {
                result = result * oddPrime + fixtureClass.hashCode();
            }

            return result;
        }
    }
}
