/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import static org.jtiger.framework.TestResultFactory.newTestResult;
import static org.jtiger.framework.TestResultType.SUCCESS;
import static org.jtiger.framework.TestResultType.FAILURE;
import static org.jtiger.framework.TestResultType.IGNORED_ANNOTATED;
import static org.jtiger.framework.TestResultType.IGNORED_CANNOT_INVOKE;
import static org.jtiger.framework.TestResultType.FAILURE_SETUP;
import static org.jtiger.framework.TestResultType.FAILURE_TEARDOWN;

final class TestRunnerFactory
{
    private TestRunnerFactory()
    {

    }

    static TestRunner newTestRunner()
    {
        return new TestRunnerImpl();
    }

    private static final class TestRunnerImpl implements TestRunner
    {
        TestRunnerImpl()
        {

        }

        public TestResult run(final Method m, final Object fixture, final SetUpTearDown sutd) throws NullPointerException
        {
            if(m == null || fixture == null || sutd == null)
            {
                throw new NullPointerException();
            }

            if(m.isAnnotationPresent(Ignore.class))
            {
                return newTestResult(IGNORED_ANNOTATED,
                        m.getAnnotation(Ignore.class).value(),  null, null, m);
            }

            if(m.getDeclaringClass().isAnnotationPresent(Ignore.class))
            {
                return newTestResult(IGNORED_ANNOTATED,
                        m.getDeclaringClass().getAnnotation(Ignore.class).value(),  null, null, m);
            }

            if(m.getParameterTypes().length != 0)
            {
                return newTestResult(IGNORED_CANNOT_INVOKE,
                        Messages.wrongMethodSignature(), null, null, m);
            }

            final ExpectException ee = m.getAnnotation(ExpectException.class);
            
            try
            {
                sutd.setUp(fixture);
            }
            catch(SetUpException e)
            {
                return newTestResult(FAILURE_SETUP, e.getMessage(), e, null, m);
            }

            try
            {
                final long t1 = System.currentTimeMillis();
                final Object ret = m.invoke(fixture);
                final long t2 = System.currentTimeMillis();

                m.setAccessible(true);

                final String message = ret == null ? null : ret.toString();

                return newTestResult(ee == null ? SUCCESS : FAILURE, message, null, t2 - t1, m);
            }
            catch(IllegalAccessException e)
            {
                return newTestResult(FAILURE, e.getMessage(), null, null, m);
            }
            catch(InvocationTargetException e)
            {
                final Throwable cause = e.getCause();

                if(ee != null)
                {
                    final Class<? extends Throwable> expected = ee.value();
                    final boolean subclass = ee.subclass();

                    if(subclass ? expected.isAssignableFrom(cause.getClass()) : expected == cause.getClass())
                    {
                        return newTestResult(SUCCESS, cause.getMessage(), cause, null, m);
                    }
                }

                return newTestResult(FAILURE, cause.getMessage(), cause, null, m);
            }
            finally
            {
                try
                {
                    sutd.tearDown(fixture);
                }
                catch(TearDownException e)
                {
                    return newTestResult(FAILURE_TEARDOWN, e.getMessage(), e, null, m);
                }
            }
        }
    }
}
