/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework.junit;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import org.jtiger.framework.SetUpException;
import org.jtiger.framework.SetUpTearDown;
import org.jtiger.framework.TearDownException;

/**
 * This implementation of {@link SetUpTearDown} is intended to be used to execute test cases that have been written
 * using <a href="http://www.junit.org/" target="_blank">the JUnit Test Framework</a>. The implementation is passed to the
 * framework by returning an instance from {@link org.jtiger.framework.FixturesRunnerConfig#getSutdClass()} or by
 * passing the qualified class name as {@link org.jtiger.framework.FixturesRunnerMain#ARG_SUTD_CLASS the appropriate
 * argument} on the command line to {@link org.jtiger.framework.FixturesRunnerMain#main(String[]) the framework main
 * method}.
 *
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class JUnitSetUpTearDown implements SetUpTearDown
{
    private static final String SETUP_METHOD = "setUp";
    private static final String TEARDOWN_METHOD = "tearDown";

    /**
     * Create a default instance of <tt>JUnitSetUpTearDown</tt>.
     */
    public JUnitSetUpTearDown()
    {

    }

    /**
     * Perform the setting up of <a href="http://www.junit.org/" target="_blank">JUnit Test Framework</a> test cases.
     *
     * @param fixture The test fixture to set up.
     * @throws SetUpException If a <code>setUp</code> method cannot be found or if an error occurs during reflective
     * invocation of the JUnit <code>setUp</code> method.
     */
    public void setUp(final Object fixture) throws SetUpException
    {
        final Method m = JUnitSetUpTearDown.getSetUpMethod(fixture.getClass());

        if(m == null)
        {
            throw new SetUpException();
        }
        else
        {
            if(!m.isAccessible())
            {
                m.setAccessible(true);
            }

            try
            {
                m.invoke(fixture);
            }
            catch(IllegalAccessException e)
            {
                throw new SetUpException(e.getMessage(), e);
            }
            catch(InvocationTargetException e)
            {
                throw new SetUpException(e.getMessage(), e);
            }
        }
    }

    /**
     * Perform the tearing down of <a href="http://www.junit.org/" target="_blank">JUnit Test Framework</a> test cases.
     *
     * @param fixture The test fixture to tear down.
     * @throws TearDownException If a <code>tearDown</code> method cannot be found or if an error occurs during
     * reflective invocation of the JUnit <code>tearDown</code> method.
     */
    public void tearDown(final Object fixture) throws TearDownException
    {
        final Method m = JUnitSetUpTearDown.getTearDownMethod(fixture.getClass());

        if(m == null)
        {
            throw new TearDownException();
        }
        else
        {
            if(!m.isAccessible())
            {
                m.setAccessible(true);
            }

            try
            {
                m.invoke(fixture);
            }
            catch(IllegalAccessException e)
            {
                throw new TearDownException(e.getMessage(), e);
            }
            catch(InvocationTargetException e)
            {
                throw new TearDownException(e.getMessage(), e);
            }
        }
    }

    private static Method getSetUpMethod(Class<?> c)
    {
        while(c != null)
        {
            final Method[] methods = c.getDeclaredMethods();

            for(Method m : methods)
            {
                if(m.getName().equals(JUnitSetUpTearDown.SETUP_METHOD) &&
                        m.getParameterTypes().length == 0)
                {
                    return m;
                }
            }

            c = c.getSuperclass();
        }

        return null;
    }

    private static Method getTearDownMethod(Class<?> c)
    {
        while(c != null)
        {
            final Method[] methods = c.getDeclaredMethods();

            for(Method m : methods)
            {
                if(m.getName().equals(JUnitSetUpTearDown.TEARDOWN_METHOD) &&
                        m.getParameterTypes().length == 0)
                {
                    return m;
                }
            }

            c = c.getSuperclass();
        }

        return null;
    }
}
