package org.jtiger.report.failure;

import static org.jtiger.framework.TestResultType.FAILURE;
import static org.jtiger.framework.TestResultType.FAILURE_SETUP;
import static org.jtiger.framework.TestResultType.FAILURE_TEARDOWN;
import org.jtiger.framework.FixtureResultsHandler;
import org.jtiger.framework.FixtureResults;
import org.jtiger.framework.ReadOnlyArray;
import org.jtiger.framework.FixtureResult;
import org.jtiger.framework.TestResult;
import org.jtiger.framework.TestResultType;

/**
 * A result handler that outputs to <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/System.html#out">the standard output stream</a>
 * information regarding test failures. A test failure is one that has a result type of {@link TestResultType#FAILURE},
 * {@link TestResultType#FAILURE_SETUP} or {@link TestResultType#FAILURE_TEARDOWN}.
 * <br/>
 * This class was inspired by suggestions from Pierre Legall.
 *
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class FailureFixtureResultsHandler implements FixtureResultsHandler
{
    /**
     * Create a default <tt>FailureFixtureResultsHandler</tt>.
     */
    public FailureFixtureResultsHandler()
    {

    }

    /**
     * Outputs test failure information to
     * <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/System.html#out">the standard output stream</a>.
     *
     * @param results The test run results to produce the failure output of.
     * @param params This parameter is not used.
     */
    public void handleResult(final FixtureResults results, final ReadOnlyArray<String> params)
    {
        for(FixtureResult fr : results)
        {
            boolean first = true;

            for(TestResult tr : fr)
            {
                final TestResultType trt = tr.getTestResultType();

                if (trt == FAILURE || trt == FAILURE_SETUP || trt == FAILURE_TEARDOWN)
                {
                    if(first)
                    {
                        first = false;
                    }
                    else
                    {
                        System.out.println();
                    }

                    final Throwable t = tr.getException();

                    if(t == null)
                    {
                        System.out.println(t);
                        System.out.println(new StringBuilder("  at ")
                                .append(tr.getFixtureClass())
                                .append('.')
                                .append(tr.getTestMethodName()));
                    }
                    else
                    {
                        System.out.println(t);

                        final StackTraceElement[] stes = t.getStackTrace();

                        for (StackTraceElement ste : stes)
                        {
                            System.out.println(new StringBuilder("  at ").append(ste));
                        }
                    }
                }
            }
        }
    }
}
