/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.report.html;

import java.beans.XMLEncoder;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.jtiger.framework.FixtureResultsHandlerException;
import org.jtiger.framework.ReadOnlyArray;

final class OverallHtmlFileWriterFactory
{
    private static final String XSL_RESOURCE = "/org/jtiger/report/xsl/overall.xsl";
    private static final String FILENAME = "overall.html";
    private static final String PARAM_BUILD_TIME = "buildTime";
    private static final String PARAM_SUCCESS = "success";
    private static final String PARAM_FAILURE = "failure";
    private static final String PARAM_FAILURE_SETUP = "failure.setup";
    private static final String PARAM_FAILURE_TEARDOWN = "failure.teardown";
    private static final String PARAM_IGNORED_ANNOTATED = "ignored.annotated";
    private static final String PARAM_IGNORED_CANNOT_INVOKE = "ignored.cannot.invoke";

    private OverallHtmlFileWriterFactory()
    {

    }

    static OverallHtmlFileWriter newOverallHtmlFileWriter()
    {
        return new OverallHtmlFileWriterImpl();
    }

    private static final class OverallHtmlFileWriterImpl implements OverallHtmlFileWriter
    {
        OverallHtmlFileWriterImpl()
        {

        }

        public void writeOverallHtml(final ReadOnlyArray<OverallResult> results,
                                     final File destination,
                                     final FilenamePolicy policy,
                                     final OverallTotal total,
                                     final String buildTime)
                throws FixtureResultsHandlerException
        {
            final ByteArrayOutputStream out = new ByteArrayOutputStream();
            final XMLEncoder encoder = new XMLEncoder(out);

            for(OverallResult result : results)
            {
                final Object bean = new OverallResultBeanImpl(result.getName(),
                        policy.nextFilename(), result.getDescription(), result.getElapsedTime(), result.getResult());
                encoder.writeObject(bean);
            }

            encoder.close();

            InputStream in = null;

            try
            {
                in = getClass().getResourceAsStream(XSL_RESOURCE);
                final Transformer t = TransformerFactory.newInstance().newTransformer(new StreamSource(in));
                t.setParameter(PARAM_BUILD_TIME, buildTime);
                t.setParameter(PARAM_SUCCESS, total.getSuccess());
                t.setParameter(PARAM_FAILURE, total.getFailure());
                t.setParameter(PARAM_FAILURE_SETUP, total.getFailureSetUp());
                t.setParameter(PARAM_FAILURE_TEARDOWN, total.getFailureTearDown());
                t.setParameter(PARAM_IGNORED_ANNOTATED, total.getIgnoredAnnotated());
                t.setParameter(PARAM_IGNORED_CANNOT_INVOKE, total.getIgnoredCannotInvoke());
                OutputStream o = null;

                try
                {
                    final File f = new File(destination, FILENAME);
                    o = new FileOutputStream(f);
                    final StreamResult r = new StreamResult(o);
                    t.transform(new StreamSource(new ByteArrayInputStream(out.toByteArray())), r);
                }
                catch(FileNotFoundException e)
                {
                    throw new FixtureResultsHandlerException(e.getMessage(), e);
                }
                finally
                {
                    if(o != null)
                    {
                        try
                        {
                            o.close();
                        }
                        catch(IOException e)
                        {
                            throw new FixtureResultsHandlerException(e.getMessage(), e);
                        }
                    }
                }
            }
            catch(TransformerConfigurationException e)
            {
                throw new FixtureResultsHandlerException(e.getMessage(), e);
            }
            catch(TransformerException e)
            {
                throw new FixtureResultsHandlerException(e.getMessage(), e);
            }
            finally
            {
                if(in != null)
                {
                    try
                    {
                        in.close();
                    }
                    catch(IOException e)
                    {
                        throw new FixtureResultsHandlerException(e.getMessage(), e);
                    }
                }
            }
        }
    }
}
