/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.report.xml;

import java.beans.Encoder;
import java.beans.Expression;
import java.beans.PersistenceDelegate;
import java.beans.XMLEncoder;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.File;
import org.jtiger.framework.FixtureResults;
import org.jtiger.framework.FixtureResultsHandler;
import org.jtiger.framework.FixtureResultsHandlerException;
import org.jtiger.framework.ReadOnlyArray;

/**
 * A result handler that produces a XML report of test run results.
 * The report is written to a file which is passed in the parameters to
 * {@link #handleResult(org.jtiger.framework.FixtureResults, org.jtiger.framework.ReadOnlyArray) the handleResult method}.
 *
 * @author Tony Morris
 * @version 2.1<br/>
 *          <i>Build Number 0376</i><br/>
 *          <i>Build Time 2006-07-28 01:50.16.218 CET (GMT + 1)</i>
 */
public final class XmlFixtureResultsHandler implements FixtureResultsHandler
{
    /**
     * Create a default <tt>XmlFixtureResultsHandler</tt>.
     */
    public XmlFixtureResultsHandler()
    {

    }

    /**
     * Write a XML report of the given test run results to a file which is passed in the parameters.
     * If the <tt>params</tt> are empty, the current directory is used to write the report to in a file called
     * 'result.xml'.
     *
     * @param results The test run results to produce the XML report of.
     * @param params The parameters that contain at least one parameter which is the file to write the XML report to
     * otherwise the current directory is used.
     * @throws FixtureResultsHandlerException If an error occurs while writing the XML report.
     */
    public void handleResult(final FixtureResults results, final ReadOnlyArray<String> params) throws FixtureResultsHandlerException
    {
        final File destination;

        if(params == null || params.length() == 0)
        {
            destination = new File("result.xml");
        }
        else
        {
            destination = new File(params.get(0));
        }

        XMLEncoder encoder = null;

        try
        {
            encoder = new XMLEncoder(new FileOutputStream(destination));
            encoder.setPersistenceDelegate(StackTraceElement.class, new PersistenceDelegate()
            {
                public Expression instantiate(final Object oldInstance, final Encoder out)
                {
                    if(!(oldInstance instanceof StackTraceElement))
                    {
                        throw new IllegalStateException();
                    }

                    final StackTraceElement e = (StackTraceElement)oldInstance;

                    return new Expression(oldInstance, StackTraceElement.class, "new",
                            new Object[]{e.getClassName(), e.getMethodName(), e.getFileName(), e.getLineNumber()});
                }                            
            });

            encoder.writeObject(new FixtureResultsBeanImpl(results));
        }
        catch(FileNotFoundException e)
        {
            throw new FixtureResultsHandlerException(e.getMessage(), e);
        }
        finally
        {
            if(encoder != null)
            {
                encoder.close();
            }
        }
    }
}
