/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import org.jtiger.framework.Category;
import org.jtiger.framework.ExpectException;
import org.jtiger.framework.Fixture;
import org.jtiger.framework.Test;
import static org.jtiger.assertion.Reflection.assertHasDeclaredConstructor;
import static org.jtiger.assertion.Modifier.assertPrivate;
import static org.jtiger.assertion.Basic.assertTrue;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.EqualsMethodContractTesterFactory.newEqualsMethodContractTester;

@Fixture(value = "EqualsMethodContractTesterFactory", description = "Performs Unit Tests on org.jtiger.assertion.EqualsMethodContractTesterFactory")
@Category("org.jtiger.assertion.EqualsMethodContractTesterFactory")
public final class TestEqualsMethodContractTesterFactory
{
    public TestEqualsMethodContractTesterFactory()
    {

    }

    @Test
    public void constructor() throws InstantiationException, IllegalAccessException, InvocationTargetException
    {
        final Constructor<?> c = assertHasDeclaredConstructor(EqualsMethodContractTesterFactory.class, null);
        assertPrivate(c);
        assertFalse(c.isAccessible());
        c.setAccessible(true);
        c.newInstance();
    }

    @Test
    @ExpectException(NullPointerException.class)
    public void factoryMethod()
    {
        newEqualsMethodContractTester(null);
    }

    @Test
    public void testIsUnequalToNull1()
    {
        final ObjectFactory<String> factory = new ObjectFactory<String>()
        {
            public String newInstanceX()
            {
                return new String("x");
            }

            public String newInstanceY()
            {
                return new String("y");
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertTrue(tester.isUnequalToNull());
    }

    @Test
    public void testIsUnequalToNull2()
    {
        final ObjectFactory<NotUnequalToNull> factory = new ObjectFactory<NotUnequalToNull>()
        {
            public NotUnequalToNull newInstanceX()
            {
                return new NotUnequalToNull();
            }

            public NotUnequalToNull newInstanceY()
            {
                return new NotUnequalToNull();
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertFalse(tester.isUnequalToNull());
    }

    @Test
    public void testIsConsistentAcrossInvocations1()
    {
        final ObjectFactory<String> factory = new ObjectFactory<String>()
        {
            public String newInstanceX()
            {
                return new String("x");
            }

            public String newInstanceY()
            {
                return new String("y");
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertTrue(tester.isConsistentAcrossInvocations());
    }

    @Test
    public void testIsConsistentAcrossInvocations2()
    {
        final ObjectFactory<NotUnequalToNull> factory = new ObjectFactory<NotUnequalToNull>()
        {
            public NotUnequalToNull newInstanceX()
            {
                return new NotUnequalToNull();
            }

            public NotUnequalToNull newInstanceY()
            {
                return new NotUnequalToNull();
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertFalse(tester.isConsistentAcrossInvocations());
    }

    @Test
    public void testIsConsistentAcrossInvocations5()
    {
        final ObjectFactory<NotUnequalToUnequalInstance> factory = new ObjectFactory<NotUnequalToUnequalInstance>()
        {
            public NotUnequalToUnequalInstance newInstanceX()
            {
                return new NotUnequalToUnequalInstance();
            }

            public NotUnequalToUnequalInstance newInstanceY()
            {
                return new NotUnequalToUnequalInstance();
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertFalse(tester.isConsistentAcrossInvocations());
    }

    @Test
    public void testIsReflexive1()
    {
        final ObjectFactory<String> factory = new ObjectFactory<String>()
        {
            public String newInstanceX()
            {
                return new String("x");
            }

            public String newInstanceY()
            {
                return new String("y");
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertTrue(tester.isReflexive());
    }

    @Test
    public void testIsReflexive2()
    {
        final ObjectFactory<NotReflexive> factory = new ObjectFactory<NotReflexive>()
        {
            public NotReflexive newInstanceX()
            {
                return new NotReflexive();
            }

            public NotReflexive newInstanceY()
            {
                return new NotReflexive();
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertFalse(tester.isReflexive());
    }

    @Test
    public void testIsSymmetric1()
    {
        final ObjectFactory<String> factory = new ObjectFactory<String>()
        {
            public String newInstanceX()
            {
                return new String("x");
            }

            public String newInstanceY()
            {
                return new String("y");
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertTrue(tester.isSymmetric());
    }

    @Test
    public void testIsSymmetric2()
    {
        final ObjectFactory<NotSymmetric> factory = new ObjectFactory<NotSymmetric>()
        {
            public NotSymmetric newInstanceX()
            {
                return new NotSymmetric();
            }

            public NotSymmetric newInstanceY()
            {
                return new NotSymmetric();
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertFalse(tester.isSymmetric());
    }

    @Test
    public void testIsTransitive1()
    {
        final ObjectFactory<String> factory = new ObjectFactory<String>()
        {
            public String newInstanceX()
            {
                return new String("x");
            }

            public String newInstanceY()
            {
                return new String("y");
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertTrue(tester.isTransitive());
    }

    @Test
    public void testIsTransitive2()
    {
        final ObjectFactory<NotTransitive> factory = new ObjectFactory<NotTransitive>()
        {
            public NotTransitive newInstanceX()
            {
                return new NotTransitive();
            }

            public NotTransitive newInstanceY()
            {
                return new NotTransitive();
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertFalse(tester.isTransitive());
    }

    @Test
    public void testIsUnequalToUnequalInstance1()
    {
        final ObjectFactory<String> factory = new ObjectFactory<String>()
        {
            public String newInstanceX()
            {
                return new String("x");
            }

            public String newInstanceY()
            {
                return new String("y");
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertTrue(tester.isUnequalToUnequalInstance());
    }

    @Test
    public void testIsUnequalToUnequalInstance2()
    {
        final ObjectFactory<NotUnequalToUnequalInstance> factory = new ObjectFactory<NotUnequalToUnequalInstance>()
        {
            public NotUnequalToUnequalInstance newInstanceX()
            {
                return new NotUnequalToUnequalInstance();
            }

            public NotUnequalToUnequalInstance newInstanceY()
            {
                return new NotUnequalToUnequalInstance();
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertFalse(tester.isUnequalToUnequalInstance());
    }

    @Test
    public void testIsUnequalToArbitraryType()
    {
        final ObjectFactory<String> factory = new ObjectFactory<String>()
        {
            public String newInstanceX()
            {
                return new String("x");
            }

            public String newInstanceY()
            {
                return new String("y");
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertTrue(tester.isUnequalToArbitraryType());
    }

    @Test
    public void testFillsContract1()
    {
        final ObjectFactory<String> factory = new ObjectFactory<String>()
        {
            public String newInstanceX()
            {
                return new String("x");
            }

            public String newInstanceY()
            {
                return new String("y");
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertTrue(tester.fillsContract());
    }

    @Test
    public void testFillsContract2()
    {
        final ObjectFactory<NotReflexive> factory = new ObjectFactory<NotReflexive>()
        {
            public NotReflexive newInstanceX()
            {
                return new NotReflexive();
            }

            public NotReflexive newInstanceY()
            {
                return new NotReflexive();
            }
        };

        final EqualsMethodContractTester tester = newEqualsMethodContractTester(factory);
        assertFalse(tester.fillsContract());
    }

    private static final class NotUnequalToNull
    {
        @Override
        public boolean equals(final Object o)
        {
            if(o == null)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }

    private static final class NotReflexive
    {
        @Override
        public boolean equals(final Object o)
        {
            return !(o == null || this == o);
        }
    }

    private static final class NotSymmetric
    {
        private static NotSymmetric o1;
        private static NotSymmetric o2;

        @Override
        public boolean equals(final Object o)
        {
            if(o == null)
            {
                return false;
            }
            else if(o1 == null || o2 == null)
            {
                o1 = this;
                o2 = (NotSymmetric)o;
                return true;
            }
            else
            {
                return !(this == o2 && o == o1);
            }
        }
    }

    private static final class NotTransitive
    {
        private static NotTransitive o1;
        private static NotTransitive o2;

        private NotTransitive()
        {

        }

        @Override
        public boolean equals(final Object o)
        {
            if(o == null)
            {
                return false;
            }
            else if(this == o)
            {
                return true;
            }
            else if(o1 == null || o2 == null && new Throwable().getStackTrace()[0].getMethodName().equals("isTransitive"))
            {
                o1 = this;
                o2 = (NotTransitive)o;
                return true;
            }
            else
            {
                return !(this == NotTransitive.o1 && o != NotTransitive.o2);
            }
        }
    }

    private static final class NotUnequalToUnequalInstance
    {
        private NotUnequalToUnequalInstance()
        {

        }

        @Override
        public boolean equals(final Object o)
        {
            return o != null;
        }
    }
}
