/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.assertion;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import org.jtiger.framework.Category;
import org.jtiger.framework.Fixture;
import org.jtiger.framework.SetUp;
import org.jtiger.framework.Test;
import static org.jtiger.assertion.Basic.assertTrue;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Reflection.assertHasDeclaredConstructor;
import static org.jtiger.assertion.Reflection.assertHasDeclaredMethod;
import static org.jtiger.assertion.Reflection.assertHasDeclaredField;
import static org.jtiger.assertion.Modifier.assertPrivate;
import static org.jtiger.assertion.ModifierTesterFactory.newModifierTester;

@Fixture(value = "ModifierTesterFactory", description = "Performs Unit Tests on org.jtiger.assertion.ModifierTesterFactory")
@Category("org.jtiger.assertion.ModifierTesterFactory")
public final class TestModifierTesterFactory
{
    private ModifierTester tester;

    public TestModifierTesterFactory()
    {

    }

    @SetUp
    public void SetUpModifierTester()
    {
        tester = newModifierTester();
    }

    @Test
    public void testConstructor() throws InstantiationException, IllegalAccessException, InvocationTargetException
    {
        final Constructor<?> c = assertHasDeclaredConstructor(ModifierTesterFactory.class, null);
        assertPrivate(c);
        assertFalse(c.isAccessible());
        c.setAccessible(true);
        c.newInstance();
    }

    @Test
    public void testIsAbstract()
    {
        final Class<?> c = AbstractClass.class;
        assertTrue(tester.isAbstract(c));
        final Method m = assertHasDeclaredMethod(c, "abstractMethod", null);
        assertTrue(tester.isAbstract(m));
    }

    @Test
    public void testIsFinal()
    {
        final Class<?> c = FinalClass.class;
        assertTrue(tester.isFinal(c));
        final Method m = assertHasDeclaredMethod(c, "finalMethod", null);
        assertTrue(tester.isFinal(m));
        final Field f = assertHasDeclaredField(c, "finalField");
        assertTrue(tester.isFinal(f));
    }

    @Test
    public void testIsNative()
    {
        final Class<?> c = NativeClass.class;
        final Method m = assertHasDeclaredMethod(c, "nativeMethod", null);
        assertTrue(tester.isNative(m));
    }

    @Test
    public void testIsPrivate()
    {
        final Class<?> c = PrivateClass.class;
        assertTrue(tester.isPrivate(c));
        final Constructor<?> ctor = assertHasDeclaredConstructor(PrivateConstructor.class, null);
        assertTrue(tester.isPrivate(ctor));
        final Method m = assertHasDeclaredMethod(c, "m", null);
        assertTrue(tester.isPrivate(m));
        final Field f = assertHasDeclaredField(c, "f");
        assertTrue(tester.isPrivate(f));
    }

    @Test
    public void testIsProtected()
    {
        final Class<?> c = ProtectedClass.class;
        assertTrue(tester.isProtected(c));
        final Constructor<?> ctor = assertHasDeclaredConstructor(ProtectedClass.class, null);
        assertTrue(tester.isProtected(ctor));
        final Method m = assertHasDeclaredMethod(c, "m", null);
        assertTrue(tester.isProtected(m));
        final Field f = assertHasDeclaredField(c, "f");
        assertTrue(tester.isProtected(f));
    }

    @Test
    public void testIsPublic()
    {
        final Class<?> c = PublicClass.class;
        assertTrue(tester.isPublic(c));
        final Constructor<?> ctor = assertHasDeclaredConstructor(PublicClass.class, null);
        assertTrue(tester.isPublic(ctor));
        final Method m = assertHasDeclaredMethod(c, "m", null);
        assertTrue(tester.isPublic(m));
        final Field f = assertHasDeclaredField(c, "f");
        assertTrue(tester.isPublic(f));
    }

    @Test
    public void testIsDefaultAccess()
    {
        final Class<?> c = DefaultAccessClass.class;
        assertTrue(tester.isDefaultAccess(c));
        final Constructor<?> ctor = assertHasDeclaredConstructor(DefaultAccessClass.class, null);
        assertTrue(tester.isDefaultAccess(ctor));
        final Method m = assertHasDeclaredMethod(c, "m", null);
        assertTrue(tester.isDefaultAccess(m));
        final Field f = assertHasDeclaredField(c, "f");
        assertTrue(tester.isDefaultAccess(f));

        final Class<?> c2 = PrivateClass.class;
        assertFalse(tester.isDefaultAccess(c2));
        final Constructor<?> ctor2 = assertHasDeclaredConstructor(PrivateClass.class, null);
        assertFalse(tester.isDefaultAccess(ctor2));
        final Method m2 = assertHasDeclaredMethod(c2, "m", null);
        assertFalse(tester.isDefaultAccess(m2));
        final Field f2 = assertHasDeclaredField(c2, "f");
        assertFalse(tester.isDefaultAccess(f2));

        final Class<?> c3 = ProtectedClass.class;
        assertFalse(tester.isDefaultAccess(c3));
        final Constructor<?> ctor3 = assertHasDeclaredConstructor(ProtectedClass.class, null);
        assertFalse(tester.isDefaultAccess(ctor3));
        final Method m3 = assertHasDeclaredMethod(c3, "m", null);
        assertFalse(tester.isDefaultAccess(m3));
        final Field f3 = assertHasDeclaredField(c3, "f");
        assertFalse(tester.isDefaultAccess(f3));

        final Class<?> c4 = PublicClass.class;
        assertFalse(tester.isDefaultAccess(c4));
        final Constructor<?> ctor4 = assertHasDeclaredConstructor(PublicClass.class, null);
        assertFalse(tester.isDefaultAccess(ctor4));
        final Method m4 = assertHasDeclaredMethod(c4, "m", null);
        assertFalse(tester.isDefaultAccess(m4));
        final Field f4 = assertHasDeclaredField(c4, "f");
        assertFalse(tester.isDefaultAccess(f4));
    }

    @Test
    public void testIsStatic()
    {
        final Class<?> c = StaticClass.class;
        assertTrue(tester.isStatic(c));
        final Method m = assertHasDeclaredMethod(c, "m", null);
        assertTrue(tester.isStatic(m));
        final Field f = assertHasDeclaredField(c, "f");
        assertTrue(tester.isStatic(f));
    }

    @Test
    public void testIsStrictfp()
    {
        final Class<?> c = StrictfpClass.class;
        assertTrue(tester.isStrictfp(c));
    }

    @Test
    public void testIsSynchronized()
    {
        final Class<?> c = SynchronizedClass.class;
        final Method m = assertHasDeclaredMethod(c, "m", null);
        assertTrue(tester.isSynchronized(m));
    }

    @Test
    public void testIsTransient()
    {
        final Class<?> c = TransientClass.class;
        final Field f = assertHasDeclaredField(c, "f");
        assertTrue(tester.isTransient(f));
    }

    @Test
    public void testIsVolatile()
    {
        final Class<?> c = VolatileClass.class;
        final Field f = assertHasDeclaredField(c, "f");
        assertTrue(tester.isVolatile(f));
    }

    abstract class AbstractClass
    {
        abstract void abstractMethod();
    }

    private final class FinalClass
    {
        final int finalField = 7;

        final void finalMethod()
        {

        }
    }

    private static final class NativeClass
    {
        native void nativeMethod();
    }

    private static final class PrivateClass
    {
        private int f;

        private PrivateClass()
        {
            m();
        }

        private int m()
        {
            f = 7;
            return f;
        }
    }

    protected static final class ProtectedClass
    {
        protected int f;

        protected ProtectedClass()
        {

        }

        protected final void m()
        {

        }
    }

    static final class DefaultAccessClass
    {
        int f;

        DefaultAccessClass()
        {

        }

        final void m()
        {

        }
    }

    public static final class PublicClass
    {
        public int f;

        public PublicClass()
        {

        }

        public final void m()
        {

        }
    }

    private static final class StaticClass
    {
        static int f;

        static void m()
        {

        }
    }

    private static final strictfp class StrictfpClass
    {

    }

    private static final class SynchronizedClass
    {
        public synchronized void m()
        {

        }
    }

    private static final class TransientClass
    {
        transient int f;
    }

    private static final class VolatileClass
    {
        volatile int f;
    }

    static final class PrivateConstructor
    {
        private PrivateConstructor()
        {

        }
    }
}

