/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertTrue;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertNotSame;
import static org.jtiger.assertion.Serialization.assertSerializes;
import static org.jtiger.assertion.Serialization.assertSerializesEqual;
import static org.jtiger.assertion.ObjectFactoryContract.assertObjectFactoryFillsContract;
import static org.jtiger.assertion.EqualsMethodContract.assertEqualsMethodFillsContract;
import static org.jtiger.assertion.HashCodeMethodContract.assertHashCodeMethodFillsContract;
import org.jtiger.assertion.AssertionException;
import org.jtiger.assertion.ObjectFactory;
import java.util.Iterator;

final class SequenceTesterFactory
{
    private SequenceTesterFactory() throws UnsupportedOperationException
    {
        throw new UnsupportedOperationException();
    }

    public static <E> SequenceTester<E> newSequenceTester()
    {
        return new SequenceTesterImpl<E>();
    }

    private static final class SequenceTesterImpl<E> implements SequenceTester<E>
    {
        SequenceTesterImpl()
        {

        }

        public void serialization(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            assertSerializes(creator.createSequence(7, 8, 9));
            assertSerializesEqual(creator.createSequence(7, 8, 9));
            assertSerializes(creator.createSequence(7, 8, 9)
                    .append(creator.createSequence(10, 11, 12)));
            assertSerializesEqual(creator.createSequence(7, 8, 9)
                    .append(creator.createSequence(10, 11, 12)));
            assertSerializes(creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(1, 5));
            assertSerializesEqual(creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(1, 5));
            assertSerializes(creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(0, 5));
            assertSerializesEqual(creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(0, 5));
            assertSerializes(creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(2, 6));
            assertSerializesEqual(creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(2, 6));
        }

        public void get(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> s1 = creator.createSequence(7, 8, 9);
            assertEqual(7, s1.get(0));
            assertEqual(8, s1.get(1));
            assertEqual(9, s1.get(2));

            final Sequence<Integer> s2 = creator.createSequence(7, 8, 9)
                    .append(creator.createSequence(10, 11, 12));
            assertEqual(7, s2.get(0));
            assertEqual(8, s2.get(1));
            assertEqual(9, s2.get(2));
            assertEqual(10, s2.get(3));
            assertEqual(11, s2.get(4));
            assertEqual(12, s2.get(5));

            final Sequence<Integer> s3 = creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(1, 5);
            assertEqual(8, s3.get(0));
            assertEqual(9, s3.get(1));
            assertEqual(10, s3.get(2));
            assertEqual(11, s3.get(3));

            final Sequence<Integer> s4 = creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(0, 5);
            assertEqual(7, s4.get(0));
            assertEqual(8, s4.get(1));
            assertEqual(9, s4.get(2));
            assertEqual(10, s4.get(3));
            assertEqual(11, s4.get(4));

            final Sequence<Integer> s5 = creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(2, 6);
            assertEqual(9, s5.get(0));
            assertEqual(10, s5.get(1));
            assertEqual(11, s5.get(2));
            assertEqual(12, s5.get(3));
        }

        public void length(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            assertEqual(0, creator.createSequence(new Integer[0]).length());
            assertEqual(1, creator.createSequence(1).length());
            assertEqual(3, creator.createSequence(7, 8, 9).length());
            assertEqual(6, creator.createSequence(7, 8, 9, 10, 11, 12).length());
            assertEqual(6, creator.createSequence(7, 8, 9).append(creator.createSequence(10, 11, 12)).length());
            assertEqual(4, creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(1, 5).length());
            assertEqual(5, creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(0, 5).length());
            assertEqual(4, creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(2, 6).length());
        }

        public void append(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> s1 = creator.createSequence(7, 8, 9).append(creator.createSequence(10, 11, 12));
            assertEqual(6, s1.length());
            assertEqual(7, s1.get(0));
            assertEqual(8, s1.get(1));
            assertEqual(9, s1.get(2));
            assertEqual(10, s1.get(3));
            assertEqual(11, s1.get(4));
            assertEqual(12, s1.get(5));

            final Sequence<Integer> s2 = creator.createSequence(7, 8, 9)
                    .append(creator.createSequence(10, 11, 12))
                    .append(creator.createSequence(13, 14, 15));
            assertEqual(9, s2.length());
            assertEqual(7, s2.get(0));
            assertEqual(8, s2.get(1));
            assertEqual(9, s2.get(2));
            assertEqual(10, s2.get(3));
            assertEqual(11, s2.get(4));
            assertEqual(12, s2.get(5));
            assertEqual(13, s2.get(6));
            assertEqual(14, s2.get(7));
            assertEqual(15, s2.get(8));

            final Sequence<Integer> s3 = creator.createSequence(7, 8, 9)
                    .append(creator.createSequence(10, 11, 12))
                    .getSubSequence(1, 5)
                    .append(creator.createSequence(13, 14, 15, 16, 17, 18))
                    .getSubSequence(3, 8);
            assertEqual(5, s3.length());
            assertEqual(11, s3.get(0));
            assertEqual(13, s3.get(1));
            assertEqual(14, s3.get(2));
            assertEqual(15, s3.get(3));
            assertEqual(16, s3.get(4));

            final Sequence<Integer> s4 = creator.createSequence(7, 8, 9)
                    .append(creator.createSequence(new Integer[0]))
                    .append(creator.createSequence(10, 11, 12));
            assertEqual(6, s4.length());
            assertEqual(7, s4.get(0));
            assertEqual(8, s4.get(1));
            assertEqual(9, s4.get(2));
            assertEqual(10, s4.get(3));
            assertEqual(11, s4.get(4));
            assertEqual(12, s4.get(5));

            final Sequence<Integer> s5 = creator.createSequence(new Integer[0])
                    .append(creator.createSequence(7, 8, 9))
                    .append(creator.createSequence(10, 11, 12));
            assertEqual(6, s5.length());
            assertEqual(7, s5.get(0));
            assertEqual(8, s5.get(1));
            assertEqual(9, s5.get(2));
            assertEqual(10, s5.get(3));
            assertEqual(11, s5.get(4));
            assertEqual(12, s5.get(5));
        }

        public void getSubSequence(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> s1 = creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(1, 4);
            assertEqual(3, s1.length());
            assertEqual(8, s1.get(0));
            assertEqual(9, s1.get(1));
            assertEqual(10, s1.get(2));

            final Sequence<Integer> s2 = creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(0, 0);
            assertEqual(0, s2.length());

            final Sequence<Integer> s3 = creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(0, 1);
            assertEqual(1, s3.length());
            assertEqual(7, s3.get(0));

            final Sequence<Integer> s4 = creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(6, 6);
            assertEqual(0, s4.length());

            final Sequence<Integer> s5 = creator.createSequence(7, 8, 9, 10, 11, 12).getSubSequence(4, 6);
            assertEqual(2, s5.length());
            assertEqual(11, s5.get(0));
            assertEqual(12, s5.get(1));

            final Sequence<Integer> s6 = creator.createSequence(7, 8, 9, 10, 11, 12)
                    .getSubSequence(2, 6)
                    .append(creator.createSequence(13, 14, 15))
                    .getSubSequence(2, 6);
            assertEqual(4, s6.length());
            assertEqual(11, s6.get(0));
            assertEqual(12, s6.get(1));
            assertEqual(13, s6.get(2));
            assertEqual(14, s6.get(3));
        }

        public void insert(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> s1 = creator.createSequence(8, 10).insert(1, creator.createSequence(9));
            assertEqual(3, s1.length());
            assertEqual(8, s1.get(0));
            assertEqual(9, s1.get(1));
            assertEqual(10, s1.get(2));

            final Sequence<Integer> s2 = creator.createSequence(9, 10).insert(0, creator.createSequence(8));
            assertEqual(3, s2.length());
            assertEqual(8, s2.get(0));
            assertEqual(9, s2.get(1));
            assertEqual(10, s2.get(2));

            final Sequence<Integer> s3 = creator.createSequence(8, 9).insert(2, creator.createSequence(10));
            assertEqual(3, s3.length());
            assertEqual(8, s3.get(0));
            assertEqual(9, s3.get(1));
            assertEqual(10, s3.get(2));
        }

        public void delete(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> s1 = creator.createSequence(8, 9, 7, 10).delete(2, 3);
            assertEqual(3, s1.length());
            assertEqual(8, s1.get(0));
            assertEqual(9, s1.get(1));
            assertEqual(10, s1.get(2));

            final Sequence<Integer> s2 = creator.createSequence(7, 8, 9, 10).delete(0, 1);
            assertEqual(3, s2.length());
            assertEqual(8, s2.get(0));
            assertEqual(9, s2.get(1));
            assertEqual(10, s2.get(2));

            final Sequence<Integer> s3 = creator.createSequence(8, 9, 10, 7).delete(3, 4);
            assertEqual(3, s3.length());
            assertEqual(8, s3.get(0));
            assertEqual(9, s3.get(1));
            assertEqual(10, s3.get(2));

            final Sequence<Integer> s4 = creator.createSequence(8, 9, 10).delete(2, 2);
            assertEqual(3, s4.length());
            assertEqual(8, s4.get(0));
            assertEqual(9, s4.get(1));
            assertEqual(10, s4.get(2));
        }

        public void indexOf1(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> s1 = creator.createSequence(7, 8, 9, 10, 7, 11, 12, 9, 15);
            assertEqual(0, s1.indexOf(7));
            assertEqual(1, s1.indexOf(8));
            assertEqual(2, s1.indexOf(9));
            assertEqual(3, s1.indexOf(10));
            assertEqual(5, s1.indexOf(11));
            assertEqual(6, s1.indexOf(12));
            assertEqual(8, s1.indexOf(15));
            assertTrue(s1.indexOf(21) < 0);
            assertTrue(s1.indexOf(null)< 0);

            final Sequence<Integer> s2 = creator.createSequence(7, 8, 9, 10, 7, 11, 12, 9, 15)
                    .getSubSequence(2, 7)
                    .append(creator.createSequence(7, 2, 11));
            assertEqual(0, s2.indexOf(9));
            assertEqual(1, s2.indexOf(10));
            assertEqual(2, s2.indexOf(7));
            assertEqual(3, s2.indexOf(11));
            assertEqual(4, s2.indexOf(12));
            assertEqual(6, s2.indexOf(2));
            assertTrue(s2.indexOf(21) < 0);
            assertTrue(s2.indexOf(null) < 0);
            assertEqual(0, creator.createSequence(new Integer[]{null}).indexOf(null));
        }

        public void indexOf2(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> s = creator.createSequence(7, 8, 9, 10, 7, 11, 12,
                9, 15, 15, 10, -1, 7, 9, 9, 9, null, 7);
            assertEqual(0, s.indexOf(0, 7));
            assertEqual(1, s.indexOf(0, 8));
            assertEqual(2, s.indexOf(0, 9));
            assertEqual(3, s.indexOf(0, 10));
            assertEqual(5, s.indexOf(0, 11));
            assertEqual(6, s.indexOf(0, 12));
            assertEqual(8, s.indexOf(0, 15));
            assertEqual(11, s.indexOf(0, -1));
            assertEqual(16, s.indexOf(0, null));
            assertTrue(s.indexOf(0, 21) < 0);

            assertEqual(1, s.indexOf(1, 8));
            assertEqual(2, s.indexOf(1, 9));
            assertEqual(3, s.indexOf(1, 10));
            assertEqual(4, s.indexOf(1, 7));
            assertEqual(5, s.indexOf(1, 11));
            assertEqual(6, s.indexOf(1, 12));
            assertEqual(8, s.indexOf(1, 15));
            assertEqual(11, s.indexOf(1, -1));
            assertEqual(16, s.indexOf(1, null));
            assertTrue(s.indexOf(1, 21) < 0);

            assertEqual(2, s.indexOf(2, 9));
            assertEqual(3, s.indexOf(2, 10));
            assertEqual(4, s.indexOf(2, 7));
            assertEqual(5, s.indexOf(2, 11));
            assertEqual(6, s.indexOf(2, 12));
            assertEqual(8, s.indexOf(2, 15));
            assertEqual(11, s.indexOf(2, -1));
            assertEqual(16, s.indexOf(2, null));
            assertTrue(s.indexOf(2, 21) < 0);

            assertEqual(3, s.indexOf(3, 10));
            assertEqual(4, s.indexOf(3, 7));
            assertEqual(5, s.indexOf(3, 11));
            assertEqual(6, s.indexOf(3, 12));
            assertEqual(7, s.indexOf(3, 9));
            assertEqual(8, s.indexOf(3, 15));
            assertEqual(11, s.indexOf(3, -1));
            assertEqual(16, s.indexOf(3, null));
            assertTrue(s.indexOf(3, 21) < 0);

            assertEqual(4, s.indexOf(4, 7));
            assertEqual(5, s.indexOf(4, 11));
            assertEqual(6, s.indexOf(4, 12));
            assertEqual(7, s.indexOf(4, 9));
            assertEqual(8, s.indexOf(4, 15));
            assertEqual(10, s.indexOf(4, 10));
            assertEqual(11, s.indexOf(4, -1));
            assertEqual(16, s.indexOf(4, null));
            assertTrue(s.indexOf(4, 21) < 0);

            assertEqual(5, s.indexOf(5, 11));
            assertEqual(6, s.indexOf(5, 12));
            assertEqual(7, s.indexOf(5, 9));
            assertEqual(8, s.indexOf(5, 15));
            assertEqual(10, s.indexOf(5, 10));
            assertEqual(11, s.indexOf(5, -1));
            assertEqual(12, s.indexOf(5, 7));
            assertEqual(16, s.indexOf(5, null));
            assertTrue(s.indexOf(5, 21) < 0);

            assertEqual(6, s.indexOf(6, 12));
            assertEqual(7, s.indexOf(6, 9));
            assertEqual(8, s.indexOf(6, 15));
            assertEqual(10, s.indexOf(6, 10));
            assertEqual(11, s.indexOf(6, -1));
            assertEqual(12, s.indexOf(6, 7));
            assertEqual(16, s.indexOf(6, null));
            assertTrue(s.indexOf(6, 21) < 0);

            assertEqual(7, s.indexOf(7, 9));
            assertEqual(8, s.indexOf(7, 15));
            assertEqual(10, s.indexOf(7, 10));
            assertEqual(11, s.indexOf(7, -1));
            assertEqual(12, s.indexOf(7, 7));
            assertEqual(16, s.indexOf(7, null));
            assertTrue(s.indexOf(7, 21) < 0);

            assertEqual(8, s.indexOf(8, 15));
            assertEqual(10, s.indexOf(8, 10));
            assertEqual(11, s.indexOf(8, -1));
            assertEqual(12, s.indexOf(8, 7));
            assertEqual(13, s.indexOf(8, 9));
            assertEqual(16, s.indexOf(8, null));
            assertTrue(s.indexOf(8, 21) < 0);

            assertEqual(9, s.indexOf(9, 15));
            assertEqual(10, s.indexOf(9, 10));
            assertEqual(11, s.indexOf(9, -1));
            assertEqual(12, s.indexOf(9, 7));
            assertEqual(13, s.indexOf(9, 9));
            assertEqual(16, s.indexOf(9, null));
            assertTrue(s.indexOf(9, 21) < 0);

            assertEqual(10, s.indexOf(10, 10));
            assertEqual(11, s.indexOf(10, -1));
            assertEqual(12, s.indexOf(10, 7));
            assertEqual(13, s.indexOf(10, 9));
            assertEqual(16, s.indexOf(10, null));
            assertTrue(s.indexOf(10, 21) < 0);

            assertEqual(11, s.indexOf(11, -1));
            assertEqual(12, s.indexOf(11, 7));
            assertEqual(13, s.indexOf(11, 9));
            assertEqual(16, s.indexOf(11, null));
            assertTrue(s.indexOf(11, 21) < 0);

            assertEqual(12, s.indexOf(12, 7));
            assertEqual(13, s.indexOf(12, 9));
            assertEqual(16, s.indexOf(12, null));
            assertTrue(s.indexOf(12, 21) < 0);

            assertEqual(13, s.indexOf(13, 9));
            assertEqual(16, s.indexOf(13, null));
            assertEqual(17, s.indexOf(13, 7));
            assertTrue(s.indexOf(13, 21) < 0);

            assertEqual(14, s.indexOf(14, 9));
            assertEqual(16, s.indexOf(14, null));
            assertEqual(17, s.indexOf(14, 7));
            assertTrue(s.indexOf(14, 21) < 0);

            assertEqual(15, s.indexOf(15, 9));
            assertEqual(16, s.indexOf(15, null));
            assertEqual(17, s.indexOf(15, 7));
            assertTrue(s.indexOf(15, 21) < 0);

            assertEqual(16, s.indexOf(16, null));
            assertEqual(17, s.indexOf(16, 7));
            assertTrue(s.indexOf(16, 21) < 0);

            assertEqual(17, s.indexOf(17, 7));
            assertTrue(s.indexOf(17, 21) < 0);

            assertTrue(s.indexOf(18, 21) < 0);
        }

        public void lastIndexOf1(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> s1 = creator.createSequence(7, 8, 9, 10, 7, 11, 12, 9, 15);
            assertEqual(8, s1.lastIndexOf(15));
            assertEqual(7, s1.lastIndexOf(9));
            assertEqual(6, s1.lastIndexOf(12));
            assertEqual(5, s1.lastIndexOf(11));
            assertEqual(4, s1.lastIndexOf(7));
            assertEqual(3, s1.lastIndexOf(10));
            assertEqual(1, s1.lastIndexOf(8));
            assertTrue(s1.lastIndexOf(21) < 0);
            assertTrue(s1.lastIndexOf(null) < 0);

            final Sequence<Integer> s2 = creator.createSequence(7, 8, 9, 10, 7, 11, 12, 9, 15)
                    .getSubSequence(2, 7)
                    .append(creator.createSequence(7, 2, 11));
            assertEqual(7, s2.lastIndexOf(11));
            assertEqual(6, s2.lastIndexOf(2));
            assertEqual(5, s2.lastIndexOf(7));
            assertEqual(4, s2.lastIndexOf(12));
            assertEqual(1, s2.lastIndexOf(10));
            assertEqual(0, s2.lastIndexOf(9));
            assertTrue(s2.lastIndexOf(21) < 0);
            assertTrue(s2.lastIndexOf(null) < 0);
            assertEqual(0, creator.createSequence(new Integer[]{null}).lastIndexOf(null));
        }

        public void lastIndexOf2(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> s = creator.createSequence(7, 8, 9, 10, 7, 11, 12,
                    9, 15, 15, 10, -1, 7, 9, 9, 9, null, 7);

            assertEqual(17, s.lastIndexOf(18, 7));
            assertEqual(16, s.lastIndexOf(18, null));
            assertEqual(15, s.lastIndexOf(18, 9));
            assertEqual(11, s.lastIndexOf(18, -1));
            assertEqual(10, s.lastIndexOf(18, 10));
            assertEqual(9, s.lastIndexOf(18, 15));
            assertEqual(6, s.lastIndexOf(18, 12));
            assertEqual(5, s.lastIndexOf(18, 11));
            assertEqual(1, s.lastIndexOf(18, 8));
            assertTrue(s.lastIndexOf(17, 21) < 0);

            assertEqual(16, s.lastIndexOf(17, null));
            assertEqual(15, s.lastIndexOf(17, 9));
            assertEqual(12, s.lastIndexOf(17, 7));
            assertEqual(11, s.lastIndexOf(17, -1));
            assertEqual(10, s.lastIndexOf(17, 10));
            assertEqual(9, s.lastIndexOf(17, 15));
            assertEqual(6, s.lastIndexOf(17, 12));
            assertEqual(5, s.lastIndexOf(17, 11));
            assertEqual(1, s.lastIndexOf(17, 8));
            assertTrue(s.lastIndexOf(16, 21) < 0);

            assertEqual(15, s.lastIndexOf(16, 9));
            assertEqual(12, s.lastIndexOf(16, 7));
            assertEqual(11, s.lastIndexOf(16, -1));
            assertEqual(10, s.lastIndexOf(16, 10));
            assertEqual(9, s.lastIndexOf(16, 15));
            assertEqual(6, s.lastIndexOf(16, 12));
            assertEqual(5, s.lastIndexOf(16, 11));
            assertEqual(1, s.lastIndexOf(16, 8));
            assertTrue(s.lastIndexOf(15, 21) < 0);

            assertEqual(14, s.lastIndexOf(15, 9));
            assertEqual(12, s.lastIndexOf(15, 7));
            assertEqual(11, s.lastIndexOf(15, -1));
            assertEqual(10, s.lastIndexOf(15, 10));
            assertEqual(9, s.lastIndexOf(15, 15));
            assertEqual(6, s.lastIndexOf(15, 12));
            assertEqual(5, s.lastIndexOf(15, 11));
            assertEqual(1, s.lastIndexOf(15, 8));
            assertTrue(s.lastIndexOf(15, 21) < 0);

            assertEqual(13, s.lastIndexOf(14, 9));
            assertEqual(12, s.lastIndexOf(14, 7));
            assertEqual(11, s.lastIndexOf(14, -1));
            assertEqual(10, s.lastIndexOf(14, 10));
            assertEqual(9, s.lastIndexOf(14, 15));
            assertEqual(6, s.lastIndexOf(14, 12));
            assertEqual(5, s.lastIndexOf(14, 11));
            assertEqual(1, s.lastIndexOf(14, 8));
            assertTrue(s.lastIndexOf(14, 21) < 0);

            assertEqual(12, s.lastIndexOf(13, 7));
            assertEqual(11, s.lastIndexOf(13, -1));
            assertEqual(10, s.lastIndexOf(13, 10));
            assertEqual(9, s.lastIndexOf(13, 15));
            assertEqual(7, s.lastIndexOf(13, 9));
            assertEqual(6, s.lastIndexOf(13, 12));
            assertEqual(5, s.lastIndexOf(13, 11));
            assertEqual(1, s.lastIndexOf(13, 8));
            assertTrue(s.lastIndexOf(13, 21) < 0);

            assertEqual(11, s.lastIndexOf(12, -1));
            assertEqual(10, s.lastIndexOf(12, 10));
            assertEqual(9, s.lastIndexOf(12, 15));
            assertEqual(7, s.lastIndexOf(12, 9));
            assertEqual(6, s.lastIndexOf(12, 12));
            assertEqual(5, s.lastIndexOf(12, 11));
            assertEqual(4, s.lastIndexOf(12, 7));
            assertEqual(1, s.lastIndexOf(12, 8));
            assertTrue(s.lastIndexOf(12, 21) < 0);

            assertEqual(10, s.lastIndexOf(11, 10));
            assertEqual(9, s.lastIndexOf(11, 15));
            assertEqual(7, s.lastIndexOf(11, 9));
            assertEqual(6, s.lastIndexOf(11, 12));
            assertEqual(5, s.lastIndexOf(11, 11));
            assertEqual(4, s.lastIndexOf(11, 7));
            assertEqual(1, s.lastIndexOf(11, 8));
            assertTrue(s.lastIndexOf(11, 21) < 0);

            assertEqual(9, s.lastIndexOf(10, 15));
            assertEqual(7, s.lastIndexOf(10, 9));
            assertEqual(6, s.lastIndexOf(10, 12));
            assertEqual(5, s.lastIndexOf(10, 11));
            assertEqual(4, s.lastIndexOf(10, 7));
            assertEqual(3, s.lastIndexOf(10, 10));
            assertEqual(1, s.lastIndexOf(10, 8));
            assertTrue(s.lastIndexOf(10, 21) < 0);

            assertEqual(8, s.lastIndexOf(9, 15));
            assertEqual(7, s.lastIndexOf(9, 9));
            assertEqual(6, s.lastIndexOf(9, 12));
            assertEqual(5, s.lastIndexOf(9, 11));
            assertEqual(4, s.lastIndexOf(9, 7));
            assertEqual(3, s.lastIndexOf(9, 10));
            assertEqual(1, s.lastIndexOf(9, 8));
            assertTrue(s.lastIndexOf(9, 21) < 0);

            assertEqual(7, s.lastIndexOf(8, 9));
            assertEqual(6, s.lastIndexOf(8, 12));
            assertEqual(5, s.lastIndexOf(8, 11));
            assertEqual(4, s.lastIndexOf(8, 7));
            assertEqual(3, s.lastIndexOf(8, 10));
            assertEqual(1, s.lastIndexOf(8, 8));
            assertTrue(s.lastIndexOf(8, 21) < 0);

            assertEqual(6, s.lastIndexOf(7, 12));
            assertEqual(5, s.lastIndexOf(7, 11));
            assertEqual(4, s.lastIndexOf(7, 7));
            assertEqual(3, s.lastIndexOf(7, 10));
            assertEqual(2, s.lastIndexOf(7, 9));
            assertEqual(1, s.lastIndexOf(7, 8));
            assertTrue(s.lastIndexOf(7, 21) < 0);

            assertEqual(5, s.lastIndexOf(6, 11));
            assertEqual(4, s.lastIndexOf(6, 7));
            assertEqual(3, s.lastIndexOf(6, 10));
            assertEqual(2, s.lastIndexOf(6, 9));
            assertEqual(1, s.lastIndexOf(6, 8));
            assertTrue(s.lastIndexOf(6, 21) < 0);

            assertEqual(4, s.lastIndexOf(5, 7));
            assertEqual(3, s.lastIndexOf(5, 10));
            assertEqual(2, s.lastIndexOf(5, 9));
            assertEqual(1, s.lastIndexOf(5, 8));
            assertTrue(s.lastIndexOf(5, 21) < 0);

            assertEqual(3, s.lastIndexOf(4, 10));
            assertEqual(2, s.lastIndexOf(4, 9));
            assertEqual(1, s.lastIndexOf(4, 8));
            assertEqual(0, s.lastIndexOf(4, 7));
            assertTrue(s.lastIndexOf(4, 21) < 0);

            assertEqual(2, s.lastIndexOf(3, 9));
            assertEqual(1, s.lastIndexOf(3, 8));
            assertEqual(0, s.lastIndexOf(3, 7));
            assertTrue(s.lastIndexOf(3, 21) < 0);

            assertEqual(1, s.lastIndexOf(2, 8));
            assertEqual(0, s.lastIndexOf(2, 7));
            assertTrue(s.lastIndexOf(2, 21) < 0);

            assertEqual(0, s.lastIndexOf(1, 7));
            assertTrue(s.lastIndexOf(1, 21) < 0);

            assertTrue(s.lastIndexOf(0, 21) < 0);
        }

        public void iterator(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Iterator<Integer> i = creator.createSequence(7, 8, 9, 10, 7,
                    11, 12, 9, 15).iterator();
            assertTrue(i.hasNext());
            assertEqual(7, i.next());
            assertTrue(i.hasNext());
            assertEqual(8, i.next());
            assertTrue(i.hasNext());
            assertEqual(9, i.next());
            assertTrue(i.hasNext());
            assertEqual(10, i.next());
            assertTrue(i.hasNext());
            assertEqual(7, i.next());
            assertTrue(i.hasNext());
            assertEqual(11, i.next());
            assertTrue(i.hasNext());
            assertEqual(12, i.next());
            assertTrue(i.hasNext());
            assertEqual(9, i.next());
            assertTrue(i.hasNext());
            assertEqual(15, i.next());
            assertFalse(i.hasNext());

            assertFalse(creator.createSequence().iterator().hasNext());
        }

        public void getIterator1(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final SequenceIterator<Integer> i = creator.createSequence(7, 8, 9, 10, 7,
                    11, 12, 9, 15).getIterator();
            assertTrue(i.hasNext());
            assertEqual(0, i.getPosition());
            assertEqual(7, i.next());
            assertTrue(i.hasNext());
            assertEqual(1, i.getPosition());
            assertEqual(8, i.next());
            assertTrue(i.hasNext());
            assertEqual(2, i.getPosition());
            assertEqual(9, i.next());
            assertTrue(i.hasNext());
            assertEqual(3, i.getPosition());
            assertEqual(10, i.next());
            assertTrue(i.hasNext());
            assertEqual(4, i.getPosition());
            assertEqual(7, i.next());
            assertTrue(i.hasNext());
            assertEqual(5, i.getPosition());
            assertEqual(11, i.next());
            assertTrue(i.hasNext());
            assertEqual(6, i.getPosition());
            assertEqual(12, i.next());
            assertTrue(i.hasNext());
            assertEqual(7, i.getPosition());
            assertEqual(9, i.next());
            assertTrue(i.hasNext());
            assertEqual(8, i.getPosition());
            assertEqual(15, i.next());
            assertFalse(i.hasNext());
            assertEqual(9, i.getPosition());

            assertTrue(i.hasPrevious());
            assertEqual(15, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(8, i.getPosition());
            assertEqual(9, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(7, i.getPosition());
            assertEqual(12, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(6, i.getPosition());
            assertEqual(11, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(5, i.getPosition());
            assertEqual(7, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(4, i.getPosition());
            assertEqual(10, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(3, i.getPosition());
            assertEqual(9, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(2, i.getPosition());
            assertEqual(8, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(1, i.getPosition());
            assertEqual(7, i.previous());
            assertFalse(i.hasPrevious());
            assertEqual(0, i.getPosition());

            assertTrue(i.hasNext());
            assertEqual(7, i.next());
            assertTrue(i.hasNext());
            assertEqual(1, i.getPosition());
            assertEqual(8, i.next());
            assertTrue(i.hasNext());
            assertEqual(2, i.getPosition());
            assertEqual(9, i.next());
            assertTrue(i.hasNext());
            assertEqual(3, i.getPosition());
            assertEqual(10, i.next());
            assertTrue(i.hasNext());
            assertEqual(4, i.getPosition());
            assertEqual(7, i.next());
            assertTrue(i.hasNext());
            assertEqual(5, i.getPosition());
            assertEqual(11, i.next());
            assertTrue(i.hasNext());
            assertEqual(6, i.getPosition());
            assertEqual(12, i.next());
            assertTrue(i.hasNext());
            assertEqual(7, i.getPosition());
            assertEqual(9, i.next());
            assertTrue(i.hasNext());
            assertEqual(8, i.getPosition());
            assertEqual(15, i.next());
            assertFalse(i.hasNext());
            assertEqual(9, i.getPosition());

            assertFalse(creator.createSequence().getIterator().hasNext());
            assertFalse(creator.createSequence().getIterator().hasPrevious());
        }

        public void getIterator2(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final SequenceIterator<Integer> i = creator.createSequence(7, 8, 9, 10, 11, 12).getIterator(2);

            assertTrue(i.hasNext());
            assertEqual(2, i.getPosition());
            assertEqual(9, i.next());
            assertTrue(i.hasNext());
            assertEqual(3, i.getPosition());
            assertEqual(10, i.next());
            assertTrue(i.hasNext());
            assertEqual(4, i.getPosition());
            assertEqual(11, i.next());
            assertTrue(i.hasNext());
            assertEqual(5, i.getPosition());
            assertEqual(12, i.next());
            assertFalse(i.hasNext());
            assertEqual(6, i.getPosition());

            assertTrue(i.hasPrevious());
            assertEqual(12, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(5, i.getPosition());
            assertEqual(11, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(4, i.getPosition());
            assertEqual(10, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(3, i.getPosition());
            assertEqual(9, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(2, i.getPosition());
            assertEqual(8, i.previous());
            assertTrue(i.hasPrevious());
            assertEqual(1, i.getPosition());
            assertEqual(7, i.previous());
            assertFalse(i.hasPrevious());
            assertEqual(0, i.getPosition());

            assertTrue(i.hasNext());
            assertEqual(7, i.next());
            assertTrue(i.hasNext());
            assertEqual(1, i.getPosition());
            assertEqual(8, i.next());
            assertTrue(i.hasNext());
            assertEqual(2, i.getPosition());
            assertEqual(9, i.next());
            assertEqual(3, i.getPosition());
        }

        public void isEmpty(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            assertFalse(creator.createSequence(7).isEmpty());
            assertFalse(creator.createSequence(7, 8, 9).isEmpty());
            assertTrue(creator.createSequence(new Integer[0]).isEmpty());
        }

        public void _toString(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            assertEqual("[]", creator.createSequence(new Integer[0]).toString());
            assertEqual("[7][8][9]", creator.createSequence(7, 8, 9).toString());
            assertEqual("[7][8][9][10][11][12]", creator.createSequence(7, 8, 9, 10, 11, 12).toString());
            assertEqual("[9][10][7][11][12][7][2][11]", creator.createSequence(7, 8, 9, 10, 7, 11, 12, 9, 15)
                    .getSubSequence(2, 7)
                    .append(creator.createSequence(7, 2, 11)).toString());
        }

        public void equalsHashCode(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final ObjectFactory<Sequence<Integer>> factory1 = new ObjectFactory<Sequence<Integer>>()
            {
                public Sequence<Integer> newInstanceX()
                {
                    return creator.createSequence(7, 8, 9);
                }

                public Sequence<Integer> newInstanceY()
                {
                    return creator.createSequence(7, 8, 9, 10, 11, 12);
                }
            };

            final ObjectFactory<Sequence<Integer>> factory2 = new ObjectFactory<Sequence<Integer>>()
            {
                public Sequence<Integer> newInstanceX()
                {
                    return creator.createSequence(7, 8, 9, 10, 7, 11, 12, 9, 15)
                        .getSubSequence(2, 7).append(creator.createSequence(7, 14, 17));
                }

                public Sequence<Integer> newInstanceY()
                {
                    return creator.createSequence(7, 8, 9, 10, 7, 11, 12, 9, 15)
                        .getSubSequence(2, 7).append(creator.createSequence(7, 2, 11));
                }
            };

            final ObjectFactory<Sequence<Integer>> factory3 = new ObjectFactory<Sequence<Integer>>()
            {
                public Sequence<Integer> newInstanceX()
                {
                    return creator.createSequence(7, null, 9);
                }

                public Sequence<Integer> newInstanceY()
                {
                    return creator.createSequence(7, 8, 9);
                }
            };

            assertObjectFactoryFillsContract(factory1);
            assertEqualsMethodFillsContract(factory1);
            assertHashCodeMethodFillsContract(factory1);

            assertObjectFactoryFillsContract(factory2);
            assertEqualsMethodFillsContract(factory2);
            assertHashCodeMethodFillsContract(factory2);

            assertObjectFactoryFillsContract(factory3);
            assertEqualsMethodFillsContract(factory3);
            assertHashCodeMethodFillsContract(factory3);
        }

        public void clone(final SequenceCreator<Integer> creator) throws NullPointerException, AssertionException
        {
            if(creator == null)
            {
                throw new NullPointerException();
            }

            final Sequence<Integer> a1 = creator.createSequence(7, 8, 9);
            final Sequence<Integer> b1 = a1.clone();
            assertNotSame(a1, b1);
            assertEqual(a1, b1);
        }
    }
}
