/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import static org.jtiger.assertion.Reflection.assertHasDeclaredConstructor;
import static org.jtiger.assertion.Modifier.assertPrivate;
import static org.jtiger.assertion.Basic.assertNull;
import static org.jtiger.assertion.Basic.assertNotNull;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertSame;
import static org.jtiger.assertion.Basic.assertTrue;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.framework.ArgsConverterFactory.newArgsToFixtureRunnerParams;
import org.jtiger.framework.junit.JUnitTestDefinition;
import org.jtiger.framework.junit.JUnitSetUpTearDown;
import org.jtiger.report.html.HtmlFixtureResultsHandler;
import org.jtiger.report.xml.XmlFixtureResultsHandler;
import org.jtiger.report.text.TextFixtureResultsHandler;

@Fixture(value = "ArgsConverterFactory", description = "Performs Unit Tests on org.jtiger.framework.ArgsConverterFactory")
@Category("org.jtiger.framework.ArgsConverterFactory")
public final class TestArgsConverterFactory
{
    public TestArgsConverterFactory()
    {

    }

    @Test
    public void constructor() throws InstantiationException, IllegalAccessException, InvocationTargetException
    {
        final Constructor<?> c = assertHasDeclaredConstructor(ArgsConverterFactory.class, null);
        assertPrivate(c);
        assertFalse(c.isAccessible());
        c.setAccessible(true);
        c.newInstance();
    }

    @Test
    @ExpectException(NullPointerException.class)
    public void convertNullArg() throws ClassNotFoundException
    {
        final String[] args = null;
        newArgsToFixtureRunnerParams().convert(args);
    }

    @Test
    public void zeroLengthArg() throws ClassNotFoundException
    {
        assertNull(newArgsToFixtureRunnerParams().convert());
    }

    @Test
    public void fixtureClassesArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_FIXTURE_CLASSES, getClass().getName(), String.class.getName());
        assertNotNull(args);
        assertNull(args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(2, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertSame(getClass(), args.getFixturesRunnerConfig().getFixtureClasses().get(0));
        assertSame(String.class, args.getFixturesRunnerConfig().getFixtureClasses().get(1));
        assertNull(args.getFixturesRunnerConfig().getDefinitionClass());
        assertNull(args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void definitionClassArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_DEFINITION_CLASS, DummyTestDefinition.class.getName());
        assertNotNull(args);
        assertNull(args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertSame(DummyTestDefinition.class, args.getFixturesRunnerConfig().getDefinitionClass());
        assertNull(args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void sutdClassArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_SUTD_CLASS, DummySetUpTearDown.class.getName());
        assertNotNull(args);
        assertNull(args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertNull(args.getFixturesRunnerConfig().getDefinitionClass());
        assertSame(DummySetUpTearDown.class, args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void haltOnFailureArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_HALT_ON_FAILURE);
        assertNotNull(args);
        assertNull(args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertNull(args.getFixturesRunnerConfig().getDefinitionClass());
        assertNull(args.getFixturesRunnerConfig().getSutdClass());
        assertTrue(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void junitArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_JUNIT);
        assertNotNull(args);
        assertNull(args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertSame(JUnitTestDefinition.class, args.getFixturesRunnerConfig().getDefinitionClass());
        assertSame(JUnitSetUpTearDown.class, args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void categoriesArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_CATEGORIES, "cat1", "cat2", "cat3");
        assertNotNull(args);
        assertNull(args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertNull(args.getFixturesRunnerConfig().getDefinitionClass());
        assertNull(args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(3, args.getFixturesRunnerConfig().getCategories().length());
        assertEqual("cat1", args.getFixturesRunnerConfig().getCategories().get(0));
        assertEqual("cat2", args.getFixturesRunnerConfig().getCategories().get(1));
        assertEqual("cat3", args.getFixturesRunnerConfig().getCategories().get(2));
    }

    @Test
    public void resultHandlerArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_RESULT, DummyFixtureResultsHandler.class.getName());
        assertNotNull(args);
        assertSame(DummyFixtureResultsHandler.class, args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertNull(args.getFixturesRunnerConfig().getDefinitionClass());
        assertNull(args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void resultHandlerParametersArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_RESULT_PARAMETERS, "param1", "param2", "param3");
        assertNotNull(args);
        assertNull(args.getResultHandlerClass());
        assertEqual(3, args.getResultHandlerParameters().length());
        assertEqual("param1", args.getResultHandlerParameters().get(0));
        assertEqual("param2", args.getResultHandlerParameters().get(1));
        assertEqual("param3", args.getResultHandlerParameters().get(2));
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertNull(args.getFixturesRunnerConfig().getDefinitionClass());
        assertNull(args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void textResultHandlerParametersArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_RESULT, "~text");
        assertNotNull(args);
        assertSame(TextFixtureResultsHandler.class, args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertNull(args.getFixturesRunnerConfig().getDefinitionClass());
        assertNull(args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void xmlResultHandlerParametersArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_RESULT, "~xml");
        assertNotNull(args);
        assertSame(XmlFixtureResultsHandler.class, args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertNull(args.getFixturesRunnerConfig().getDefinitionClass());
        assertNull(args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void htmlResultHandlerParametersArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_RESULT, "~html");
        assertNotNull(args);
        assertSame(HtmlFixtureResultsHandler.class, args.getResultHandlerClass());
        assertEqual(0, args.getResultHandlerParameters().length());
        assertEqual(0, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertNull(args.getFixturesRunnerConfig().getDefinitionClass());
        assertNull(args.getFixturesRunnerConfig().getSutdClass());
        assertFalse(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(0, args.getFixturesRunnerConfig().getCategories().length());
    }

    @Test
    public void unrecognizedArg() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert("random", "arguments");
        assertNull(args);
    }

    @Test
    public void allArgs() throws ClassNotFoundException
    {
        final Args args = newArgsToFixtureRunnerParams().convert(
                FixturesRunnerMain.ARG_FIXTURE_CLASSES, getClass().getName(), String.class.getName(),
                FixturesRunnerMain.ARG_DEFINITION_CLASS, DummyTestDefinition.class.getName(),
                FixturesRunnerMain.ARG_SUTD_CLASS, DummySetUpTearDown.class.getName(),
                FixturesRunnerMain.ARG_HALT_ON_FAILURE,
                FixturesRunnerMain.ARG_CATEGORIES, "cat1", "cat2", "cat3",
                FixturesRunnerMain.ARG_RESULT, DummyFixtureResultsHandler.class.getName(),
                FixturesRunnerMain.ARG_RESULT_PARAMETERS, "param1", "param2", "param3");
        assertNotNull(args);
        assertSame(DummyFixtureResultsHandler.class, args.getResultHandlerClass());
        assertEqual(3, args.getResultHandlerParameters().length());
        assertEqual("param1", args.getResultHandlerParameters().get(0));
        assertEqual("param2", args.getResultHandlerParameters().get(1));
        assertEqual("param3", args.getResultHandlerParameters().get(2));
        assertEqual(2, args.getFixturesRunnerConfig().getFixtureClasses().length());
        assertSame(getClass(), args.getFixturesRunnerConfig().getFixtureClasses().get(0));
        assertSame(String.class, args.getFixturesRunnerConfig().getFixtureClasses().get(1));
        assertSame(DummyTestDefinition.class, args.getFixturesRunnerConfig().getDefinitionClass());
        assertSame(DummySetUpTearDown.class, args.getFixturesRunnerConfig().getSutdClass());
        assertTrue(args.getFixturesRunnerConfig().isHaltOnFailure());
        assertEqual(3, args.getFixturesRunnerConfig().getCategories().length());
        assertEqual("cat1", args.getFixturesRunnerConfig().getCategories().get(0));
        assertEqual("cat2", args.getFixturesRunnerConfig().getCategories().get(1));
        assertEqual("cat3", args.getFixturesRunnerConfig().getCategories().get(2));
    }

    @Test
    public void valuesOf()
    {
        for(ArgsConverterFactory.ArgReadState s : ArgsConverterFactory.ArgReadState.values())
        {
            assertSame(s, ArgsConverterFactory.ArgReadState.valueOf(s.toString()));
        }
    }

    private static final class DummyTestDefinition implements TestDefinition
    {
        public boolean isTest(final Method m)
        {
            return false;
        }
    }

    private static final class DummySetUpTearDown implements SetUpTearDown
    {
        public void setUp(final Object fixture)
        {

        }

        public void tearDown(final Object fixture)
        {

        }
    }

    private static final class DummyFixtureResultsHandler implements FixtureResultsHandler
    {
        public void handleResult(final FixtureResults results, final ReadOnlyArray<String> params)
        {

        }
    }
}
