/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.framework.SequenceFactory.newSequence;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import org.jtiger.assertion.ObjectFactory;
import static org.jtiger.framework.ArgsFactory.newArgs;
import static org.jtiger.assertion.ObjectFactoryContract.assertObjectFactoryFillsContract;
import static org.jtiger.assertion.EqualsMethodContract.assertEqualsMethodFillsContract;
import static org.jtiger.assertion.HashCodeMethodContract.assertHashCodeMethodFillsContract;
import static org.jtiger.assertion.Reflection.assertHasDeclaredConstructor;
import static org.jtiger.assertion.Modifier.assertPrivate;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertSame;
import static org.jtiger.assertion.Basic.assertNull;
import static org.jtiger.assertion.Basic.assertNotNull;

@Fixture(value = "ArgsFactory", description = "Performs Unit Tests on org.jtiger.framework.ArgsFactory")
@Category("org.jtiger.framework.ArgsFactory")
public final class TestArgsFactory
{
    public TestArgsFactory()
    {

    }

    @Test
    public void constructor() throws InstantiationException, IllegalAccessException, InvocationTargetException
    {
        final Constructor<?> c = assertHasDeclaredConstructor(ArgsFactory.class, null);
        assertPrivate(c);
        assertFalse(c.isAccessible());
        c.setAccessible(true);
        c.newInstance();
    }

    @Test
    public void newArgs1()
    {
        final Args args = newArgs(null, null, null);
        assertNotNull(args);
        assertNull(args.getFixturesRunnerConfig());
        assertNull(args.getResultHandlerClass());
        assertNull(args.getResultHandlerParameters());
    }

    @Test
    public void newArgs2()
    {
        final FixturesRunnerConfig c = new DummyFixturesRunnerConfig();
        final Args args = newArgs(c, null, null);
        assertNotNull(args);
        assertSame(c, args.getFixturesRunnerConfig());
        assertNull(args.getResultHandlerClass());
        assertNull(args.getResultHandlerParameters());
    }

    @Test
    public void newArgs3()
    {
        final Args args = newArgs(null, DummyFixtureResultsHandler.class, null);
        assertNotNull(args);
        assertNull(args.getFixturesRunnerConfig());
        assertSame(DummyFixtureResultsHandler.class, args.getResultHandlerClass());
        assertNull(args.getResultHandlerParameters());
    }

    @Test
    public void newArgs4()
    {
        final ReadOnlyArray<String> a = newSequence(new String[0]);
        final Args args = newArgs(null, null, a);
        assertNotNull(args);
        assertNull(args.getFixturesRunnerConfig());
        assertNull(args.getResultHandlerClass());
        assertSame(a, args.getResultHandlerParameters());
    }

    @Test
    public void newArgs5()
    {
        final ReadOnlyArray<String> a = newSequence(new String[0]);
        final FixturesRunnerConfig c = new DummyFixturesRunnerConfig();
        final Args args = newArgs(c, DummyFixtureResultsHandler.class, a);
        assertNotNull(args);
        assertSame(c, args.getFixturesRunnerConfig());
        assertSame(DummyFixtureResultsHandler.class, args.getResultHandlerClass());
        assertSame(a, args.getResultHandlerParameters());
    }

    @Test
    public void testToString()
    {
        final FixturesRunnerConfig c = new DummyFixturesRunnerConfig();
        final Args args = newArgs(c, DummyFixtureResultsHandler.class, newSequence(new String[0]));
        assertNotNull(args);
        assertEqual("[" + c + "][" + DummyFixtureResultsHandler.class + "][" + newSequence(new String[0]) + "]", args.toString());
    }

    @Test
    public void equalsHashCode1()
    {
        assertFalse(newArgs(null, null, null).equals(new Object()));

        final FixturesRunnerConfig c = new DummyFixturesRunnerConfig();

        final ObjectFactory<Args> factory = new ObjectFactory<Args>()
        {
            public Args newInstanceX()
            {
                return newArgs(null, null, null);
            }

            public Args newInstanceY()
            {
                return newArgs(c, null, null);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode2()
    {
        assertFalse(newArgs(null, null, null).equals(new Object()));

        final FixturesRunnerConfig c = new DummyFixturesRunnerConfig();

        final ObjectFactory<Args> factory = new ObjectFactory<Args>()
        {
            public Args newInstanceX()
            {
                return newArgs(c, null, null);
            }

            public Args newInstanceY()
            {
                return newArgs(null, DummyFixtureResultsHandler.class, null);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode3()
    {
        assertFalse(newArgs(null, null, null).equals(new Object()));

        final ReadOnlyArray<String> params = newSequence(new String[0]);

        final ObjectFactory<Args> factory = new ObjectFactory<Args>()
        {
            public Args newInstanceX()
            {
                return newArgs(null, DummyFixtureResultsHandler.class, null);
            }

            public Args newInstanceY()
            {
                return newArgs(null, null, params);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
    }

    private static final class DummyFixturesRunnerConfig implements FixturesRunnerConfig
    {
        DummyFixturesRunnerConfig()
        {

        }

        public ReadOnlyArray<Class<?>> getFixtureClasses()
        {
            return null;
        }

        public Class<? extends TestDefinition> getDefinitionClass()
        {
            return null;
        }

        public Class<? extends SetUpTearDown> getSutdClass()
        {
            return null;
        }

        public boolean isHaltOnFailure()
        {
            return false;
        }

        public ReadOnlyArray<String> getCategories()
        {
            return null;
        }
    }

    private static final class DummyFixtureResultsHandler implements FixtureResultsHandler
    {
        DummyFixtureResultsHandler()
        {

        }

        public void handleResult(final FixtureResults results, final ReadOnlyArray<String> params)
        {

        }
    }
}
