/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.framework.SequenceFactory.newSequence;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import org.jtiger.DummySetUpTearDown;
import org.jtiger.DummyTestDefinition;
import org.jtiger.assertion.ObjectFactory;
import static org.jtiger.assertion.Reflection.assertHasDeclaredField;
import static org.jtiger.assertion.Serialization.assertSerializes;
import static org.jtiger.assertion.Serialization.assertSerializesEqual;
import static org.jtiger.assertion.Reflection.assertHasDeclaredConstructor;
import static org.jtiger.assertion.Modifier.assertPrivate;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertSame;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.framework.FixturesRunnerConfigFactory.newFixturesRunnerConfig;
import static org.jtiger.assertion.ObjectFactoryContract.assertObjectFactoryFillsContract;
import static org.jtiger.assertion.HashCodeMethodContract.assertHashCodeMethodFillsContract;
import static org.jtiger.assertion.EqualsMethodContract.assertEqualsMethodFillsContract;

@Fixture(value = "FixturesRunnerConfigFactory", description = "Performs Unit Tests on org.jtiger.framework.FixturesRunnerConfigFactory")
@Category("org.jtiger.framework.FixturesRunnerConfigFactory")
public final class TestFixturesRunnerConfigFactory
{
    private FixturesRunnerConfig config;

    public TestFixturesRunnerConfigFactory()
    {

    }

    @SetUp
    public void createTestList()
    {
        config = newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                DummySetUpTearDown.class,
                false,
                "cat1", "cat2");
    }

    @TearDown
    public void emptyTestList()
    {
        config = null;
    }

    @Test
    public void constructor() throws InstantiationException, IllegalAccessException, InvocationTargetException
    {
        final Constructor<?> c = assertHasDeclaredConstructor(FixturesRunnerConfigFactory.class, null);
        assertPrivate(c);
        assertFalse(c.isAccessible());
        c.setAccessible(true);
        c.newInstance();
    }

    @Test
    @ExpectException(NullPointerException.class)
    public void factoryMethod()
    {
        final String[] categories = null;

        newFixturesRunnerConfig(null, null, null, false, categories);
    }

    @Test
    public void getFixtureClasses()
    {
        final ReadOnlyArray<Class<?>> a = newSequence(new Class<?>[]{getClass()});

        assertEqual(a, config.getFixtureClasses());
    }

    @Test
    public void getDefinitionClass()
    {
        assertSame(DummyTestDefinition.class, config.getDefinitionClass());
    }

    @Test
    public void getSutdClass()
    {
        assertSame(DummySetUpTearDown.class, config.getSutdClass());
    }

    @Test
    public void isHaltOnFailure()
    {
        assertFalse(config.isHaltOnFailure());
    }

    @Test
    public void getCategories()
    {
        assertEqual(2, config.getCategories().length());
        assertEqual("cat1", config.getCategories().get(0));
        assertEqual("cat2", config.getCategories().get(1));
    }

    @Test
    public void serialization()
    {
        final FixturesRunnerConfig config = newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                DummySetUpTearDown.class,
                false,
                "cat1", "cat2", "cat3");

        assertSerializes(config);
        assertSerializesEqual(config);
        assertHasDeclaredField(config.getClass(), "serialVersionUID");
    }

    @Test
    public void testToString()
    {
        final String expected = "[" + newSequence(new Class<?>[]{getClass()}) +
                "][" + DummyTestDefinition.class +
                "][" + DummySetUpTearDown.class +
                "][" + false +
                "][" +  newSequence("cat1", "cat2") + "]";

        assertEqual(expected, config.toString());
    }

    @Test
    public void equalsHashCode1()
    {
        final ObjectFactory<FixturesRunnerConfig> factory = new ObjectFactory<FixturesRunnerConfig>()
        {
            public FixturesRunnerConfig newInstanceX()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                DummySetUpTearDown.class,
                false);
            }

            public FixturesRunnerConfig newInstanceY()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                DummySetUpTearDown.class,
                false,
                "cat1", "cat2");
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode2()
    {
        final ObjectFactory<FixturesRunnerConfig> factory = new ObjectFactory<FixturesRunnerConfig>()
        {
            public FixturesRunnerConfig newInstanceX()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                DummySetUpTearDown.class,
                false,
                "cat1", "cat2");
            }

            public FixturesRunnerConfig newInstanceY()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                DummySetUpTearDown.class,
                true,
                "cat1", "cat2");
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode3()
    {
        final ObjectFactory<FixturesRunnerConfig> factory = new ObjectFactory<FixturesRunnerConfig>()
        {
            public FixturesRunnerConfig newInstanceX()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                DummySetUpTearDown.class,
                true,
                "cat1", "cat2");
            }

            public FixturesRunnerConfig newInstanceY()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                DummySetUpTearDown.class,
                false,
                "cat1", "cat2", "cat3");
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode4()
    {
        final ObjectFactory<FixturesRunnerConfig> factory = new ObjectFactory<FixturesRunnerConfig>()
        {
            public FixturesRunnerConfig newInstanceX()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                DummySetUpTearDown.class,
                false,
                "cat1", "cat2", "cat3");
            }

            public FixturesRunnerConfig newInstanceY()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                TestDefinition.class,
                DummySetUpTearDown.class,
                false,
                "cat1", "cat2", "cat3");
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode5()
    {
        final ObjectFactory<FixturesRunnerConfig> factory = new ObjectFactory<FixturesRunnerConfig>()
        {
            public FixturesRunnerConfig newInstanceX()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                TestDefinition.class,
                DummySetUpTearDown.class,
                false,
                "cat1", "cat2", "cat3");
            }

            public FixturesRunnerConfig newInstanceY()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                SetUpTearDown.class,
                false);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode6()
    {
        final ObjectFactory<FixturesRunnerConfig> factory = new ObjectFactory<FixturesRunnerConfig>()
        {
            public FixturesRunnerConfig newInstanceX()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass()}),
                DummyTestDefinition.class,
                SetUpTearDown.class,
                false);
            }

            public FixturesRunnerConfig newInstanceY()
            {
                return newFixturesRunnerConfig(
                newSequence(new Class<?>[]{getClass(), String.class}),
                TestDefinition.class,
                DummySetUpTearDown.class,
                false);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
    }
}
