/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.List;
import java.util.LinkedList;
import org.jtiger.assertion.ObjectFactory;
import static java.util.Arrays.asList;
import static org.jtiger.assertion.Reflection.assertHasDeclaredField;
import static org.jtiger.assertion.Serialization.assertSerializes;
import static org.jtiger.assertion.Serialization.assertSerializesEqual;
import static org.jtiger.assertion.ObjectFactoryContract.assertObjectFactoryFillsContract;
import static org.jtiger.assertion.EqualsMethodContract.assertEqualsMethodFillsContract;
import static org.jtiger.assertion.HashCodeMethodContract.assertHashCodeMethodFillsContract;
import static org.jtiger.assertion.Reflection.assertHasDeclaredConstructor;
import static org.jtiger.assertion.Modifier.assertPrivate;
import static org.jtiger.assertion.Basic.assertTrue;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertNotSame;
import static org.jtiger.framework.ReadOnlyArrayFactory.newReadOnlyArray;

@Fixture(value = "ReadOnlyArrayFactory", description = "Performs Unit Tests on org.jtiger.framework.ReadOnlyArrayFactory")
@Category("org.jtiger.framework.ReadOnlyArrayFactory")
public final class TestReadOnlyArrayFactory
{
    public TestReadOnlyArrayFactory()
    {

    }

    @Test
    public void constructor() throws InstantiationException, IllegalAccessException, InvocationTargetException
    {
        final Constructor<?> c = assertHasDeclaredConstructor(ReadOnlyArrayFactory.class, null);
        assertPrivate(c);
        assertFalse(c.isAccessible());
        c.setAccessible(true);
        c.newInstance();
    }

    @Test
    @ExpectException(NullPointerException.class)
    public void factoryMethod1()
    {
        final String[] s = null;
        newReadOnlyArray(s);
    }

    @Test
    @ExpectException(NullPointerException.class)
    public void factoryMethod2()
    {
        final List<?> l = null;
        newReadOnlyArray(l);
    }

    @Test
    public void get1()
    {
        assertEqual("a", newReadOnlyArray(new String[]{"a", "b", "c"}).get(0));
        assertEqual("b", newReadOnlyArray(new String[]{"a", "b", "c"}).get(1));
        assertEqual("c", newReadOnlyArray(new String[]{"a", "b", "c"}).get(2));
        assertEqual("a", newReadOnlyArray(asList(new String[]{"a", "b", "c"})).get(0));
        assertEqual("b", newReadOnlyArray(asList(new String[]{"a", "b", "c"})).get(1));
        assertEqual("c", newReadOnlyArray(asList(new String[]{"a", "b", "c"})).get(2));
    }

    @Test
    @ExpectException(ArrayIndexOutOfBoundsException.class)
    public void get2()
    {
        newReadOnlyArray(new String[]{"a", "b", "c"}).get(-1);
    }

    @Test
    @ExpectException(ArrayIndexOutOfBoundsException.class)
    public void get3()
    {
        newReadOnlyArray(new String[]{"a", "b", "c"}).get(4);
    }

    @Test
    @ExpectException(ArrayIndexOutOfBoundsException.class)
    public void get4()
    {
        newReadOnlyArray(asList(new String[]{"a", "b", "c"})).get(-1);
    }

    @Test
    @ExpectException(ArrayIndexOutOfBoundsException.class)
    public void get5()
    {
        newReadOnlyArray(asList(new String[]{"a", "b", "c"})).get(4);
    }

    @Test
    public void length()
    {
        assertEqual(3, newReadOnlyArray(new String[]{"a", "b", "c"}).length());
        assertEqual(0, newReadOnlyArray(new String[0]).length());
        assertEqual(3, newReadOnlyArray(asList(new String[]{"a", "b", "c"})).length());
        assertEqual(0, newReadOnlyArray(asList(new String[0])).length());
    }

    @Test
    public void serialization()
    {
        final ReadOnlyArray<String> a = newReadOnlyArray("a", "b", "c");

        assertSerializes(a);
        assertSerializesEqual(a);
        assertHasDeclaredField(a.getClass(), "serialVersionUID");

        final ReadOnlyArray<String> l = newReadOnlyArray(asList("a", "b", "c"));

        assertSerializes(l);
        assertSerializesEqual(l);
        assertHasDeclaredField(l.getClass(), "serialVersionUID");
    }

    @Test
    public void testToString()
    {
        final ReadOnlyArray<String> a = newReadOnlyArray("a", "b", "c");
        assertEqual(Arrays.toString(new String[]{"a", "b", "c"}), a.toString());
        final ReadOnlyArray<String> l = newReadOnlyArray(asList("a", "b", "c"));
        assertEqual(asList("a", "b", "c").toString(), l.toString());
    }

    @Test
    public void equalsHashCodeArbitraryObject()
    {
        assertFalse(newReadOnlyArray(new String[0]).equals(new Object()));
        assertFalse(newReadOnlyArray(new LinkedList<String>()).equals(new Object()));
    }

    @Test
    public void equalsHashCode1()
    {
        final ObjectFactory<ReadOnlyArray<String>> factory = new ObjectFactory<ReadOnlyArray<String>>()
        {
            public ReadOnlyArray<String> newInstanceX()
            {
                return newReadOnlyArray("a", "b", "c");
            }

            public ReadOnlyArray<String> newInstanceY()
            {
                return newReadOnlyArray("a", "c", "b");
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode2()
    {
        final ObjectFactory<ReadOnlyArray<String>> factory = new ObjectFactory<ReadOnlyArray<String>>()
        {
            public ReadOnlyArray<String> newInstanceX()
            {
                return newReadOnlyArray("a", "c", "b");
            }

            public ReadOnlyArray<String> newInstanceY()
            {
                return newReadOnlyArray("a");
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode3()
    {
        final ObjectFactory<ReadOnlyArray<String>> factory = new ObjectFactory<ReadOnlyArray<String>>()
        {
            public ReadOnlyArray<String> newInstanceX()
            {
                return newReadOnlyArray("a");
            }

            public ReadOnlyArray<String> newInstanceY()
            {
                return newReadOnlyArray(new String[0]);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode4()
    {
        final ObjectFactory<ReadOnlyArray<String>> factory = new ObjectFactory<ReadOnlyArray<String>>()
        {
            public ReadOnlyArray<String> newInstanceX()
            {
                return newReadOnlyArray(new String[0]);
            }

            public ReadOnlyArray<String> newInstanceY()
            {
                return newReadOnlyArray("a", "b", "c", "d");
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode5()
    {
        final ObjectFactory<ReadOnlyArray<String>> factory = new ObjectFactory<ReadOnlyArray<String>>()
        {
            public ReadOnlyArray<String> newInstanceX()
            {
                return newReadOnlyArray(asList("a", "b", "c"));
            }

            public ReadOnlyArray<String> newInstanceY()
            {
                return newReadOnlyArray(asList("a", "c", "b"));
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode6()
    {
        final ObjectFactory<ReadOnlyArray<String>> factory = new ObjectFactory<ReadOnlyArray<String>>()
        {
            public ReadOnlyArray<String> newInstanceX()
            {
                return newReadOnlyArray(asList("a", "c", "b"));
            }

            public ReadOnlyArray<String> newInstanceY()
            {
                return newReadOnlyArray(asList("a"));
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode7()
    {
        final ObjectFactory<ReadOnlyArray<String>> factory = new ObjectFactory<ReadOnlyArray<String>>()
        {
            public ReadOnlyArray<String> newInstanceX()
            {
                return newReadOnlyArray(asList("a"));
            }

            public ReadOnlyArray<String> newInstanceY()
            {
                return newReadOnlyArray(new LinkedList<String>());
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode8()
    {
        final ObjectFactory<ReadOnlyArray<String>> factory = new ObjectFactory<ReadOnlyArray<String>>()
        {
            public ReadOnlyArray<String> newInstanceX()
            {
                return newReadOnlyArray(new LinkedList<String>());
            }

            public ReadOnlyArray<String> newInstanceY()
            {
                return newReadOnlyArray(asList("a", "b", "c", "d"));
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void testClone()
    {
        final ReadOnlyArray<String> a1 = newReadOnlyArray("a", "b", "c", "d");
        final ReadOnlyArray<String> b1 = a1.clone();
        assertNotSame(a1, b1);
        assertEqual(a1, b1);

        final ReadOnlyArray<String> a2 = newReadOnlyArray(asList("a", "b", "c", "d"));
        final ReadOnlyArray<String> b2 = a2.clone();
        assertNotSame(a2, b2);
        assertEqual(a2, b2);
    }

    @Test
    public void iterator1()
    {
        final ReadOnlyArray<String> a1 = newReadOnlyArray("a1", "b", "c");
        final Iterator<String> i1 = a1.iterator();
        assertTrue(i1.hasNext());
        assertEqual("a1", i1.next());
        assertTrue(i1.hasNext());
        assertEqual("b", i1.next());
        assertTrue(i1.hasNext());
        assertEqual("c", i1.next());
        assertFalse(i1.hasNext());

        final ReadOnlyArray<String> a2 = newReadOnlyArray(asList("a1", "b", "c"));
        final Iterator<String> i2 = a2.iterator();
        assertEqual("a1", i2.next());
        assertEqual("b", i2.next());
        assertEqual("c", i2.next());
    }

    @Test
    @ExpectException(NoSuchElementException.class)
    public void iterator2()
    {
        final ReadOnlyArray<String> a1 = newReadOnlyArray("a1", "b", "c");
        final Iterator<String> i1 = a1.iterator();
        assertTrue(i1.hasNext());
        assertEqual("a1", i1.next());
        assertTrue(i1.hasNext());
        assertEqual("b", i1.next());
        assertTrue(i1.hasNext());
        assertEqual("c", i1.next());
        assertFalse(i1.hasNext());
        i1.next();
    }

    @Test
    @ExpectException(UnsupportedOperationException.class)
    public void iterator3()
    {
        final ReadOnlyArray<String> a1 = newReadOnlyArray("a", "b", "c");
        final Iterator<String> i1 = a1.iterator();
        assertEqual("a", i1.next());
        assertEqual("b", i1.next());
        assertEqual("c", i1.next());
        i1.remove();
    }
}
