/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.framework.SequenceFactory.newSequence;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import org.jtiger.assertion.ObjectFactory;
import static org.jtiger.assertion.Reflection.assertHasDeclaredField;
import static org.jtiger.assertion.Serialization.assertSerializes;
import static org.jtiger.assertion.Reflection.assertHasDeclaredConstructor;
import static org.jtiger.assertion.Reflection.assertHasMethod;
import static org.jtiger.assertion.Modifier.assertPrivate;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertSame;
import static org.jtiger.assertion.ObjectFactoryContract.assertObjectFactoryFillsContract;
import static org.jtiger.assertion.EqualsMethodContract.assertEqualsMethodFillsContract;
import static org.jtiger.assertion.HashCodeMethodContract.assertHashCodeMethodFillsContract;
import static org.jtiger.framework.TestResultFactory.newTestResult;
import static org.jtiger.framework.TestResultType.SUCCESS;
import static org.jtiger.framework.TestResultType.FAILURE;

@Fixture(value = "TestResultFactory", description = "Performs Unit Tests on org.jtiger.framework.TestResultFactory")
@Category("org.jtiger.framework.TestResultFactory")
public final class TestTestResultFactory
{
    public TestTestResultFactory()
    {

    }

    @Test
    public void constructor() throws InstantiationException, IllegalAccessException, InvocationTargetException
    {
        final Constructor<?> c = assertHasDeclaredConstructor(TestResultFactory.class, null);
        assertPrivate(c);
        assertFalse(c.isAccessible());
        c.setAccessible(true);
        c.newInstance();
    }

    @Test
    @ExpectException(NullPointerException.class)
    public void factoryMethod1()
    {
        newTestResult(SUCCESS, "message", new Throwable(), 7L, null);
    }

    @Test
    public void factoryMethod2()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(SUCCESS, "message", new Throwable(), 7L, m);
        assertSame(SUCCESS, result.getTestResultType());
        assertEqual("message", result.getMessage());
        assertEqual(Throwable.class, result.getException().getClass());
        assertEqual(7L, result.getElapsedTime());
        assertEqual(newSequence(new String[]{"cat1", "cat2"}), result.getCategories());
        assertEqual("test name", result.getTestName());
        assertEqual("test description", result.getTestDescription());
        assertEqual("m", result.getTestMethodName());
        assertEqual(newSequence(new Object[0]), result.getTestMethodParameterTypes());
        assertEqual("fixture name", result.getFixtureName());
        assertEqual("fixture description", result.getFixtureDescription());
        assertSame(A.class, result.getFixtureClass());
    }

    @Test
    public void testResultType()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(FAILURE, null, null, null, m);
        assertSame(FAILURE, result.getTestResultType());
    }

    @Test
    public void message()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(null, "message", null, null, m);
        assertEqual("message", result.getMessage());
    }

    @Test
    public void exception()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final Throwable t = new Throwable();
        final TestResult result = newTestResult(null, null, t, null, m);
        assertSame(t, result.getException());
    }

    @Test
    public void elapsedTime()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(null, null, null, 7L, m);
        assertEqual(7L, result.getElapsedTime());
    }

    @Test
    public void categories()
    {
        final Method m = assertHasMethod(A.class, "categories", new Class<?>[]{});
        final TestResult result = newTestResult(null, null, null, null, m);
        assertEqual(newSequence(new String[]{"cat3", "cat1", "cat2"}), result.getCategories());
    }

    @Test
    public void testName()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(null, null, null, null, m);
        assertEqual("test name", result.getTestName());
    }

    @Test
    public void testDescription()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(null, null, null, null, m);
        assertEqual("test description", result.getTestDescription());
    }

    @Test
    public void testMethodName()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(null, null, null, null, m);
        assertEqual("m", result.getTestMethodName());
    }

    @Test
    public void testMethodParameterTypes()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(null, null, null, null, m);
        assertEqual(newSequence(new Object[0]), result.getTestMethodParameterTypes());
    }

    @Test
    public void fixtureName()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(null, null, null, null, m);
        assertEqual("fixture name", result.getFixtureName());
    }

    @Test
    public void fixtureDescription()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(null, null, null, null, m);
        assertEqual("fixture description", result.getFixtureDescription());
    }

    @Test
    public void fixtureClass()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(null, null, null, null, m);
        assertEqual(A.class, result.getFixtureClass());
    }

    @Test
    public void serialization()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(SUCCESS, "message", new Throwable(), 7L, m);

        assertSerializes(result);
        assertHasDeclaredField(result.getClass(), "serialVersionUID");
    }

    @Test
    public void testToString()
    {
        final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
        final TestResult result = newTestResult(SUCCESS, "message", new Throwable(), 7L, m);

        final String s = "[" + result.getTestResultType() + "][" +
                result.getMessage() + "][" +
                result.getException() + "][" +
                result.getElapsedTime() + "][" +
                result.getCategories() + "][" +
                result.getTestName() + "][" +
                result.getTestDescription() + "][" +
                result.getTestMethodName() + "][" +
                result.getFixtureName() + "][" +
                result.getFixtureDescription() + "][" +
                result.getFixtureClass() + "]";

        assertEqual(s, result.toString());
    }

    @Test
    public void equalsHashCode1()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(FAILURE, "message", t, 7L, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode2()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message x", t, 7L, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode3()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t1 = new Throwable();
            final Throwable t2 = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t1, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t2, 7L, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode4()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method categories = assertHasMethod(A.class, "categories", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, categories);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode5()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method testName = assertHasMethod(A.class, "testName", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, testName);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode6()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            private final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method testDescription = assertHasMethod(A.class, "testDescription", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, testDescription);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode7()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            private final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method testMethodName = assertHasMethod(A.class, "testMethodName", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, testMethodName);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }
    @Test
    public void equalsHashCode8()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(FixtureName.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode9()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(FixtureDescription.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode10()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(FixtureClass.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode11()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(null, "message", t, 7L, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode12()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, null, t, 7L, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode13()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", null, 7L, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode14()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            private final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, null, m);
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode15()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            private final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "categories", new Class<?>[]{});
                final TestResult result = newTestResult(SUCCESS, "message", t, 7L, m);
                return result;
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode16()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            private final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "categories", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "testName", new Class<?>[]{});
                final TestResult result = newTestResult(SUCCESS, "message", t, 7L, m);
                return result;
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode17()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "testName", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                final TestResult result = newTestResult(SUCCESS, "testDescription", t, 7L, m);
                return result;
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode18()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            private final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "testDescription", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(A.class, "m", new Class<?>[]{});
                final TestResult result = newTestResult(SUCCESS, "testMethodName", t, 7L, m);
                return result;
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode19()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(A.class, "testMethodName", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(FixtureName.class, "m", new Class<?>[]{});
                final TestResult result = newTestResult(SUCCESS, "message", t, 7L, m);
                return result;
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode20()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(FixtureName.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(FixtureDescription.class, "m", new Class<?>[]{});
                final TestResult result = newTestResult(SUCCESS, "message", t, 7L, m);
                return result;
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Test
    public void equalsHashCode21()
    {
        final ObjectFactory<TestResult> factory = new ObjectFactory<TestResult>()
        {
            private final Throwable t = new Throwable();

            public TestResult newInstanceX()
            {
                final Method m = assertHasMethod(FixtureDescription.class, "m", new Class<?>[]{});
                return newTestResult(SUCCESS, "message", t, 7L, m);
            }

            public TestResult newInstanceY()
            {
                final Method m = assertHasMethod(FixtureClass.class, "m", new Class<?>[]{});
                final TestResult result = newTestResult(SUCCESS, "message", t, 7L, m);
                return result;
            }
        };

        assertObjectFactoryFillsContract(factory);
        assertEqualsMethodFillsContract(factory);
        assertHashCodeMethodFillsContract(factory);
    }

    @Category({"cat1", "cat2"})
    @Fixture(value = "fixture name", description = "fixture description")
    private static final class A
    {
        @Test(value = "test name", description = "test description")
        public void m()
        {

        }

        @Category("cat3")
        @Test(value = "test name", description = "test description")
        public void categories()
        {

        }

        @Category("cat3")
        @Test(value = "test name x", description = "test description")
        public void testName()
        {

        }

        @Category("cat3")
        @Test(value = "test name", description = "test description x")
        public void testDescription()
        {

        }

        @Category("cat3")
        @Test(value = "test name", description = "test description")
        public void testMethodName()
        {

        }
    }

    @Category({"cat1", "cat2"})
    @Fixture(value = "fixture name x", description = "fixture description")
    private static final class FixtureName
    {
        @Test(value = "test name", description = "test description")
        public void m()
        {

        }
    }

    @Category({"cat1", "cat2"})
    @Fixture(value = "fixture name", description = "fixture description x")
    private static final class FixtureDescription
    {
        @Test(value = "test name", description = "test description")
        public void m()
        {

        }
    }

    @Category({"cat1", "cat2"})
    @Fixture(value = "fixture name", description = "fixture description")
    private static final class FixtureClass
    {
        @Test(value = "test name", description = "test description")
        public void m()
        {

        }
    }
}
