/*
 * JTiger Unit Testing Framework for J2SE 1.5
 * Copyright (C) 2005 Tony Morris
 *
 * This software is licenced under the
 * Common Public Licence version 1.0
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * You received a copy of this licence with this software.
 */
package org.jtiger.framework;

import static org.jtiger.framework.SequenceFactory.newSequence;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import org.jtiger.DummyFixture1;
import org.jtiger.DummyFixture2;
import org.jtiger.DummyFixture3;
import org.jtiger.DummyFixture4;
import static org.jtiger.assertion.Reflection.assertHasDeclaredConstructor;
import static org.jtiger.assertion.Modifier.assertPrivate;
import static org.jtiger.assertion.Basic.assertFalse;
import static org.jtiger.assertion.Basic.assertEqual;
import static org.jtiger.assertion.Basic.assertSame;
import static org.jtiger.framework.TestRunnerFactory.newTestRunner;
import static org.jtiger.framework.FixturesRunnerConfigFactory.newFixturesRunnerConfig;
import static org.jtiger.framework.FixturesRunnerFactory.newFixturesRunner;
import static org.jtiger.framework.TestResultType.SUCCESS;
import static org.jtiger.framework.TestResultType.FAILURE;
import static org.jtiger.framework.TestResultType.FAILURE_SETUP;
import static org.jtiger.framework.TestResultType.FAILURE_TEARDOWN;
import static org.jtiger.framework.TestResultType.IGNORED_ANNOTATED;
import static org.jtiger.framework.TestResultType.IGNORED_CANNOT_INVOKE;

@Fixture(value = "TestRunnerFactory", description = "Performs Unit Tests on org.jtiger.framework.TestRunnerFactory")
@Category("org.jtiger.framework.TestRunnerFactory")
public final class TestTestRunnerFactory
{
    public TestTestRunnerFactory()
    {

    }

    @Test
    public void constructor() throws InstantiationException, IllegalAccessException, InvocationTargetException
    {
        final Constructor<?> c = assertHasDeclaredConstructor(TestRunnerFactory.class, null);
        assertPrivate(c);
        assertFalse(c.isAccessible());
        c.setAccessible(true);
        c.newInstance();
    }

    @Test
    @ExpectException(NullPointerException.class)
    public void runNullArg1()
    {
        newTestRunner().run(null, null, null);
    }

    @Test
    @ExpectException(NullPointerException.class)
    public void runNullArg2() throws NoSuchMethodException
    {
        final Method m = getClass().getMethod(new Throwable().getStackTrace()[0].getMethodName());
        newTestRunner().run(m, null, null);
    }

    @Test
    @ExpectException(NullPointerException.class)
    public void runNullArg3() throws NoSuchMethodException
    {
        final Method m = getClass().getMethod(new Throwable().getStackTrace()[0].getMethodName());
        newTestRunner().run(m, "x", null);
    }

    @Test
    public void run() throws RunnerException
    {
        final FixturesRunner runner = newFixturesRunner();
        final FixturesRunnerConfig config = newFixturesRunnerConfig(
                newSequence(new Class<?>[]{DummyFixture1.class, DummyFixture2.class, DummyFixture3.class, DummyFixture4.class}), null, null, false);
        final FixtureResults results = runner.run(config);

        assertEqual(4, results.size());
        final FixtureResult result1 = results.get(0);
        final FixtureResult result2 = results.get(1);
        final FixtureResult result3 = results.get(2);
        final FixtureResult result4 = results.get(3);

        assertEqual(12, result1.size());
        assertEqual(1, result2.size());
        assertEqual(1, result3.size());
        assertEqual(1, result4.size());

        for(TestResult tr : result1)
        {
            if(tr.getTestMethodName().indexOf("success") != -1)
            {
                assertSame(SUCCESS, tr.getTestResultType());
            }
            else if(tr.getTestMethodName().indexOf("failure") != -1)
            {
                assertSame(FAILURE, tr.getTestResultType());
            }
            else if(tr.getTestMethodName().indexOf("ignoredAnnotated") != -1)
            {
                assertSame(IGNORED_ANNOTATED, tr.getTestResultType());
            }
            else if(tr.getTestMethodName().indexOf("ignoredCannotInvoke") != -1)
            {
                assertSame(IGNORED_CANNOT_INVOKE, tr.getTestResultType());
            }
            else if(tr.getTestMethodName().indexOf("message") != -1)
            {
                assertEqual("a message", tr.getMessage());
            }
            else if(tr.getTestMethodName().indexOf("repeat") != -1)
            {
                assertEqual("repeated", tr.getMessage());
            }
            else
            {
                throw new IllegalStateException();
            }
        }

        assertSame(IGNORED_ANNOTATED, result2.get(0).getTestResultType());
        assertSame(FAILURE_SETUP, result3.get(0).getTestResultType());
        assertSame(FAILURE_TEARDOWN, result4.get(0).getTestResultType());
    }
}
